@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - nibble, pancake */
+/* radare - LGPL - Copyright 2009-2018 - nibble, pancake */
 
 #include <getopt.c>
 #include <r_core.h>
@@ -25,7 +25,7 @@ static ut64 at = 0LL;
 static RLib *l;
 
 static int rabin_show_help(int v) {
-	printf ("Usage: rabin2 [-AcdeEghHiIjlLMqrRsSvVxzZ] [-@ at] [-a arch] [-b bits] [-B addr]\n"
+	printf ("Usage: rabin2 [-AcdeEghHiIjlLMqrRsSUvVxzZ] [-@ at] [-a arch] [-b bits] [-B addr]\n"
 		"              [-C F:C:D] [-f str] [-m addr] [-n str] [-N m:M] [-P[-P] pdb]\n"
 		"              [-o str] [-O str] [-k query] [-D lang symname] | file\n");
 	if (v) {
@@ -36,10 +36,12 @@ static int rabin_show_help(int v) {
 		" -b [bits]       set bits (32, 64 ...)\n"
 		" -B [addr]       override base address (pie bins)\n"
 		" -c              list classes\n"
+		" -cc             list classes in header format\n"
 		" -C [fmt:C:D]    create [elf,mach0,pe] with Code and Data hexpairs (see -a)\n"
 		" -d              show debug/dwarf information\n"
 		" -D lang name    demangle symbol name (-D all for bin.demangle=true)\n"
 		" -e              entrypoint\n"
+		" -ee             constructor/destructor entrypoints\n"
 		" -E              globally exportable symbols\n"
 		" -f [str]        select sub-bin named str\n"
 		" -F [binfmt]     force to use that bin plugin (ignore header check)\n"
@@ -71,6 +73,7 @@ static int rabin_show_help(int v) {
 		" -s              symbols\n"
 		" -S              sections\n"
 		" -u              unfiltered (no rename duplicated symbols/sections)\n"
+		" -U              resoUrces\n"
 		" -v              display version and quit\n"
 		" -V              Show binary version information\n"
 		" -x              extract bins contained in file\n"
@@ -89,8 +92,9 @@ static int rabin_show_help(int v) {
 		" RABIN2_MAXSTRBUF: e bin.maxstrbuf    # specify maximum buffer size\n"
 		" RABIN2_STRFILTER: e bin.strfilter    # r2 -qe bin.strfilter=? -c '' --\n"
 		" RABIN2_STRPURGE:  e bin.strpurge     # try to purge false positives\n"
-		" RABIN2_DEBASE64:  e bin.debase64     # Try to debase64 all strings\n"
+		" RABIN2_DEBASE64:  e bin.debase64     # try to debase64 all strings\n"
 		" RABIN2_DMNGLRCMD: e bin.demanglercmd # try to purge false positives\n"
+		" RABIN2_PDBSERVER: e pdb.server       # use alternative PDB server\n"
 		" RABIN2_PREFIX:    e bin.prefix       # prefix symbols/sections/relocs with a specific string\n");
 	}
 	return 1;
@@ -272,16 +276,19 @@ static int rabin_dump_symbols(int len) {
 		} else {
 			len = olen;
 		}
-		if (!(buf = malloc (len))) {
+		if (!(buf = calloc (1, len))) {
 			return false;
 		}
-		if (!(ret = malloc (len * 2 + 1))) {
+		if (!(ret = malloc ((len * 2) + 1))) {
 			free (buf);
 			return false;
 		}
-		r_buf_read_at (bin->cur->buf, symbol->paddr, buf, len);
-		r_hex_bin2str (buf, len, ret);
-		printf ("%s %s\n", symbol->name, ret);
+		if (r_buf_read_at (bin->cur->buf, symbol->paddr, buf, len) == len) {
+			r_hex_bin2str (buf, len, ret);
+			printf ("%s %s\n", symbol->name, ret);
+		} else {
+			eprintf ("Cannot read from buffer\n");
+		}
 		free (buf);
 		free (ret);
 	}
@@ -479,26 +486,38 @@ static int rabin_show_srcline(ut64 at) {
 }
 
 /* bin callback */
-static int __lib_bin_cb(struct r_lib_plugin_t *pl, void *user, void *data) {
+static int __lib_bin_cb(RLibPlugin *pl, void *user, void *data) {
 	struct r_bin_plugin_t *hand = (struct r_bin_plugin_t *)data;
 	//printf(" * Added (dis)assembly plugin\n");
 	r_bin_add (bin, hand);
 	return true;
 }
 
-static int __lib_bin_dt(struct r_lib_plugin_t *pl, void *p, void *u) {
+static int __lib_bin_dt(RLibPlugin *pl, void *p, void *u) {
 	return true;
 }
 
 /* binxtr callback */
-static int __lib_bin_xtr_cb(struct r_lib_plugin_t *pl, void *user, void *data) {
+static int __lib_bin_xtr_cb(RLibPlugin *pl, void *user, void *data) {
 	struct r_bin_xtr_plugin_t *hand = (struct r_bin_xtr_plugin_t *)data;
 	//printf(" * Added (dis)assembly plugin\n");
 	r_bin_xtr_add (bin, hand);
 	return true;
 }
 
-static int __lib_bin_xtr_dt(struct r_lib_plugin_t *pl, void *p, void *u) {
+static int __lib_bin_xtr_dt(RLibPlugin *pl, void *p, void *u) {
+	return true;
+}
+
+/* binldr callback */
+static int __lib_bin_ldr_cb(RLibPlugin *pl, void *user, void *data) {
+	struct r_bin_ldr_plugin_t *hand = (struct r_bin_ldr_plugin_t *)data;
+	//printf(" * Added (dis)assembly plugin\n");
+	r_bin_ldr_add (bin, hand);
+	return true;
+}
+
+static int __lib_bin_ldr_dt(RLibPlugin *pl, void *p, void *u) {
 	return true;
 }
 
@@ -545,7 +564,6 @@ int main(int argc, char **argv) {
 	const char *op = NULL;
 	const char *path = NULL;
 	RCoreBinFilter filter;
-	RCoreFile *cf = NULL;
 	int xtr_idx = 0; // load all files if extraction is necessary.
 	int rawstr = 0;
 	int fd = -1;
@@ -557,11 +575,12 @@ int main(int argc, char **argv) {
 	if (!(tmp = r_sys_getenv ("RABIN2_NOPLUGINS"))) {
 		char *homeplugindir = r_str_home (R2_HOMEDIR "/plugins");
 		l = r_lib_new ("radare_plugin");
-	
 		r_lib_add_handler (l, R_LIB_TYPE_BIN, "bin plugins",
-		  &__lib_bin_cb, &__lib_bin_dt, NULL);
+			&__lib_bin_cb, &__lib_bin_dt, NULL);
 		r_lib_add_handler (l, R_LIB_TYPE_BIN_XTR, "bin xtr plugins",
-		  &__lib_bin_xtr_cb, &__lib_bin_xtr_dt, NULL);
+			&__lib_bin_xtr_cb, &__lib_bin_xtr_dt, NULL);
+		r_lib_add_handler (l, R_LIB_TYPE_BIN_LDR, "bin ldr plugins",
+			&__lib_bin_ldr_cb, &__lib_bin_ldr_dt, NULL);
 		/* load plugins everywhere */
 
 		path = r_sys_getenv (R_LIB_ENV);
@@ -604,11 +623,15 @@ int main(int argc, char **argv) {
 		r_config_set (core.config, "bin.debase64", tmp);
 		free (tmp);
 	}
+	if ((tmp = r_sys_getenv ("RABIN2_PDBSERVER"))) {
+		r_config_set (core.config, "pdb.server", tmp);
+		free (tmp);
+	}
 
 #define is_active(x) (action & x)
-#define set_action(x) actions++; action |= x
+#define set_action(x) { actions++; action |= x; }
 #define unset_action(x) action &= ~x
-	while ((c = getopt (argc, argv, "DjgAf:F:a:B:G:b:cC:k:K:dD:Mm:n:N:@:isSVIHeElRwO:o:pPqQrvLhuxXzZ")) != -1) {
+	while ((c = getopt (argc, argv, "DjgAf:F:a:B:G:b:cC:k:K:dD:Mm:n:N:@:isSVIHeEUlRwO:o:pPqQrvLhuxXzZ")) != -1) {
 		switch (c) {
 		case 'g':
 			set_action (R_BIN_REQ_CLASSES);
@@ -641,7 +664,13 @@ int main(int argc, char **argv) {
 		case 'u': bin->filter = 0; break;
 		case 'k': query = optarg; break;
 		case 'K': chksum = optarg; break;
-		case 'c': set_action (R_BIN_REQ_CLASSES); break;
+		case 'c': 
+			if (is_active (R_BIN_REQ_CLASSES)) {
+				rad = R_CORE_BIN_CLASSDUMP;
+			} else {
+			  	set_action (R_BIN_REQ_CLASSES); 
+			}
+			break;
 		case 'f': arch_name = strdup (optarg); break;
 		case 'F': forcebin = optarg; break;
 		case 'b': bits = r_num_math (NULL, optarg); break;
@@ -659,7 +688,7 @@ int main(int argc, char **argv) {
 					/* to store them just dump'm all to stdout */
 					rawstr = 2;
 				} else {
-					rawstr = true;
+					rawstr = 1;
 				}
 			} else {
 				set_action (R_BIN_REQ_STRINGS);
@@ -688,8 +717,16 @@ int main(int argc, char **argv) {
 				do_demangle = argv[optind];
 			}
 			break;
-		case 'e': set_action (R_BIN_REQ_ENTRIES); break;
+		case 'e':
+			if (action & R_BIN_REQ_ENTRIES) {
+				action &= ~R_BIN_REQ_ENTRIES;
+				action |= R_BIN_REQ_INITFINI;
+			} else {
+				set_action (R_BIN_REQ_ENTRIES);
+			}
+			break;
 		case 'E': set_action (R_BIN_REQ_EXPORTS); break;
+		case 'U': set_action (R_BIN_REQ_RESOURCES); break;
 		case 'Q': set_action (R_BIN_REQ_DLOPEN); break;
 		case 'M': set_action (R_BIN_REQ_MAIN); break;
 		case 'l': set_action (R_BIN_REQ_LIBS); break;
@@ -725,6 +762,7 @@ int main(int argc, char **argv) {
 		case 'v': return blob_version ("rabin2");
 		case 'L':
 			set_action (R_BIN_REQ_LISTPLUGINS);
+			break;
 		case 'G':
 			laddr = r_num_math (NULL, optarg);
 			if (laddr == UT64_MAX) {
@@ -923,12 +961,13 @@ int main(int argc, char **argv) {
 	}
 
 	if (file && *file) {
-		cf = r_core_file_open (&core, file, R_IO_READ, 0);
-		fd = cf ? r_core_file_cur_fd (&core) : -1;
-		if (!cf || fd == -1) {
-			eprintf ("r_core: Cannot open file '%s'\n", file);
-			r_core_fini (&core);
-			return 1;
+		if (r_core_file_open (&core, file, R_IO_READ, 0)) {
+			fd = r_io_fd_get_current (core.io);
+			if (fd == -1) {
+				eprintf ("r_core: Cannot open file '%s'\n", file);
+				r_core_fini (&core);
+				return 1;
+			}
 		}
 	}
 	bin->minstrlen = r_config_get_i (core.config, "bin.minstr");
@@ -953,7 +992,11 @@ int main(int argc, char **argv) {
 	}
 	if (rawstr == 2) {
 		rawstr = false;
-		r_bin_dump_strings (core.bin->cur, bin->minstrlen);
+		RBinFile *bf = r_core_bin_cur (&core);
+		if (bf) {
+			bf->strmode = rad;
+			r_bin_dump_strings (bf, bin->minstrlen);
+		}
 	}
 	if (query) {
 		if (rad) {
@@ -1016,11 +1059,13 @@ int main(int argc, char **argv) {
 
 	run_action ("sections", R_BIN_REQ_SECTIONS, R_CORE_BIN_ACC_SECTIONS);
 	run_action ("entries", R_BIN_REQ_ENTRIES, R_CORE_BIN_ACC_ENTRIES);
+	run_action ("entries", R_BIN_REQ_INITFINI, R_CORE_BIN_ACC_INITFINI);
 	run_action ("main", R_BIN_REQ_MAIN, R_CORE_BIN_ACC_MAIN);
 	run_action ("imports", R_BIN_REQ_IMPORTS, R_CORE_BIN_ACC_IMPORTS);
 	run_action ("classes", R_BIN_REQ_CLASSES, R_CORE_BIN_ACC_CLASSES);
 	run_action ("symbols", R_BIN_REQ_SYMBOLS, R_CORE_BIN_ACC_SYMBOLS);
 	run_action ("exports", R_BIN_REQ_EXPORTS, R_CORE_BIN_ACC_EXPORTS);
+	run_action ("resources", R_BIN_REQ_RESOURCES, R_CORE_BIN_ACC_RESOURCES);
 	run_action ("strings", R_BIN_REQ_STRINGS, R_CORE_BIN_ACC_STRINGS);
 	run_action ("info", R_BIN_REQ_INFO, R_CORE_BIN_ACC_INFO);
 	run_action ("fields", R_BIN_REQ_FIELDS, R_CORE_BIN_ACC_FIELDS);
