@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake */
+/* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #define USE_THREADS 1
 #define UNCOLORIZE_NONTTY 0
@@ -22,6 +22,21 @@ static char *rabin_cmd = NULL;
 static bool threaded = false;
 static bool haveRarunProfile = false;
 static struct r_core_t r;
+static int do_analysis = 0;
+
+static bool is_valid_gdb_file(RCoreFile *fh) {
+	RIODesc *d = fh && fh->core ? r_io_desc_get (fh->core->io, fh->fd) : NULL;
+	return d && strncmp (d->name, "gdb://", 6);
+}
+
+static char* get_file_in_cur_dir(const char *filepath) {
+	filepath = r_file_basename (filepath);
+	if (r_file_exists (filepath)
+	    && !r_file_is_directory (filepath)) {
+		return r_file_abspath (filepath);
+	}
+	return NULL;
+}
 
 static int verify_version(int show) {
 	int i, ret;
@@ -91,6 +106,7 @@ static int main_help(int line) {
 		" =            read file from stdin (use -i and -c to run cmds)\n"
 		" -=           perform !=! command to run all commands remotely\n"
 		" -0           print \\x00 after init and every command\n"
+		" -2           close stderr file descriptor (silent warning messages)\n"
 		" -a [arch]    set asm.arch\n"
 		" -A           run 'aaa' command to analyze all referenced code\n"
 		" -b [bits]    set asm.bits\n"
@@ -106,18 +122,19 @@ static int main_help(int line) {
 		" -H ([var])   display variable\n"
 		" -i [file]    run script file\n"
 		" -I [file]    run script file before the file is opened\n"
-		" -k [k=v]     perform sdb query into core->sdb\n"
+		" -k [OS/kern] set asm.os (linux, macos, w32, netbsd, ...)\n"
 		" -l [lib]     load plugin file\n"
 		" -L           list supported IO plugins\n"
 		" -m [addr]    map file at given address (loadaddr)\n"
 		" -M           do not demangle symbol names\n"
 		" -n, -nn      do not load RBin info (-nn only load bin structures)\n"
 		" -N           do not load user settings and scripts\n"
-		" -o [OS/kern] set asm.os (linux, macos, w32, netbsd, ...)\n"
 		" -q           quiet mode (no prompt) and quit after -i\n"
+		" -Q           quiet mode (no prompt) and quit faster (quickLeak=true)\n"
 		" -p [prj]     use project, list if no arg, load if no file\n"
 		" -P [file]    apply rapatch file and quit\n"
-		" -R [rarun2]  specify rarun2 profile to load (same as -e dbg.profile=X)\n"
+		" -r [rarun2]  specify rarun2 profile to load (same as -e dbg.profile=X)\n"
+		" -R [rr2rule] specify custom rarun2 directive\n"
 		" -s [addr]    initial seek\n"
 		" -S           start r2 in sandbox mode\n"
 #if USE_THREADS
@@ -126,19 +143,20 @@ static int main_help(int line) {
 		" -u           set bin.filter=false to get raw sym/sec/cls names\n"
 		" -v, -V       show radare2 version (-V show lib versions)\n"
 		" -w           open file in write mode\n"
-		" -X [rr2rule] specify custom rarun2 directive\n"
+		" -x           open without exec-flag (asm.emu will not work), See io.exec\n"
+		" -X           same as -e bin.usextr=false (useful for dyldcache)\n"
 		" -z, -zz      do not load strings or load them even in raw\n");
 	}
 	if (line == 2) {
 		char *homedir = r_str_home (R2_HOMEDIR);
 		printf (
 		"Scripts:\n"
-		" system   "R2_PREFIX"/share/radare2/radare2rc\n"
-		" user     ~/.radare2rc ${RHOMEDIR}/radare2/radare2rc (and radare2rc.d/)\n"
-		" file     ${filename}.r2\n"
+		" system       ${R2_PREFIX}/share/radare2/radare2rc\n"
+		" user         ~/.radare2rc ${RHOMEDIR}/radare2/radare2rc (and radare2rc.d/)\n"
+		" file         ${filename}.r2\n"
 		"Plugins:\n"
-		" binrc    ~/.config/radare2/rc.d/bin-<format>/ (elf, elf64, mach0, ..)\n"
-		" plugins  "R2_PREFIX"/lib/radare2/last\n"
+		" binrc        ~/.config/radare2/rc.d/bin-<format>/ (elf, elf64, mach0, ..)\n"
+		" plugins      "R2_PREFIX"/lib/radare2/last\n"
 		" USER_PLUGINS ~/.config/radare2/plugins\n"
 		" LIBR_PLUGINS "R2_PREFIX"/lib/radare2/"R2_VERSION"\n"
 		" USER_ZIGNS   ~/.config/radare2/zigns\n"
@@ -150,10 +168,10 @@ static int main_help(int line) {
 		" VAPIDIR      path to extra vapi directory\n"
 		" R2_NOPLUGINS do not load r2 shared plugins\n"
 		"Paths:\n"
-		" PREFIX       "R2_PREFIX"\n"
-		" INCDIR       "R2_INCDIR"\n"
-		" LIBDIR       "R2_LIBDIR"\n"
-		" LIBEXT       "R_LIB_EXT"\n"
+		" R2_PREFIX    "R2_PREFIX"\n"
+		" R2_INCDIR    "R2_INCDIR"\n"
+		" R2_LIBDIR    "R2_LIBDIR"\n"
+		" R2_LIBEXT    "R_LIB_EXT"\n"
 		, homedir);
 		free (homedir);
 	}
@@ -205,7 +223,8 @@ static int main_print_var(const char *var_name) {
 // TODO: use thread to load this, split contents line, per line and use global lock
 #if USE_THREADS
 static int rabin_delegate(RThread *th) {
-	if (rabin_cmd && r_file_exists (r.file->desc->name)) {
+	RIODesc *d = r_io_desc_get (r.io, r.file->fd);
+	if (rabin_cmd && r_file_exists (d->name)) {
 		char *nptr, *ptr, *cmd = r_sys_cmd_str (rabin_cmd, NULL, NULL);
 		ptr = cmd;
 		if (ptr) {
@@ -230,7 +249,9 @@ static int rabin_delegate(RThread *th) {
 		r_str_free (rabin_cmd);
 		rabin_cmd = NULL;
 	}
-	if (th) eprintf ("rabin2: done\n");
+	if (th) {
+		eprintf ("rabin2: done\n");
+	}
 	return 0;
 }
 #endif
@@ -242,7 +263,6 @@ static void radare2_rc(RCore *r) {
 		has_debug = true;
 		R_FREE (env_debug);
 	}
-
 	char *homerc = r_str_home (".radare2rc");
 	if (homerc && r_file_is_regular (homerc)) {
 		if (has_debug) {
@@ -296,7 +316,7 @@ static bool run_commands(RList *cmds, RList *files, bool quiet) {
 		}
 		ret = r_core_run_script (&r, file);
 		if (ret == -2) {
-			eprintf ("Cannot open '%s'\n", file);
+			eprintf ("[c] Cannot open '%s'\n", file);
 		}
 		if (ret < 0 || (ret == 0 && quiet)) {
 			r_cons_flush ();
@@ -309,6 +329,9 @@ static bool run_commands(RList *cmds, RList *files, bool quiet) {
 		r_cons_flush ();
 	}
 	if (quiet) {
+		if (do_analysis) {
+			return true;
+		}
 		if (cmds && !r_list_empty (cmds)) {
 			return true;
 		}
@@ -379,11 +402,11 @@ int main(int argc, char **argv, char **envp) {
 	RListIter *iter;
 	char *cmdn, *tmp;
 	RCoreFile *fh = NULL;
+	RIODesc *iod = NULL;
 	const char *patchfile = NULL;
 	const char *prj = NULL;
 	int debug = 0;
 	int zflag = 0;
-	int do_analysis = 0;
 	int do_connect = 0;
 	bool fullfile = false;
 	int has_project;
@@ -392,7 +415,7 @@ int main(int argc, char **argv, char **envp) {
 	int help = 0;
 	int run_anal = 1;
 	int run_rc = 1;
- 	int ret, c, perms = R_IO_READ;
+ 	int ret, c, perms = R_IO_READ | R_IO_EXEC;
 	bool sandbox = false;
 	ut64 baddr = UT64_MAX;
 	ut64 seek = UT64_MAX;
@@ -413,25 +436,33 @@ int main(int argc, char **argv, char **envp) {
 	RList *evals = r_list_new ();
 	RList *files = r_list_new ();
 	RList *prefiles = r_list_new ();
+
+#define LISTS_FREE() \
+		{ \
+			r_list_free (cmds); \
+			r_list_free (evals); \
+			r_list_free (files); \
+			r_list_free (prefiles); \
+		}
+
 	int va = 1; // set va = 0 to load physical offsets from rbin
 	bool noStderr = false;
 
 	r_sys_set_environ (envp);
 
-	if (r_sys_getenv ("R_DEBUG")) {
+	if ((tmp = r_sys_getenv ("R_DEBUG"))) {
 		r_sys_crash_handler ("gdb --pid %d");
+		free (tmp);
 	}
 	if (argc < 2) {
-		r_list_free (cmds);
-		r_list_free (evals);
-		r_list_free (files);
-		r_list_free (prefiles);
+		LISTS_FREE ();
 		return main_help (1);
 	}
 	r_core_init (&r);
 	if (argc == 2 && !strcmp (argv[1], "-p")) {
 		r_core_project_list (&r, 0);
 		r_cons_flush ();
+		LISTS_FREE ();
 		return 0;
 	}
 	// HACK TO PERMIT '#!/usr/bin/r2 - -i' hashbangs
@@ -447,10 +478,11 @@ int main(int argc, char **argv, char **envp) {
 	// -H option without argument
 	if (argc == 2 && !strcmp (argv[1], "-H")) {
 		main_print_var (NULL);
+		LISTS_FREE ();
 		return 0;
 	}
 
-	while ((c = getopt (argc, argv, "=02AMCwfF:H:hm:e:nk:NdqQs:p:b:B:a:Lui:I:l:P:R:c:D:vVSzuX:"
+	while ((c = getopt (argc, argv, "=02AMCwxfF:H:hm:e:nk:NdqQs:p:b:B:a:Lui:I:l:P:R:r:c:D:vVSzuX"
 #if USE_THREADS
 "t"
 #endif
@@ -477,17 +509,23 @@ int main(int argc, char **argv, char **envp) {
 		case 'a':
 			asmarch = optarg;
 			break;
-		case 'z': zflag++; break;
+		case 'z':
+			zflag++;
+			break;
 		case 'A':
-			if (!do_analysis) do_analysis ++;
-			do_analysis++;
+			do_analysis += do_analysis ? 1: 2;
 			break;
 		case 'b': asmbits = optarg; break;
 		case 'B':
 			baddr = r_num_math (r.num, optarg);
 			va = 2;
 			break;
-		case 'c': r_list_append (cmds, optarg); break;
+		case 'X':
+			r_config_set (r.config, "bin.usextr", "false");
+			break;
+		case 'c':
+			r_list_append (cmds, optarg);
+			break;
 		case 'C':
 			do_connect = true;
 			break;
@@ -502,6 +540,7 @@ int main(int argc, char **argv, char **envp) {
 			if (!strcmp (optarg, "?")) {
 				r_debug_plugin_list (r.dbg, 'q');
 				r_cons_flush();
+				LISTS_FREE ();
 				return 0;
 			}
 			break;
@@ -524,6 +563,7 @@ int main(int argc, char **argv, char **envp) {
 			break;
 		case 'H':
 			main_print_var (optarg);
+			LISTS_FREE ();
 			return 0;
 		case 'i':
 			r_list_append (files, optarg);
@@ -559,9 +599,13 @@ int main(int argc, char **argv, char **envp) {
 			if (!strcmp (optarg, "?")) {
 				r_core_project_list (&r, 0);
 				r_cons_flush ();
+				LISTS_FREE ();
 				return 0;
 			}
 			r_config_set (r.config, "prj.name", optarg);
+			// FIXME: Doing this here will overwrite -e flags coming before -p on the cmdline.
+			r_core_project_open (&r, r_config_get (r.config, "prj.name"), threaded);
+			r_config_set (r.config, "bin.strings", "false");
 			break;
 		case 'P':
 			patchfile = optarg;
@@ -576,10 +620,13 @@ int main(int argc, char **argv, char **envp) {
 			r_config_set (r.config, "cfg.fortunes", "false");
 			quiet = true;
 			break;
-		case 'R':
+		case 'r':
 			haveRarunProfile = true;
 			r_config_set (r.config, "dbg.profile", optarg);
 			break;
+		case 'R':
+			customRarunProfile = r_str_appendf (customRarunProfile, "%s\n", optarg);
+			break;
 		case 's':
 			s_seek = optarg;
 			break;
@@ -594,25 +641,51 @@ int main(int argc, char **argv, char **envp) {
 		case 'v':
 			if (quiet) {
 				printf ("%s\n", R2_VERSION);
+				LISTS_FREE ();
 				return 0;
 			} else {
 				verify_version (0);
+				LISTS_FREE ();
 				return blob_version ("radare2");
 			}
 		case 'V':
 			return verify_version (1);
 		case 'w':
-			perms = R_IO_READ | R_IO_WRITE;
+			perms |= R_IO_WRITE;
 			break;
-		case 'X':
-			customRarunProfile = r_str_appendf (customRarunProfile, "%s\n", optarg);
+		case 'x':
+			perms &= ~R_IO_EXEC;
+			r_config_set (r.config, "io.exec", "false");
 			break;
 		default:
 			help++;
 		}
 	}
 	if (noStderr) {
-		close (2);
+		if (-1 == close (2)) {
+			eprintf ("Failed to close stderr");
+			return 1;
+		}
+#if __WINDOWS__ && !__CYGWIN__
+		const char nul[] = "nul";
+#else
+		const char nul[] = "/dev/null";
+#endif
+		int new_stderr = open (nul, O_RDWR);
+		if (-1 == new_stderr) {
+			eprintf ("Failed to open %s", nul);
+			return 1;
+		}
+		if (2 != new_stderr) {
+			if (-1 == dup2 (new_stderr, 2)) {
+				eprintf ("Failed to dup2 stderr");
+				return 1;
+			}
+			if (-1 == close (new_stderr)) {
+				eprintf ("Failed to close %s", nul);
+				return 1;
+			}
+		}
 	}
 	{
 		const char *dbg_profile = r_config_get (r.config, "dbg.profile");
@@ -652,16 +725,12 @@ int main(int argc, char **argv, char **envp) {
 		}
 		r_io_plugin_list (r.io);
 		r_cons_flush ();
-		r_list_free (evals);
-		r_list_free (files);
-		r_list_free (cmds);
+		LISTS_FREE ();
 		return 0;
 	}
 
 	if (help > 0) {
-		r_list_free (evals);
-		r_list_free (files);
-		r_list_free (cmds);
+		LISTS_FREE ();
 		return main_help (help > 1? 2: 0);
 	}
 	if (customRarunProfile) {
@@ -682,13 +751,20 @@ int main(int argc, char **argv, char **envp) {
 		const char *src = haveRarunProfile? pfile: argv[optind];
 		if (src && *src) {
 			char *uri = strdup (src);
-			char *p = strstr (uri, "://");
-			if (p) {
-				*p = 0;
-				debugbackend = uri;
-				debug = 2;
-			} else {
-				free (uri);
+			if (uri) {
+				char *p = strstr (uri, "://");
+				if (p) {
+					*p = 0;
+					// TODO: this must be specified by the io plugin, not hardcoded here
+					if (!strcmp (uri, "winedbg")) {
+						debugbackend = "io";
+					} else {
+						debugbackend = uri;
+					}
+					debug = 2;
+				} else {
+					free (uri);
+				}
 			}
 		}
 	}
@@ -735,16 +811,31 @@ int main(int argc, char **argv, char **envp) {
 		break;
 	}
 
-	switch (va) {
-	case 0:
-		r_config_set_i (r.config, "io.va", false);
-		baddr = UT64_MAX;
-		break;
-	}
-
 	if (run_rc) {
 		radare2_rc (&r);
 	}
+
+	if (r_config_get_i (r.config, "zign.autoload")) {
+		char *path = r_file_abspath (r_config_get (r.config, "dir.zigns"));
+		char *complete_path = NULL;
+		RList *list = r_sys_dir (path);
+		RListIter *iter;
+		char *file = NULL;
+		r_list_foreach (list, iter, file) {
+			if (file && *file && *file != '.') {
+				complete_path = r_str_newf ("%s"R_SYS_DIR"%s", path, file);
+				if (r_str_endswith (complete_path, "gz")) {
+					r_sign_load_gz (r.anal, complete_path);
+				} else {
+					r_sign_load (r.anal, complete_path);
+				}
+				r_str_free (complete_path);
+			}
+		}
+		r_list_free (list);
+		free (path);
+	}
+
 	// if (argv[optind] && r_file_is_directory (argv[optind]))
 	if (pfile && r_file_is_directory (pfile)) {
 		if (debug) {
@@ -752,7 +843,7 @@ int main(int argc, char **argv, char **envp) {
 			return 1;
 		}
 		if (chdir (argv[optind])) {
-			eprintf ("Cannot open directory\n");
+			eprintf ("[d] Cannot open directory\n");
 			return 1;
 		}
 	} else if (argv[optind] && !strcmp (argv[optind], "=")) {
@@ -761,13 +852,13 @@ int main(int argc, char **argv, char **envp) {
 		ut8 *buf = (ut8 *)r_stdin_slurp (&sz);
 		close (0);
 		if (buf && sz > 0) {
-			char path[1024];
+			char path[128];
 			snprintf (path, sizeof (path) - 1, "malloc://%d", sz);
 			fh = r_core_file_open (&r, path, perms, mapaddr);
 			if (!fh) {
 				r_cons_flush ();
 				free (buf);
-				eprintf ("Cannot open %s\n", path);
+				eprintf ("[=] Cannot open '%s'\n", path);
 				return 1;
 			}
 			r_io_write_at (r.io, 0, buf, sz);
@@ -778,15 +869,14 @@ int main(int argc, char **argv, char **envp) {
 			eprintf ("Cannot slurp from stdin\n");
 			return 1;
 		}
-	} else if (strcmp (argv[optind - 1], "--")) {
+	} else if (strcmp (argv[optind - 1], "--") && !(r_config_get (r.config, "prj.name") && r_config_get (r.config, "prj.name")[0]) ) {
 		if (debug) {
 			if (asmbits) {
 				r_config_set (r.config, "asm.bits", asmbits);
 			}
 			r_config_set (r.config, "search.in", "dbg.map"); // implicit?
-			r_config_set_i (r.config, "io.va", false); // implicit?
 			r_config_set (r.config, "cfg.debug", "true");
-			perms = R_IO_READ | R_IO_WRITE;
+			perms = R_IO_READ | R_IO_WRITE | R_IO_EXEC;
 			if (optind >= argc) {
 				eprintf ("No program given to -d\n");
 				return 1;
@@ -798,18 +888,53 @@ int main(int argc, char **argv, char **envp) {
 					if (!haveRarunProfile) {
 						pfile = strdup (argv[optind++]);
 					}
-					perms = R_IO_READ; // XXX. should work with rw too
+					perms = R_IO_READ | R_IO_EXEC; // XXX. should work with rw too
 					debug = 2;
 					if (!strstr (pfile, "://")) {
 						optind--; // take filename
 					}
 					fh = r_core_file_open (&r, pfile, perms, mapaddr);
-/*
-					if (fh) {
-						r_core_bin_load (&r, pfile);
-						r_debug_use (r.dbg, debugbackend);
+					iod = (r.io && fh) ? r_io_desc_get (r.io, fh->fd) : NULL;
+					if (!strcmp (debugbackend, "gdb")) {
+						const char *filepath = r_config_get (r.config, "dbg.exe.path");
+						ut64 addr = baddr;
+						if (addr == UINT64_MAX) {
+							addr = r_config_get_i (r.config, "bin.baddr");
+						}
+						if (filepath && r_file_exists (filepath)
+						    && !r_file_is_directory (filepath)) {
+							char *newpath = r_file_abspath (filepath);
+							if (newpath) {
+								if (iod) {
+									free (iod->name);
+									iod->name = newpath;
+								}
+								if (addr == UINT64_MAX) {
+									addr = r_debug_get_baddr (r.dbg, newpath);
+								}
+								r_core_bin_load (&r, NULL, addr);
+							}
+						} else if (is_valid_gdb_file (fh)) {
+							filepath = iod->name;
+							if (r_file_exists (filepath)
+							    && !r_file_is_directory (filepath)) {
+								if (addr == UINT64_MAX) {
+									addr = r_debug_get_baddr (r.dbg, filepath);
+								}
+								r_core_bin_load (&r, filepath, addr);
+							} else if ((filepath = get_file_in_cur_dir (filepath))) {
+								// Present in local directory
+								if (iod) {
+									free (iod->name);
+									iod->name = (char*) filepath;
+								}
+								if (addr == UINT64_MAX) {
+									addr = r_debug_get_baddr (r.dbg, filepath);
+								}
+								r_core_bin_load (&r, NULL, addr);
+							}
+						}
 					}
-*/
 				}
 			} else {
 				const char *f = (haveRarunProfile && pfile)? pfile: argv[optind];
@@ -857,15 +982,22 @@ int main(int argc, char **argv, char **envp) {
 				pfile = file;
 			}
 		}
+		if (asmarch) {
+			r_config_set (r.config, "asm.arch", asmarch);
+		}
+		if (asmbits) {
+			r_config_set (r.config, "asm.bits", asmbits);
+		}
+		if (asmos) {
+			r_config_set (r.config, "asm.os", asmos);
+		}
 
 		if (!debug || debug == 2) {
 			const char *dbg_profile = r_config_get (r.config, "dbg.profile");
 			if (optind == argc && dbg_profile && *dbg_profile) {
 				fh = r_core_file_open (&r, pfile, perms, mapaddr);
 				if (fh) {
-					if (!r_core_bin_load (&r, pfile, baddr)) {
-						r_config_set_i (r.config, "io.va", false);
-					}
+					r_core_bin_load (&r, pfile, baddr);
 				}
 			}
 			if (optind < argc) {
@@ -881,6 +1013,10 @@ int main(int argc, char **argv, char **envp) {
 						}
 					}
 					if (fh) {
+						iod = r.io ? r_io_desc_get (r.io, fh->fd) : NULL;
+						if (perms & R_IO_EXEC) {
+							iod->flags |= R_IO_EXEC;
+						}
 						if (run_anal > 0) {
 #if USE_THREADS
 							if (!rabin_th)
@@ -892,21 +1028,21 @@ int main(int argc, char **argv, char **envp) {
 									filepath = file? strstr (file, "://"): NULL;
 									filepath = filepath ? filepath + 3 : pfile;
 								}
-								if (r.file && r.file->desc && r.file->desc->name)
-									filepath = r.file->desc->name;
-
+								if (r.file && iod && (iod->fd == r.file->fd) && iod->name) {
+									filepath = iod->name;
+								}
 								/* Load rbin info from r2 dbg:// or r2 /bin/ls */
 								/* the baddr should be set manually here */
-								if (!r_core_bin_load (&r, filepath, baddr)) {
-									r_config_set_i (r.config, "io.va", false);
-								}
+								(void)r_core_bin_load (&r, filepath, baddr);
 							}
 						} else {
+							r_io_map_new (r.io, iod->fd, perms, 0LL, mapaddr, r_io_desc_size (iod), true);
+							// r_io_map_new (r.io, iod->fd, iod->flags, 0LL, 0LL, r_io_desc_size (iod));
 							if (run_anal < 0) {
 								// PoC -- must move -rk functionalitiy into rcore
 								// this may be used with caution (r2 -nn $FILE)
 								r_core_cmdf (&r, "Sf");
-								r_core_cmdf (&r, ".!rabin2 -rk. \"%s\"", r.file->desc->name);
+								r_core_cmdf (&r, ".!rabin2 -rk. \"%s\"", iod->name);
 							}
 						}
 					}
@@ -916,16 +1052,29 @@ int main(int argc, char **argv, char **envp) {
 				if (prj && *prj) {
 					pfile = r_core_project_info (&r, prj);
 					if (pfile) {
-						fh = r_core_file_open (&r, pfile, perms, mapaddr);
+						if (!fh) {
+							fh = r_core_file_open (&r, pfile, perms, mapaddr);
+						}
 						// run_anal = 0;
 						run_anal = -1;
 					} else {
 						eprintf ("Cannot find project file\n");
 					}
+				} else {
+					if (fh) {
+						iod = r.io ? r_io_desc_get (r.io, fh->fd) : NULL;
+						if (iod) {
+							perms = iod->flags;
+							r_io_map_new (r.io, iod->fd, perms, 0LL, 0LL, r_io_desc_size (iod), true);
+						}
+					}
 				}
 			}
 		} else {
-			fh = r_core_file_open (&r, pfile, perms, mapaddr);
+			RCoreFile *f = r_core_file_open (&r, pfile, perms, mapaddr);
+			if (f) {
+				fh = f;
+			}
 			if (fh) {
 				r_debug_use (r.dbg, is_gdb ? "gdb" : debugbackend);
 			}
@@ -959,9 +1108,9 @@ int main(int argc, char **argv, char **envp) {
 			if (pfile && *pfile) {
 				r_cons_flush ();
 				if (perms & R_IO_WRITE) {
-					eprintf ("Cannot open '%s' for writing.\n", pfile);
+					eprintf ("[w] Cannot open '%s' for writing.\n", pfile);
 				} else {
-					eprintf ("Cannot open '%s'\n", pfile);
+					eprintf ("[r] Cannot open '%s'\n", pfile);
 				}
 			} else {
 				eprintf ("Missing file to open\n");
@@ -971,12 +1120,17 @@ int main(int argc, char **argv, char **envp) {
 		if (!r.file) { // no given file
 			return 1;
 		}
+		if (r.bin->cur && r.bin->cur->o && r.bin->cur->o->info && r.bin->cur->o->info->rclass && !strcmp ("fs", r.bin->cur->o->info->rclass)) {
+			const char *fstype = r.bin->cur->o->info->bclass;
+			r_core_cmdf (&r, "m /root %s @ 0", fstype);
+		}
+		iod = r.io ? r_io_desc_get (r.io, fh->fd) : NULL;
 #if USE_THREADS
 		if (run_anal > 0 && threaded) {
 			// XXX: if no rabin2 in path that may fail
 			// TODO: pass -B 0 ? for pie bins?
 			rabin_cmd = r_str_newf ("rabin2 -rSIeMzisR%s %s",
-					(debug || r.io->va) ? "" : "p", r.file->desc->name);
+					(debug || r.io->va) ? "" : "p", iod->name);
 			/* TODO: only load data if no project is used */
 			lock = r_th_lock_new (false);
 			rabin_th = r_th_new (&rabin_delegate, lock, 0);
@@ -986,7 +1140,6 @@ int main(int argc, char **argv, char **envp) {
 		if (mapaddr) {
 			r_core_seek (&r, mapaddr, 1);
 		}
-
 		r_list_foreach (evals, iter, cmdn) {
 			r_config_eval (r.config, cmdn);
 			r_cons_flush ();
@@ -1008,18 +1161,10 @@ int main(int argc, char **argv, char **envp) {
 			r_config_set (r.config, "asm.os", asmos);
 		}
 
-		(void)r_core_bin_update_arch_bits (&r);
-
-		debug = r.file && r.file->desc && r.file->desc->plugin && \
-			r.file->desc->plugin->isdbg;
+		debug = r.file && iod && (r.file->fd == iod->fd) && iod->plugin && \
+			iod->plugin->isdbg;
 		if (debug) {
-			if (baddr != UT64_MAX) {
-				//setup without attach again because there is dpa call
-				//producing two attach and it's annoying
-				r_core_setup_debugger (&r, debugbackend, false);
-			} else {
-				r_core_setup_debugger (&r, debugbackend, true);
-			}
+			r_core_setup_debugger (&r, debugbackend, baddr == UT64_MAX);
 		}
 		if (!debug && r_flag_get (r.flags, "entry0")) {
 			r_core_cmd0 (&r, "s entry0");
@@ -1032,21 +1177,22 @@ int main(int argc, char **argv, char **envp) {
 		}
 
 		if (fullfile) {
-			r_core_block_size (&r, r_io_desc_size (r.io, r.file->desc));
+			r_core_block_size (&r, r_io_desc_size (iod));
 		}
 
 		r_core_seek (&r, r.offset, 1); // read current block
 
 		/* check if file.sha1 has changed */
-		if (!strstr (r.file->desc->uri, "://")) {
+		if (!strstr (iod->uri, "://")) {
 			const char *npath, *nsha1;
 			char *path = strdup (r_config_get (r.config, "file.path"));
 			char *sha1 = strdup (r_config_get (r.config, "file.sha1"));
 			has_project = r_core_project_open (&r, r_config_get (r.config, "prj.name"), threaded);
+			iod = r.io ? r_io_desc_get (r.io, fh->fd) : NULL;
 			if (has_project) {
 				r_config_set (r.config, "bin.strings", "false");
 			}
-			if (r_core_hash_load (&r, r.file->desc->name) == false) {
+			if (r_core_hash_load (&r, iod->name) == false) {
 				//eprintf ("WARNING: File hash not calculated\n");
 			}
 			nsha1 = r_config_get (r.config, "file.sha1");
@@ -1065,7 +1211,6 @@ int main(int argc, char **argv, char **envp) {
 			r_config_eval (r.config, cmdn);
 			r_cons_flush ();
 		}
-		r_list_free (evals);
 
 		// no flagspace selected by default the beginning
 		r.flags->space_idx = -1;
@@ -1074,10 +1219,11 @@ int main(int argc, char **argv, char **envp) {
 			char f[128];
 			snprintf (f, sizeof (f), "%s.r2", pfile);
 			if (r_file_exists (f)) {
-				if (!quiet) {
-					eprintf ("NOTE: Loading '%s' script.\n", f);
+				// TODO: should 'q' unset the interactive bit?
+				bool isInteractive = r_config_get_i (r.config, "scr.interactive");
+				if (isInteractive && r_cons_yesno ('n', "Do you want to run the '%s' script? (y/N) ", f)) {
+					r_core_cmd_file (&r, f);
 				}
-				r_core_cmd_file (&r, f);
 			}
 		}
 	}
@@ -1115,10 +1261,14 @@ int main(int argc, char **argv, char **envp) {
 #endif
 #endif
 	if (fullfile) {
-		r_core_block_size (&r, r_io_desc_size (r.io, r.file->desc));
+		r_core_block_size (&r, r_io_desc_size (iod));
+	}
+	if (perms & R_IO_WRITE) {
+		r_core_cmd0 (&r, "omfg+w");
 	}
 	ret = run_commands (cmds, files, quiet);
 	r_list_free (cmds);
+	r_list_free (evals);
 	r_list_free (files);
 	if (ret) {
 		ret = 0;
@@ -1126,7 +1276,7 @@ int main(int argc, char **argv, char **envp) {
 	}
 	if (r_config_get_i (r.config, "scr.prompt")) {
 		if (run_rc && r_config_get_i (r.config, "cfg.fortunes")) {
-			r_core_print_fortune (&r);
+			r_core_fortune_print_random (&r);
 			r_cons_flush ();
 		}
 	}
@@ -1146,7 +1296,7 @@ int main(int argc, char **argv, char **envp) {
 			r_core_seek (&r, 0, 1);
 			free (data);
 		} else {
-			eprintf ("Cannot open '%s'\n", patchfile);
+			eprintf ("[p] Cannot open '%s'\n", patchfile);
 		}
 	}
 	if ((patchfile && !quiet) || !patchfile) {
@@ -1159,6 +1309,11 @@ int main(int argc, char **argv, char **envp) {
 
 		// no flagspace selected by default the beginning
 		r.flags->space_idx = -1;
+		if (!debug && r.bin && r.bin->cur && r.bin->cur->o && r.bin->cur->o->info) {
+			if (r.bin->cur->o->info->arch) {
+				r_core_cmd0 (&r, "aeip");
+			}
+		}
 		for (;;) {
 #if USE_THREADS
 			do {
@@ -1204,10 +1359,8 @@ int main(int argc, char **argv, char **envp) {
 							if (r_config_get_i (r.config, "dbg.exitkills") &&
 									r_cons_yesno ('y', "Do you want to kill the process? (Y/n)")) {
 								r_debug_kill (r.dbg, 0, false, 9); // KILL
-#if __WINDOWS__
 							} else {
 								r_debug_detach (r.dbg, r.dbg->pid);
-#endif
 							}
 						} else continue;
 					}
@@ -1243,7 +1396,6 @@ int main(int argc, char **argv, char **envp) {
 
 	/* capture return value */
 	ret = r.num->value;
-
 beach:
 	if (quietLeak) {
 		exit (ret);
@@ -1256,7 +1408,7 @@ beach:
 	r_core_fini (&r);
 	r_cons_set_raw (0);
 	free (file);
-	r_str_const_free ();
+	r_str_const_free (NULL);
 	r_cons_free ();
 	return ret;
 }
