@@ -3,6 +3,13 @@
 #include <r_diff.h>
 #include <r_core.h>
 #include <r_hash.h>
+#include <limits.h>
+#include <getopt.c>
+#ifdef _MSC_VER
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
+#endif
 
 #include "../blob/version.c"
 
@@ -11,6 +18,7 @@ enum {
 	MODE_DIFF_STRS,
 	MODE_DIFF_IMPORTS,
 	MODE_DIST,
+	MODE_DIST_MYERS,
 	MODE_DIST_LEVENSTEIN,
 	MODE_CODE,
 	MODE_GRAPH,
@@ -236,7 +244,7 @@ static int cb(RDiff *d, void *user, RDiffOp *op) {
 			}
 			printf ("\n");
 			if (core) {
-				int len = R_MAX(4, op->b_len);
+				int len = R_MAX (4, op->b_len);
 				RAsmCode *ac = r_asm_mdisassemble (core->assembler, op->b_buf, len);
 				if (quiet) {
 					char *bufasm = r_str_prefix_all (strdup (ac->buf_asm), "+ ");
@@ -244,7 +252,7 @@ static int cb(RDiff *d, void *user, RDiffOp *op) {
 					free (bufasm);
 				} else {
 					char *bufasm = r_str_prefix_all (strdup (ac->buf_asm), Color_GREEN"+ ");
-					printf ("%s\n"Color_RESET, bufasm);
+					printf ("%s\n" Color_RESET, bufasm);
 					free (bufasm);
 				}
 				// r_asm_code_free (ac);
@@ -261,13 +269,135 @@ static int cb(RDiff *d, void *user, RDiffOp *op) {
 	return 0;
 }
 
+static ut64 gdiff_start = 0;
+
+void print_bytes(const void *p, size_t len, bool big_endian) {
+	size_t i;
+	for (i = 0; i < len; ++i) {
+		ut8 ch = ((ut8*) p)[big_endian ? (len - i - 1) : i];
+		write (1, &ch, 1);
+	}
+}
+
+static int bcb(RDiff *d, void *user, RDiffOp *op) {
+	ut64 offset_diff = op->a_off - gdiff_start;
+	unsigned char opcode;
+	unsigned short USAddr = 0;
+	int IAddr = 0;
+	unsigned char UCLen = 0;
+	unsigned short USLen = 0;
+	int ILen = 0;
+	
+	// we copy from gdiff_start to a_off
+	if (offset_diff > 0) {
+
+		// size for the position
+		if (gdiff_start <= USHRT_MAX) {
+			opcode = 249;
+			USAddr = (unsigned short) gdiff_start;
+		} else if (gdiff_start <= INT_MAX) {
+			opcode = 252;
+			IAddr = (int) gdiff_start;
+		} else {
+			opcode = 255;
+		}
+
+		// size for the length
+		if (opcode != 255 && offset_diff <= UCHAR_MAX) {
+			UCLen = (unsigned char) offset_diff;
+		} else if (opcode != 255 && offset_diff <= USHRT_MAX) {
+			USLen = (unsigned short) offset_diff;
+			opcode += 1;
+		} else if (opcode != 255 && offset_diff <= INT_MAX) {
+			ILen = (int) offset_diff;
+			opcode += 2;
+		} else if (offset_diff > INT_MAX) {
+			int times = offset_diff / INT_MAX;
+			int max = INT_MAX;
+			size_t i;
+			for (i = 0; i < times; i++) {
+				print_bytes (&opcode, sizeof (opcode), true);
+				// XXX this is overflowingly wrong
+				// XXX print_bytes (&gdiff_start + i * max, sizeof (gdiff_start), true);
+				print_bytes (&max, sizeof (max), true);
+			}
+		}
+
+		// print opcode for COPY
+		print_bytes (&opcode, sizeof (opcode), true);
+
+		// print position for COPY
+		if (opcode <= 251) {
+			print_bytes (&USAddr, sizeof (USAddr), true);
+		} else if (opcode < 255) {
+			print_bytes (&IAddr, sizeof (IAddr), true);
+		} else {
+			print_bytes (&gdiff_start, sizeof (gdiff_start), true);
+		}
+		
+		// print length for COPY
+		switch (opcode) {
+		case 249:
+		case 252:
+			print_bytes (&UCLen, sizeof (UCLen), true);
+			break;
+		case 250:
+		case 253:
+			print_bytes (&USLen, sizeof (USLen), true);
+			break;
+		case 251:
+		case 254:
+		case 255:
+			print_bytes (&ILen, sizeof (ILen), true);
+			break;
+		}
+	}
+	
+	// we append data
+	if (op->b_len <= 246) {
+		ut8 data = op->b_len;
+		write (1, &data, 1);
+	} else if (op->b_len <= USHRT_MAX) {
+		USLen = (ut16) op->b_len;
+		ut8 data = 247;
+		write (1, &data, 1);
+		print_bytes (&USLen, sizeof (USLen), true);
+	} else if (op->b_len <= INT_MAX) {
+		ut8 data = 248;
+		write (1, &data, 1);
+		ILen = (int) op->b_len;
+		print_bytes (&ILen, sizeof (ILen), true);
+	} else {
+		// split into multiple DATA, because op->b_len is greater than INT_MAX
+		int times = op->b_len / INT_MAX;
+		int max = INT_MAX;
+		size_t i;
+		for(i = 0;i < times; i++) {
+			ut8 data = 248;
+			write (1, &data, 1);
+			print_bytes (&max, sizeof (max), true);
+			print_bytes (op->b_buf, max, false);
+			op->b_buf += max;
+		}
+		op->b_len = op->b_len % max;
+
+		// print the remaining size
+		int remain_size = op->b_len;
+		print_bytes(&remain_size, sizeof(remain_size), true);
+	}
+	print_bytes(op->b_buf, op->b_len, false);
+	gdiff_start = op->b_off + op->b_len;
+	return 0;
+}
+
 static int show_help(int v) {
-	printf ("Usage: radiff2 [-abcCdjrspOxuUvV] [-A[A]] [-g sym] [-t %%] [file] [file]\n");
+	printf ("Usage: radiff2 [-abBcCdjrspOxuUvV] [-A[A]] [-g sym] [-t %%] [file] [file]\n");
 	if (v) {
 		printf (
 			"  -a [arch]  specify architecture plugin to use (x86, arm, ..)\n"
 			"  -A [-A]    run aaa or aaaa after loading each binary (see -C)\n"
 			"  -b [bits]  specify register size for arch (16 (thumb), 32, 64, ..)\n"
+			"  -B         output in binary diff (GDIFF)\n"
 			"  -c         count of changes\n"
 			"  -C         graphdiff code (columns: off-A, match-ratio, off-B) (see -A)\n"
 			"  -d         use delta diffing\n"
@@ -282,8 +412,8 @@ static int show_help(int v) {
 			"  -p         use physical addressing (io.va=0)\n"
 			"  -q         quiet mode (disable colors, reduce output)\n"
 			"  -r         output in radare commands\n"
-			"  -s         compute text distance\n"
-			"  -ss        compute text distance (using levenstein algorithm)\n"
+			"  -s         compute edit distance (no substitution, Eugene W. Myers' O(ND) diff algorithm)\n"
+			"  -ss        compute Levenshtein edit distance (substitution is allowed, O(N^2))\n"
 			"  -S [name]  sort code diff (name, namelen, addr, size, type, dist) (only for -C or -g)\n"
 			"  -t [0-100] set threshold for code diff (default is 70%%)\n"
 			"  -x         show two column hexdump diffing\n"
@@ -418,7 +548,7 @@ static ut8 *slurp(RCore **c, const char *file, int *sz) {
 		size = r_io_size (io);
 		if (size > 0 && size < ST32_MAX) {
 			data = calloc (1, size);
-			if (r_io_read_at (io, 0, data, size) == size) {
+			if (r_io_read_at (io, 0, data, size)) {
 				if (sz) {
 					*sz = size;
 				}
@@ -429,7 +559,7 @@ static ut8 *slurp(RCore **c, const char *file, int *sz) {
 		} else {
 			eprintf ("slurp: Invalid file size\n");
 		}
-		r_io_close (io, d);
+		r_io_desc_close (d);
 		return data;
 	}
 	return (ut8 *) r_file_slurp (file, sz);
@@ -536,11 +666,11 @@ int main(int argc, char **argv) {
 	int mode = MODE_DIFF;
 	int diffops = 0;
 	int threshold = -1;
-	double sim;
+	double sim = 0.0;
 
 	evals = r_list_newf (NULL);
 
-	while ((o = getopt (argc, argv, "Aa:b:CDe:npg:G:OijrhcdsS:uUvVxt:zq")) != -1) {
+	while ((o = getopt (argc, argv, "Aa:b:BCDe:npg:G:OijrhcdsS:uUvVxt:zq")) != -1) {
 		switch (o) {
 		case 'a':
 			arch = optarg;
@@ -551,6 +681,9 @@ int main(int argc, char **argv) {
 		case 'b':
 			bits = atoi (optarg);
 			break;
+		case 'B':
+			diffmode = 'B';
+			break;
 		case 'e':
 			r_list_append (evals, optarg);
 			break;
@@ -603,6 +736,8 @@ int main(int argc, char **argv) {
 		case 's':
 			if (mode == MODE_DIST) {
 				mode = MODE_DIST_LEVENSTEIN;
+			} else if (mode == MODE_DIST_LEVENSTEIN) {
+				mode = MODE_DIST_MYERS;
 			} else {
 				mode = MODE_DIST;
 			}
@@ -658,7 +793,7 @@ int main(int argc, char **argv) {
 		if (!c || !c2) {
 			eprintf ("Cannot open '%s'\n", r_str_get (file2));
 			return 1;
-		}
+		}	
 		if (arch) {
 			r_config_set (c->config, "asm.arch", arch);
 			r_config_set (c2->config, "asm.arch", arch);
@@ -677,7 +812,7 @@ int main(int argc, char **argv) {
 		r_anal_diff_setup_i (c2->anal, diffops, threshold, threshold);
 		if (pdc) {
 			if (!addr) {
-				addr = "entry0";
+				//addr = "entry0";
 				addr = "main";
 			}
 			/* should be in mode not in bool pdc */
@@ -772,8 +907,16 @@ int main(int argc, char **argv) {
 			printf ("\"}],\n");
 			printf ("\"changes\":[");
 		}
+		if (diffmode == 'B') {
+			write (1, "\xd1\xff\xd1\xff", 4);
+			write (1, "\x04", 1);
+		}
 		if (diffmode == 'U') {
 			r_diff_buffers_unified (d, bufa, sza, bufb, szb);
+		} else if (diffmode == 'B') {
+			r_diff_set_callback (d, &bcb, 0);
+			r_diff_buffers (d, bufa, sza, bufb, szb);
+			write (1, "\x00", 1);
 		} else {
 			r_diff_set_callback (d, &cb, 0); // (void *)(size_t)diffmode);
 			r_diff_buffers (d, bufa, sza, bufb, szb);
@@ -784,14 +927,23 @@ int main(int argc, char **argv) {
 		r_diff_free (d);
 		break;
 	case MODE_DIST:
+	case MODE_DIST_MYERS:
 	case MODE_DIST_LEVENSTEIN:
-	{
-		RDiff *d = r_diff_new ();
-		d->verbose = verbose;
-		d->levenstein = (mode == MODE_DIST_LEVENSTEIN);
-		r_diff_buffers_distance (d, bufa, sza, bufb, szb, &count, &sim);
-		r_diff_free (d);
-	}
+		{
+			RDiff *d = r_diff_new ();
+			if (d) {
+				d->verbose = verbose;
+				if (mode == MODE_DIST_LEVENSTEIN) {
+					d->type = 'l';
+				} else if (mode == MODE_DIST_MYERS) {
+					d->type = 'm';
+				} else {
+					d->type = 0;
+				}
+				r_diff_buffers_distance (d, bufa, sza, bufb, szb, &count, &sim);
+				r_diff_free (d);
+			}
+		}
 		printf ("similarity: %.3f\n", sim);
 		printf ("distance: %d\n", count);
 		break;
