@@ -89,10 +89,12 @@ static void r_anal_ex_perform_revisit_bb_cb(RAnal *anal, RAnalState *state, ut64
 }
 
 R_API int r_anal_ex_bb_address_comparator(RAnalBlock *a, RAnalBlock *b){
-	if (a->addr == b->addr)
+	if (a->addr == b->addr) {
 		return 0;
-	else if (a->addr < b->addr)
+	}
+	if (a->addr < b->addr) {
 		return -1;
+	}
 	// a->addr > b->addr
 	return 1;
 }
@@ -100,7 +102,8 @@ R_API int r_anal_ex_bb_address_comparator(RAnalBlock *a, RAnalBlock *b){
 R_API int r_anal_ex_bb_head_comparator(RAnalBlock *a, RAnalBlock *b){
 	if (a->head == b->head) {
 		return 0;
-	} else if (a->head < b->head) {
+	}
+	if (a->head < b->head) {
 		return -1;
 	}
 	// a->head > b->head
@@ -111,7 +114,7 @@ R_API void r_anal_ex_clone_op_switch_to_bb (RAnalBlock *bb, RAnalOp *op) {
 	RListIter *iter;
 	RAnalCaseOp *caseop = NULL;
 
-	if (op->switch_op) {
+	if (bb && op && op->switch_op) {
 		bb->switch_op = r_anal_switch_op_new (op->switch_op->addr,
 				op->switch_op->min_val,
 				op->switch_op->max_val);
@@ -146,7 +149,6 @@ R_API RAnalOp * r_anal_ex_get_op(RAnal *anal, RAnalState *state, ut64 addr) {
 	}
 	state->current_op = current_op;
 	return current_op;
-
 }
 
 R_API RAnalBlock * r_anal_ex_get_bb(RAnal *anal, RAnalState *state, ut64 addr) {
@@ -175,12 +177,13 @@ R_API RAnalBlock * r_anal_ex_get_bb(RAnal *anal, RAnalState *state, ut64 addr) {
 		current_bb->op_bytes = malloc (current_bb->op_sz);
 		if (current_bb->op_bytes) {
 			int buf_len = r_anal_state_get_len (state, addr);
-			if (current_bb->op_sz < buf_len) {
-				eprintf ("Oops\n");
+			if (current_bb->op_sz > buf_len) {
 				r_anal_bb_free (current_bb);
 				return NULL;
 			}
-			memcpy (current_bb->op_bytes, r_anal_state_get_buf_by_addr(state, addr), current_bb->op_sz);
+			memcpy (current_bb->op_bytes,
+				r_anal_state_get_buf_by_addr (state, addr),
+				current_bb->op_sz);
 		}
 	}
 	state->current_bb = current_bb;
@@ -206,19 +209,18 @@ R_API void r_anal_ex_update_bb_cfg_head_tail( RAnalBlock *start, RAnalBlock * he
 			bb = bb->next;
 			bb->head = head;
 			bb->tail = tail;
-		}while (bb->next != NULL && !(bb->type & R_ANAL_BB_TYPE_TAIL));
+		} while (bb->next && !(bb->type & R_ANAL_BB_TYPE_TAIL));
 	}
 }
 
-R_API RList * r_anal_ex_perform_analysis(RAnal *anal, RAnalState *state, ut64 addr) {
-	if (anal->cur && anal->cur->analysis_algorithm) {
+R_API RList* r_anal_ex_perform_analysis(RAnal *anal, RAnalState *state, ut64 addr) {
+	if (anal && anal->cur && anal->cur->analysis_algorithm) {
 		return anal->cur->analysis_algorithm (anal, state, addr);
 	}
-
 	return r_anal_ex_analysis_driver (anal, state, addr);
 }
 
-R_API RList * r_anal_ex_analysis_driver(RAnal *anal, RAnalState *state, ut64 addr ) {
+R_API RList* r_anal_ex_analysis_driver(RAnal *anal, RAnalState *state, ut64 addr ) {
 	ut64 consumed_iter = 0;
 	ut64 bytes_consumed = 0, len = r_anal_state_get_len (state, addr);
 	RAnalBlock *pcurrent_bb   = state->current_bb,
@@ -293,13 +295,11 @@ R_API RList * r_anal_ex_analysis_driver(RAnal *anal, RAnalState *state, ut64 add
 		state->current_op = NULL;
 		state->current_bb = NULL;
 		if (!consumed_iter) {
-			eprintf("No bytes consumed, bailing!\n");
+			eprintf ("No bytes consumed, bailing!\n");
 			break;
 		}
 		consumed_iter = 0;
 	}
-
-
 	r_anal_op_free (state->current_op);
 	r_anal_ex_perform_post_anal (anal, state, addr);
 	state->current_op = pcurrent_op;
@@ -397,20 +397,23 @@ ut64 extract_code_op(ut64 ranal2_op_type) {
 
 
 ut64 extract_load_store_op(ut64 ranal2_op_type) {
-	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_PUSH) == R_ANAL_EX_LDST_OP_PUSH)
+	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_PUSH) == R_ANAL_EX_LDST_OP_PUSH) {
 		return R_ANAL_OP_TYPE_PUSH;
-	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_POP) == R_ANAL_EX_LDST_OP_POP )
+	}
+	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_POP) == R_ANAL_EX_LDST_OP_POP) {
 		return R_ANAL_OP_TYPE_POP;
-	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_MOV) == R_ANAL_EX_LDST_OP_MOV)
+	}
+	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_MOV) == R_ANAL_EX_LDST_OP_MOV) {
 		return R_ANAL_OP_TYPE_MOV;
-	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_EFF_ADDR) == R_ANAL_EX_LDST_OP_EFF_ADDR)
+	}
+	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_EFF_ADDR) == R_ANAL_EX_LDST_OP_EFF_ADDR) {
 		return R_ANAL_OP_TYPE_LEA;
+	}
 	return R_ANAL_OP_TYPE_UNK;
 }
 
 
 ut64 extract_unknown_op(ut64 ranal2_op_type) {
-
 	if ( (ranal2_op_type & R_ANAL_EX_CODEOP_JMP) == R_ANAL_EX_CODEOP_JMP )  return R_ANAL_OP_TYPE_UJMP;
 	if ( (ranal2_op_type & R_ANAL_EX_CODEOP_CALL) == R_ANAL_EX_CODEOP_CALL) return R_ANAL_OP_TYPE_UCALL;
 	if ( (ranal2_op_type & R_ANAL_EX_LDST_OP_PUSH) == R_ANAL_EX_LDST_OP_PUSH) return R_ANAL_OP_TYPE_UPUSH;
@@ -445,28 +448,27 @@ ut64 extract_bin_op(ut64 ranal2_op_type) {
 
 
 R_API ut64 r_anal_ex_map_anal_ex_to_anal_op_type (ut64 ranal2_op_type) {
-
 	switch (ranal2_op_type) {
-		case R_ANAL_EX_NULL_OP: return R_ANAL_OP_TYPE_NULL;
-		case R_ANAL_EX_NOP: return R_ANAL_OP_TYPE_NOP;
-		case R_ANAL_EX_ILL_OP: return R_ANAL_OP_TYPE_ILL;
-		default: break;
+	case R_ANAL_EX_NULL_OP: return R_ANAL_OP_TYPE_NULL;
+	case R_ANAL_EX_NOP: return R_ANAL_OP_TYPE_NOP;
+	case R_ANAL_EX_ILL_OP: return R_ANAL_OP_TYPE_ILL;
+	default: break;
 	}
-
-	if ( ranal2_op_type & R_ANAL_EX_UNK_OP)
+	if (ranal2_op_type & R_ANAL_EX_UNK_OP) {
 		return extract_unknown_op(ranal2_op_type);
-
-	if ( ranal2_op_type & R_ANAL_EX_CODE_OP)
+	}
+	if (ranal2_op_type & R_ANAL_EX_CODE_OP) {
 		return extract_code_op(ranal2_op_type);
-
-	if ( ranal2_op_type & R_ANAL_EX_REP_OP)
-		return R_ANAL_OP_TYPE_REP | r_anal_ex_map_anal_ex_to_anal_op_type ( ranal2_op_type & ~R_ANAL_EX_REP_OP );
-
-	if ( ranal2_op_type & (R_ANAL_EX_LOAD_OP | R_ANAL_EX_STORE_OP ))
+	}
+	if (ranal2_op_type & R_ANAL_EX_REP_OP) {
+		ut64 ret = r_anal_ex_map_anal_ex_to_anal_op_type (ranal2_op_type & ~R_ANAL_EX_REP_OP);
+		return R_ANAL_OP_TYPE_REP | ret;
+	}
+	if (ranal2_op_type & (R_ANAL_EX_LOAD_OP | R_ANAL_EX_STORE_OP)) {
 		return extract_load_store_op(ranal2_op_type);
-
-	if ( ranal2_op_type & R_ANAL_EX_BIN_OP)
+	}
+	if (ranal2_op_type & R_ANAL_EX_BIN_OP) {
 		return extract_bin_op(ranal2_op_type);
-
+	}
 	return R_ANAL_OP_TYPE_UNK;
 }
