@@ -48,6 +48,11 @@ static bool isnum(RAnalEsil *esil, const char *str, ut64 *num) {
 	return false;
 }
 
+static bool ispackedreg(RAnalEsil *esil, const char *str) {
+	RRegItem *ri = r_reg_get (esil->anal->reg, str, -1);
+	return ri? ri->packed_size > 0: false;
+}
+
 static bool isregornum(RAnalEsil *esil, const char *str, ut64 *num) {
 	if (!r_anal_esil_reg_read (esil, str, num, NULL)) {
 		if (!isnum (esil, str, num)) {
@@ -89,6 +94,7 @@ R_API RAnalEsil *r_anal_esil_new(int stacksize, int iotrap) {
 	esil->ops = sdb_new0 ();
 	esil->iotrap = iotrap;
 	esil->interrupts = sdb_new0 ();
+	esil->sessions = r_list_newf (r_anal_esil_session_free);
 	return esil;
 }
 
@@ -197,11 +203,14 @@ R_API void r_anal_esil_free(RAnalEsil *esil) {
 	esil->interrupts = NULL;
 	sdb_free (esil->stats);
 	esil->stats = NULL;
+	sdb_free (esil->db_trace);
+	esil->db_trace = NULL;
 	r_anal_esil_stack_free (esil);
 	free (esil->stack);
 	if (esil->anal && esil->anal->cur && esil->anal->cur->esil_fini) {
 		esil->anal->cur->esil_fini (esil);
 	}
+	r_list_free (esil->sessions);
 	free (esil->cmd_intr);
 	free (esil->cmd_trap);
 	free (esil->cmd_mdev);
@@ -211,18 +220,32 @@ R_API void r_anal_esil_free(RAnalEsil *esil) {
 }
 
 static ut8 esil_internal_sizeof_reg(RAnalEsil *esil, const char *r) {
-	RRegItem *ri;
 	if (!esil || !esil->anal || !esil->anal->reg || !r) {
 		return 0;
 	}
-	ri = r_reg_get (esil->anal->reg, r, -1);
+	RRegItem *ri = r_reg_get (esil->anal->reg, r, -1);
 	return ri? ri->size: 0;
 }
 
+static bool alignCheck(RAnalEsil *esil, ut64 addr) {
+	int dataAlign = r_anal_archinfo (esil->anal, R_ANAL_ARCHINFO_DATA_ALIGN);
+	if (dataAlign > 0) {
+		if (addr % dataAlign) {
+			return false;
+		}
+	}
+	return true;
+}
+
 static int internal_esil_mem_read(RAnalEsil *esil, ut64 addr, ut8 *buf, int len) {
 	if (!esil || !esil->anal || !esil->anal->iob.io) {
 		return 0;
 	}
+	if (!alignCheck (esil, addr)) {
+		esil->trap = R_ANAL_TRAP_READ_ERR;
+		esil->trap_code = addr;
+		return false;
+	}
 	if (esil->cmd_mdev && esil->mdev_range) {
 		if (r_str_range_in (esil->mdev_range, addr)) {
 			if (esil->cmd (esil, esil->cmd_mdev, addr, 0)) {
@@ -230,14 +253,42 @@ static int internal_esil_mem_read(RAnalEsil *esil, ut64 addr, ut8 *buf, int len)
 			}
 		}
 	}
-	return esil->anal->iob.read_at (esil->anal->iob.io, addr, buf, len);
+	//TODO: Check if error return from read_at.(on previous version of r2 this call always return len)
+	(void)esil->anal->iob.read_at (esil->anal->iob.io, addr, buf, len);
+	// check if request addres is mapped , if dont fire trap and esil ioer callback
+	// now with siol, read_at return true/false cant be used to check error vs len
+	if (!esil->anal->iob.is_valid_offset (esil->anal->iob.io, addr, false)) {
+		if (esil->iotrap) {
+			esil->trap = R_ANAL_TRAP_READ_ERR;
+			esil->trap_code = addr;
+		}
+		if (esil->cmd && esil->cmd_ioer && *esil->cmd_ioer) {
+			esil->cmd (esil, esil->cmd_ioer, esil->address, 0);
+		}
+	}
+	return len;
 }
 
 static int internal_esil_mem_read_no_null(RAnalEsil *esil, ut64 addr, ut8 *buf, int len) {
 	if (!esil || !esil->anal || !esil->anal->iob.io || !addr) {
 		return 0;
 	}
-	return esil->anal->iob.read_at (esil->anal->iob.io, addr, buf, len);
+	if (!alignCheck (esil, addr)) {
+		esil->trap = R_ANAL_TRAP_READ_ERR;
+		esil->trap_code = addr;
+		return false;
+	}
+	//TODO: Check if error return from read_at.(on previous version of r2 this call always return len)
+	(void)esil->anal->iob.read_at (esil->anal->iob.io, addr, buf, len);
+	// check if request addres is mapped , if dont fire trap and esil ioer callback
+	// now with siol, read_at return true/false cant be used to check error vs len
+	if (!esil->anal->iob.is_valid_offset (esil->anal->iob.io, addr, false)) {
+		if (esil->iotrap) {
+			esil->trap = R_ANAL_TRAP_READ_ERR;
+			esil->trap_code = addr;
+		}
+	}
+	return len;
 }
 
 R_API int r_anal_esil_mem_read(RAnalEsil *esil, ut64 addr, ut8 *buf, int len) {
@@ -248,6 +299,11 @@ R_API int r_anal_esil_mem_read(RAnalEsil *esil, ut64 addr, ut8 *buf, int len) {
 	if (esil->cb.hook_mem_read) {
 		ret = esil->cb.hook_mem_read (esil, addr, buf, len);
 	}
+	if (!alignCheck (esil, addr)) {
+		esil->trap = R_ANAL_TRAP_READ_ERR;
+		esil->trap_code = addr;
+		return false;
+	}
 	if (!ret && esil->cb.mem_read) {
 		ret = esil->cb.mem_read (esil, addr, buf, len);
 		if (ret != len) {
@@ -268,10 +324,15 @@ R_API int r_anal_esil_mem_read(RAnalEsil *esil, ut64 addr, ut8 *buf, int len) {
 }
 
 static int internal_esil_mem_write(RAnalEsil *esil, ut64 addr, const ut8 *buf, int len) {
-	int ret;
+	int ret = 0;
 	if (!esil || !esil->anal || !esil->anal->iob.io || esil->nowrite) {
 		return 0;
 	}
+	if (!alignCheck (esil, addr)) {
+		esil->trap = R_ANAL_TRAP_READ_ERR;
+		esil->trap_code = addr;
+		return false;
+	}
 	if (esil->cmd_mdev && esil->mdev_range) {
 		if (r_str_range_in (esil->mdev_range, addr)) {
 			if (esil->cmd (esil, esil->cmd_mdev, addr, 1)) {
@@ -279,8 +340,12 @@ static int internal_esil_mem_write(RAnalEsil *esil, ut64 addr, const ut8 *buf, i
 			}
 		}
 	}
-	ret = esil->anal->iob.write_at (esil->anal->iob.io, addr, buf, len);
-	if (ret != len) {
+	if (esil->anal->iob.write_at (esil->anal->iob.io, addr, buf, len)) {
+		ret = len;
+	}
+	// check if request addres is mapped , if dont fire trap and esil ioer callback
+	// now with siol, write_at return true/false cant be used to check error vs len
+	if (!esil->anal->iob.is_valid_offset (esil->anal->iob.io, addr, false)) {
 		if (esil->iotrap) {
 			esil->trap = R_ANAL_TRAP_WRITE_ERR;
 			esil->trap_code = addr;
@@ -293,15 +358,19 @@ static int internal_esil_mem_write(RAnalEsil *esil, ut64 addr, const ut8 *buf, i
 }
 
 static int internal_esil_mem_write_no_null(RAnalEsil *esil, ut64 addr, const ut8 *buf, int len) {
-	int ret;
+	int ret = 0;
 	if (!esil || !esil->anal || !esil->anal->iob.io || !addr) {
 		return 0;
 	}
 	if (esil->nowrite) {
 		return 0;
 	}
-	ret = esil->anal->iob.write_at (esil->anal->iob.io, addr, buf, len);
-	if (ret != len) {
+	if (esil->anal->iob.write_at (esil->anal->iob.io, addr, buf, len)) {
+		ret = len;
+	}
+	// check if request addres is mapped , if dont fire trap and esil ioer callback
+	// now with siol, write_at return true/false cant be used to check error vs len
+	if (!esil->anal->iob.is_valid_offset (esil->anal->iob.io, addr, false)) {
 		if (esil->iotrap) {
 			esil->trap = R_ANAL_TRAP_WRITE_ERR;
 			esil->trap_code = addr;
@@ -442,13 +511,17 @@ R_API int r_anal_esil_get_parm_type(RAnalEsil *esil, const char *str) {
 	if (str[0] == ESIL_INTERNAL_PREFIX && str[1]) {
 		return R_ANAL_ESIL_PARM_INTERNAL;
 	}
-	if (!strncmp (str, "0x", 2))
+	if (!strncmp (str, "0x", 2)) {
 		return R_ANAL_ESIL_PARM_NUM;
-	if (!((IS_DIGIT(str[0])) || str[0] == '-'))
+	}
+	if (!((IS_DIGIT(str[0])) || str[0] == '-')) {
 		goto not_a_number;
-	for (i = 1; i < len; i++)
-		if (!(IS_DIGIT(str[i])))
+	}
+	for (i = 1; i < len; i++) {
+		if (!(IS_DIGIT(str[i]))) {
 			goto not_a_number;
+		}
+	}
 	return R_ANAL_ESIL_PARM_NUM;
 not_a_number:
 	if (r_reg_get (esil->anal->reg, str, -1))
@@ -619,11 +692,9 @@ R_API int r_anal_esil_reg_write(RAnalEsil *esil, const char *dst, ut64 num) {
 R_API int r_anal_esil_reg_read_nocallback(RAnalEsil *esil, const char *regname, ut64 *num, int *size) {
 	int ret;
 	void *old_hook_reg_read = (void *) esil->cb.hook_reg_read;
-
 	esil->cb.hook_reg_read = NULL;
 	ret = r_anal_esil_reg_read (esil, regname, num, size);
 	esil->cb.hook_reg_read = old_hook_reg_read;
-
 	return ret;
 }
 
@@ -641,7 +712,9 @@ R_API int r_anal_esil_reg_read(RAnalEsil *esil, const char *regname, ut64 *num,
 	}
 	if (!num) num = &localnum;
 	*num = 0LL;
-	if (size) *size = esil->anal->bits;
+	if (size) {
+		*size = esil->anal->bits;
+	}
 	if (esil->cb.hook_reg_read) {
 		ret = esil->cb.hook_reg_read (esil, regname, num, size);
 	}
@@ -656,6 +729,14 @@ static int esil_eq(RAnalEsil *esil) {
 	ut64 num, num2;
 	char *dst = r_anal_esil_pop (esil);
 	char *src = r_anal_esil_pop (esil);
+	if (ispackedreg (esil, dst)) {
+		char *src2 = r_anal_esil_pop (esil);
+		char *newreg = r_str_newf ("%sl", dst);
+		if (r_anal_esil_get_parm (esil, src2, &num2)) {
+			ret = r_anal_esil_reg_write (esil, newreg, num2);
+		}
+		free (newreg);
+	}
 
 	if (src && dst && r_anal_esil_reg_read_nocallback (esil, dst, &num, NULL)) {
 		if (r_anal_esil_get_parm (esil, src, &num2)) {
@@ -985,9 +1066,16 @@ static int esil_lsl(RAnalEsil *esil) {
 	char *src = r_anal_esil_pop (esil);
 	if (dst && r_anal_esil_get_parm (esil, dst, &num)) {
 		if (src && r_anal_esil_get_parm (esil, src, &num2)) {
-			ut64 res = num << num2;
-			r_anal_esil_pushnum (esil, res);
-			ret = 1;
+			if (num2 > sizeof (ut64) * 8) {
+				ERR ("esil_lsl: shift is too big");
+			} else {
+				if (num2 > 63) {
+					r_anal_esil_pushnum (esil, 0);
+				} else {
+					r_anal_esil_pushnum (esil, num << num2);
+				}
+				ret = 1;
+			}
 		} else {
 			ERR ("esil_lsl: empty stack");
 		}
@@ -1004,12 +1092,20 @@ static int esil_lsleq(RAnalEsil *esil) {
 	char *src = r_anal_esil_pop (esil);
 	if (dst && r_anal_esil_reg_read (esil, dst, &num, NULL)) {
 		if (src && r_anal_esil_get_parm (esil, src, &num2)) {
-			esil->old = num;
-			num <<= num2;
-			esil->cur = num;
-			esil->lastsz = esil_internal_sizeof_reg (esil, dst);
-			r_anal_esil_reg_write (esil, dst, num);
-			ret = 1;
+			if (num2 > sizeof (ut64) * 8) {
+				ERR ("esil_lsleq: shift is too big");
+			} else {
+				esil->old = num;
+				if (num2 > 63) {
+					num = 0;
+				} else {
+					num <<= num2;
+				}
+				esil->cur = num;
+				esil->lastsz = esil_internal_sizeof_reg (esil, dst);
+				r_anal_esil_reg_write (esil, dst, num);
+				ret = 1;
+			}
 		} else {
 			ERR ("esil_lsleq: empty stack");
 		}
@@ -1026,7 +1122,7 @@ static int esil_lsr(RAnalEsil *esil) {
 	char *src = r_anal_esil_pop (esil);
 	if (dst && r_anal_esil_get_parm (esil, dst, &num)) {
 		if (src && r_anal_esil_get_parm (esil, src, &num2)) {
-			ut64 res = num >> num2;
+			ut64 res = num >> R_MIN(num2, 63);
 			r_anal_esil_pushnum (esil, res);
 			ret = 1;
 		} else {
@@ -1079,10 +1175,10 @@ static int esil_asreq(RAnalEsil *esil) {
 			}
 			if (isNegative) {
 				if (regsize == 32) {
-					op_num = -op_num;
+					op_num = -(st64)op_num;
 					if (op_num >> param_num) {
 						op_num >>= param_num;
-						op_num = -op_num;
+						op_num = -(st64)op_num;
 					} else {
 						op_num = -1;
 					}
@@ -1133,8 +1229,8 @@ static int esil_asr(RAnalEsil *esil) {
 				ut64 mask = (regsize - 1);
 				param_num &= mask;
 				ut64 left_bits = 0;
-				if (op_num & (1 << (regsize - 1))) {
-					left_bits = (1 << param_num) - 1;
+				if (op_num & (1UL << (regsize - 1))) {
+					left_bits = (1UL << param_num) - 1;
 					left_bits <<= regsize - param_num;
 				}
 				op_num = left_bits | (op_num >> param_num);
@@ -1162,7 +1258,7 @@ static int esil_ror(RAnalEsil *esil) {
 		if (src && r_anal_esil_get_parm (esil, src, &num2)) {
 			ut64 mask = (regsize - 1);
 			num2 &= mask;
-			ut64 res = (num >> num2) | (num << ((-num2) & mask));
+			ut64 res = (num >> num2) | (num << ((-(st64)num2) & mask));
 			r_anal_esil_pushnum (esil, res);
 			ret = 1;
 		} else {
@@ -1183,7 +1279,7 @@ static int esil_rol(RAnalEsil *esil) {
 		if (src && r_anal_esil_get_parm (esil, src, &num2)) {
 			ut64 mask = (regsize - 1);
 			num2 &= mask;
-			ut64 res = (num << num2) | (num >> ((-num2) & mask));
+			ut64 res = (num << num2) | (num >> ((-(st64)num2) & mask));
 			r_anal_esil_pushnum (esil, res);
 			ret = 1;
 		} else {
@@ -1657,7 +1753,7 @@ static int esil_deceq(RAnalEsil *esil) {
 /* POKE */
 static int esil_poke_n(RAnalEsil *esil, int bits) {
 	ut64 bitmask = genmask (bits - 1);
-	ut64 num, addr;
+	ut64 num, num2, addr;
 	ut8 b[8] = {0};
 	ut64 n;
 	char *dst = r_anal_esil_pop (esil);
@@ -1671,9 +1767,25 @@ static int esil_poke_n(RAnalEsil *esil, int bits) {
 	//eprintf ("GONA POKE %d src:%s dst:%s\n", bits, src, dst);
 	if (src && r_anal_esil_get_parm (esil, src, &num)) {
 		if (dst && r_anal_esil_get_parm (esil, dst, &addr)) {
+			if (bits == 128) {
+				char *src2 = r_anal_esil_pop (esil);
+				if (src2 && r_anal_esil_get_parm (esil, src2, &num2)) {
+					r_write_ble (b, num, esil->anal->big_endian, 64);
+					ret = r_anal_esil_mem_write (esil, addr, b, bytes);
+					r_write_ble (b, num2, esil->anal->big_endian, 64);
+					ret = r_anal_esil_mem_write (esil, addr + 8, b, bytes);
+					return ret;
+				}
+				return -1;
+			}
 			int type = r_anal_esil_get_parm_type (esil, src);
 			if (type != R_ANAL_ESIL_PARM_INTERNAL) {
+				// this is a internal peek performed before a poke
+				// we disable hooks to avoid run hooks on internal peeks
+				void * oldhook = (void*)esil->cb.hook_mem_read;
+				esil->cb.hook_mem_read = NULL;
 				r_anal_esil_mem_read (esil, addr, b, bytes);
+				esil->cb.hook_mem_read = oldhook;
 				n = r_read_ble64 (b, esil->anal->big_endian);
 				esil->old = n;
 				esil->cur = num;
@@ -1692,18 +1804,27 @@ static int esil_poke_n(RAnalEsil *esil, int bits) {
 static int esil_poke1(RAnalEsil *esil) {
 	return esil_poke_n (esil, 8);
 }
+
 static int esil_poke2(RAnalEsil *esil) {
 	return esil_poke_n (esil, 16);
 }
+
 static int esil_poke3(RAnalEsil *esil) {
 	return esil_poke_n (esil, 24);
 }
+
 static int esil_poke4(RAnalEsil *esil) {
 	return esil_poke_n (esil, 32);
 }
+
 static int esil_poke8(RAnalEsil *esil) {
 	return esil_poke_n (esil, 64);
 }
+
+static int esil_poke16(RAnalEsil *esil) {
+	return esil_poke_n (esil, 128);
+}
+
 static int esil_poke(RAnalEsil *esil) {
 	return esil_poke_n (esil, esil->anal->bits);
 }
@@ -1756,16 +1877,27 @@ static int esil_poke_some(RAnalEsil *esil) {
 /* PEEK */
 
 static int esil_peek_n(RAnalEsil *esil, int bits) {
+	if (bits & 7) {
+		return 0;
+	}
 	char res[32];
 	ut64 addr;
 	int ret = 0, bytes = bits / 8;
 	char *dst = r_anal_esil_pop (esil);
-	if (bits & 7) {
-		free (dst);
-		return 0;
-	}
 	//eprintf ("GONA PEEK %d dst:%s\n", bits, dst);
 	if (dst && isregornum (esil, dst, &addr)) {
+		if (bits == 128) {
+			ut8 a[sizeof(ut64) * 2] = {0};
+			ret = r_anal_esil_mem_read (esil, addr, a, bytes);
+			ut64 b = r_read_ble64 (&a, 0); //esil->anal->big_endian);
+			ut64 c = r_read_ble64 (&a[8], 0); //esil->anal->big_endian);
+			snprintf (res, sizeof (res), "0x%" PFMT64x, b);
+			r_anal_esil_push (esil, res);
+			snprintf (res, sizeof (res), "0x%" PFMT64x, c);
+			r_anal_esil_push (esil, res);
+			free (dst);
+			return ret;
+		}
 		ut64 bitmask = genmask (bits - 1);
 		ut8 a[sizeof(ut64)] = {0};
 		ret = r_anal_esil_mem_read (esil, addr, a, bytes);
@@ -1784,18 +1916,28 @@ static int esil_peek_n(RAnalEsil *esil, int bits) {
 static int esil_peek1(RAnalEsil *esil) {
 	return esil_peek_n (esil, 8);
 }
+
 static int esil_peek2(RAnalEsil *esil) {
 	return esil_peek_n (esil, 16);
 }
+
 static int esil_peek3(RAnalEsil *esil) {
 	return esil_peek_n (esil, 24);
 }
+
 static int esil_peek4(RAnalEsil *esil) {
 	return esil_peek_n (esil, 32);
 }
+
 static int esil_peek8(RAnalEsil *esil) {
 	return esil_peek_n (esil, 64);
 }
+
+static int esil_peek16(RAnalEsil *esil) {
+	// packed only
+	return esil_peek_n (esil, 128);
+}
+
 static int esil_peek(RAnalEsil *esil) {
 	return esil_peek_n (esil, esil->anal->bits);
 };
@@ -2294,6 +2436,103 @@ static int esil_mem_deceq(RAnalEsil *esil) {
 	return esil_mem_deceq_n (esil, esil->anal->bits);
 }
 
+/* LSLEQ */
+
+static int esil_mem_lsleq_n(RAnalEsil *esil, int bits) {
+	int ret = 0;
+	ut64 s, d;
+	char *dst = r_anal_esil_pop (esil);
+	char *src0 = r_anal_esil_pop (esil);
+	char *src1 = NULL;
+	if (src0 && r_anal_esil_get_parm (esil, src0, &s)) {
+		if (s > sizeof (ut64) * 8) {
+			ERR ("esil_mem_lsleq_n: shift is too big");
+		} else {
+			r_anal_esil_push (esil, dst);
+			ret = (!!esil_peek_n (esil, bits));
+			src1 = r_anal_esil_pop (esil);
+			if (src1 && r_anal_esil_get_parm (esil, src1, &d)) {
+				if (s > 63) {
+					d = 0;
+				} else {
+					d <<= s;
+				}
+				r_anal_esil_pushnum (esil, d);
+				r_anal_esil_push (esil, dst);
+				ret &= (!!esil_poke_n (esil, bits));
+			} else {
+				ret = 0;
+			}
+		}
+	}
+	if (!ret) {
+		ERR ("esil_mem_lsleq_n: invalid parameters");
+	}
+	free (dst);
+	free (src0);
+	free (src1);
+	return ret;
+}
+
+static int esil_mem_lsleq1(RAnalEsil *esil) {
+	return esil_mem_lsleq_n (esil, 8);
+}
+static int esil_mem_lsleq2(RAnalEsil *esil) {
+	return esil_mem_lsleq_n (esil, 16);
+}
+static int esil_mem_lsleq4(RAnalEsil *esil) {
+	return esil_mem_lsleq_n (esil, 32);
+}
+static int esil_mem_lsleq8(RAnalEsil *esil) {
+	return esil_mem_lsleq_n (esil, 64);
+}
+static int esil_mem_lsleq(RAnalEsil *esil) {
+	return esil_mem_lsleq_n (esil, esil->anal->bits);
+}
+
+/* LSREQ */
+
+static int esil_mem_lsreq_n(RAnalEsil *esil, int bits) {
+	int ret = 0;
+	ut64 s, d;
+	char *dst = r_anal_esil_pop (esil);
+	char *src0 = r_anal_esil_pop (esil);
+	char *src1 = NULL;
+	if (src0 && r_anal_esil_get_parm (esil, src0, &s)) {
+		r_anal_esil_push (esil, dst);
+		ret = (!!esil_peek_n (esil, bits));
+		src1 = r_anal_esil_pop (esil);
+		if (src1 && r_anal_esil_get_parm (esil, src1, &d)) {
+			d >>= s;
+			r_anal_esil_pushnum (esil, d);
+			r_anal_esil_push (esil, dst);
+			ret &= (!!esil_poke_n (esil, bits));
+		} else ret = 0;
+	}
+	if (!ret)
+		ERR ("esil_mem_lsreq_n: invalid parameters");
+	free (dst);
+	free (src0);
+	free (src1);
+	return ret;
+}
+
+static int esil_mem_lsreq1(RAnalEsil *esil) {
+	return esil_mem_lsreq_n (esil, 8);
+}
+static int esil_mem_lsreq2(RAnalEsil *esil) {
+	return esil_mem_lsreq_n (esil, 16);
+}
+static int esil_mem_lsreq4(RAnalEsil *esil) {
+	return esil_mem_lsreq_n (esil, 32);
+}
+static int esil_mem_lsreq8(RAnalEsil *esil) {
+	return esil_mem_lsreq_n (esil, 64);
+}
+static int esil_mem_lsreq(RAnalEsil *esil) {
+	return esil_mem_lsreq_n (esil, esil->anal->bits);
+}
+
 /* get value of register or memory reference and push the value */
 static int esil_num(RAnalEsil *esil) {
 	char *dup_me;
@@ -2815,6 +3054,7 @@ static void r_anal_esil_setup_ops(RAnalEsil *esil) {
 	OP ("=[3]", esil_poke3);
 	OP ("=[4]", esil_poke4);
 	OP ("=[8]", esil_poke8);
+	OP ("=[16]", esil_poke16);
 	OP ("|=[]", esil_mem_oreq);
 	OP ("|=[1]", esil_mem_oreq1);
 	OP ("|=[2]", esil_mem_oreq2);
@@ -2865,6 +3105,16 @@ static void r_anal_esil_setup_ops(RAnalEsil *esil) {
 	OP ("--=[2]", esil_mem_deceq2);
 	OP ("--=[4]", esil_mem_deceq4);
 	OP ("--=[8]", esil_mem_deceq8);
+        OP ("<<=[]", esil_mem_lsleq);
+	OP ("<<=[1]", esil_mem_lsleq1);
+	OP ("<<=[2]", esil_mem_lsleq2);
+	OP ("<<=[4]", esil_mem_lsleq4);
+	OP ("<<=[8]", esil_mem_lsleq8);
+	OP (">>=[]", esil_mem_lsreq);
+	OP (">>=[1]", esil_mem_lsreq1);
+	OP (">>=[2]", esil_mem_lsreq2);
+	OP (">>=[4]", esil_mem_lsreq4);
+	OP (">>=[8]", esil_mem_lsreq8);
 	OP ("[]", esil_peek);
 	OP ("[*]", esil_peek_some);
 	OP ("=[*]", esil_poke_some);
@@ -2873,6 +3123,7 @@ static void r_anal_esil_setup_ops(RAnalEsil *esil) {
 	OP ("[3]", esil_peek3);
 	OP ("[4]", esil_peek4);
 	OP ("[8]", esil_peek8);
+	OP ("[16]", esil_peek16);
 	OP ("STACK", r_anal_esil_dumpstack);
 	OP ("REPEAT", esil_repeat);
 	OP ("POP", esil_pop);
