@@ -13,6 +13,8 @@
 #include <r_anal.h>
 #if USE_NEW_FCN_STORE
 
+#define STROFF(x) sdb_fmt (4, "flg.%"PFMT64x, x)
+
 static int cmpfun(void *a, void *b) {
 	RAnalFunction *fa = (RAnalFunction*)a;
 	RAnalFunction *fb = (RAnalFunction*)b;
@@ -20,33 +22,30 @@ static int cmpfun(void *a, void *b) {
 	return (fb->addr - fa->addr);
 }
 
+static void listrange_free_kv(HtKv *kv) {
+	free (kv->key);
+	r_list_free (kv->value);
+	free (kv);
+}
+
 R_API RListRange* r_listrange_new () {
 	RListRange *s = R_NEW (RListRange);
-	s->h = r_hashtable64_new ();
-	s->h->free = (RHashFree)r_list_free;
+	s->h = ht_new (NULL, listrange_free_kv, NULL);
 	//s->l shouldn't free is a helper structure to get data in order
 	//s->h will free the list that contains RAnalFunction
 	s->l = r_list_new ();
 	return s;
 }
 
-// FIXME: Do not hardcode such things/values!!!
-static inline ut64 r_listrange_key(ut64 addr) {
-	const ut64 KXOR = 0x18abc3e127d549ac; // XXX wrong for mingw32
-	ut64 key = addr & 0xfffffffffffff400;
-	key ^= KXOR;
-	//eprintf ("%llx = %llx\n", addr, key);
-	return key;
-	//return (addr >> RANGEBITS);
-}
-
 static inline ut64 r_listrange_next(ut64 addr) {
 	return (addr + RANGE);
 }
 
 R_API void r_listrange_free(RListRange *s) {
-	if (!s) return;
-	r_hashtable64_free (s->h);
+	if (!s) {
+		return;
+	}
+	ht_free (s->h);
 	r_list_free (s->l);
 	free (s);
 }
@@ -56,17 +55,17 @@ R_API void r_listrange_add(RListRange *s, RAnalFunction *f) {
 	RList *list;
 	ut64 from = f->addr;
 	ut64 to = f->addr + f->size;
-	for (addr = from; addr<to; addr = r_listrange_next (addr)) {
-		ut64 key = r_listrange_key (addr);
-		list = r_hashtable64_lookup (s->h, key);
+	for (addr = from; addr < to; addr = r_listrange_next (addr)) {
+		char *key = STROFF (addr);
+		list = ht_find (s->h, key, NULL);
 		if (list) {
-			if (!r_list_contains (list, f))
-			r_list_append (list, f);
+			if (!r_list_contains (list, f)) {
+				r_list_append (list, f);
+			}
 		} else {
-			list = r_list_new ();
-			list->free = (RListFree)r_anal_fcn_free;
+			list = r_list_new ((RListFree)r_anal_fcn_free);
 			r_list_append (list, f);
-			r_hashtable64_insert (s->h, key, list);
+			ht_insert (s->h, key, list);
 		}
 	}
 	r_list_add_sorted (s->l, f, cmpfun);
@@ -79,14 +78,18 @@ R_API void r_listrange_del(RListRange *s, RAnalFunction *f) {
 	from = f->addr;
 	to = f->addr + f->size;
 	for (addr = from; addr < to; addr = r_listrange_next (addr)) {
-		list = r_hashtable64_lookup (s->h, r_listrange_key (addr));
-		if (list) r_list_delete_data (list, f);
+		list = ht_find (s->h, STROFF (addr), NULL);
+		if (list) {
+			r_list_delete_data (list, f);
+		}
 	}
 	r_list_delete_data (s->l, f);
 }
 
 R_API void r_listrange_resize(RListRange *s, RAnalFunction *f, int newsize) {
-	if (!f) return;
+	if (!f) {
+		return;
+	}
 	r_listrange_del (s, f);
 	f->size = newsize;
 	r_listrange_add (s, f);
@@ -95,11 +98,12 @@ R_API void r_listrange_resize(RListRange *s, RAnalFunction *f, int newsize) {
 R_API RAnalFunction *r_listrange_find_in_range(RListRange* s, ut64 addr) {
 	RAnalFunction *f;
 	RListIter *iter;
-	RList *list = r_hashtable64_lookup (s->h, r_listrange_key (addr));
-	if (list)
-	r_list_foreach (list, iter, f) {
-		if (R_BETWEEN (f->addr, addr, f->addr+f->size))
-			return f;
+	RList *list = ht_find (s->h, STROFF (addr), NULL);
+	if (list) {
+		r_list_foreach (list, iter, f) {
+			if (R_BETWEEN (f->addr, addr, f->addr + f->_size))
+				return f;
+		}
 	}
 	return NULL;
 }
@@ -107,11 +111,13 @@ R_API RAnalFunction *r_listrange_find_in_range(RListRange* s, ut64 addr) {
 R_API RAnalFunction *r_listrange_find_root(RListRange* s, ut64 addr) {
 	RAnalFunction *f;
 	RListIter *iter;
-	RList *list = r_hashtable64_lookup (s->h, r_listrange_key (addr));
-	if (list)
-	r_list_foreach (list, iter, f) {
-		if (addr == f->addr)
-			return f;
+	RList *list = ht_find (s->h, STROFF (addr), NULL);
+	if (list) {
+		r_list_foreach (list, iter, f) {
+			if (addr == f->addr) {
+				return f;
+			}
+		}
 	}
 	return NULL;
 }
