@@ -96,6 +96,14 @@ CPU_CONST cpu_memsize_m640_m1280m_m1281_m2560_m2561[] = {
 	{ NULL, 0, 0, 0 },
 };
 
+CPU_CONST cpu_memsize_xmega128a4u[] = {
+	{ "eeprom_size", CPU_CONST_PARAM,  0x800, sizeof (ut32) },
+	{ "io_size",     CPU_CONST_PARAM, 0x1000, sizeof (ut32) },
+	{ "sram_start",  CPU_CONST_PARAM,  0x800, sizeof (ut32) },
+	{ "sram_size",   CPU_CONST_PARAM, 0x2000, sizeof (ut32) },
+	{ NULL, 0, 0, 0 },
+};
+
 CPU_CONST cpu_pagesize_5_bits[] = {
 	{ "page_size", CPU_CONST_PARAM, 5, sizeof (ut8) },
 	{ NULL, 0, 0, 0 },
@@ -115,6 +123,15 @@ CPU_MODEL cpu_models[] = {
 			NULL
 		},
 	},
+	{
+		.model = "ATxmega128a4u", .pc = 17,
+		.consts = {
+			cpu_reg_common,
+			cpu_memsize_xmega128a4u,
+			cpu_pagesize_7_bits,
+			NULL
+		}
+	},
 	{ .model = "ATmega1280",  .pc = 16, .inherit = "ATmega640" },
 	{ .model = "ATmega1281",  .pc = 16, .inherit = "ATmega640" },
 	{ .model = "ATmega2560",  .pc = 17, .inherit = "ATmega640" },
@@ -841,18 +858,18 @@ INST_HANDLER (ldd) {	// LD Rd, Y	LD Rd, Z
 		op, "ram",
 		buf[0] & 0x8 ? 'y' : 'z',	// index register Y/Z
 		0,				// no use RAMP* registers
-		!(buf[1] & 0x1)
+		!(buf[1] & 0x10)
 			? 0			// no increment
 			: buf[0] & 0x1
 				? 1		// post incremented
 				: -1,		// pre decremented
-		!(buf[1] & 0x1) ? offset : 0,	// offset or not offset
+		!(buf[1] & 0x10) ? offset : 0,	// offset or not offset
 		0);				// load operation (!st)
 	// load register
 	ESIL_A ("r%d,=,", ((buf[1] & 1) << 4) | ((buf[0] >> 4) & 0xf));
 	// cycles
 	op->cycles =
-		(buf[1] & 0x1) == 0
+		(buf[1] & 0x10) == 0
 			? (!offset ? 1 : 3)		// LDD
 			: (buf[0] & 0x3) == 0
 				? 1			// LD Rd, X
@@ -1335,12 +1352,12 @@ INST_HANDLER (std) {	// ST Y, Rr	ST Z, Rr
 		op, "ram",
 		buf[0] & 0x8 ? 'y' : 'z',	// index register Y/Z
 		0,				// no use RAMP* registers
-		!(buf[1] & 0x1)
+		!(buf[1] & 0x10)
 			? 0			// no increment
 			: buf[0] & 0x1
 				? 1		// post incremented
 				: -1,		// pre decremented
-		!(buf[1] & 0x1)
+		!(buf[1] & 0x10)
 			? (buf[1] & 0x20)	// offset
 			| ((buf[1] & 0xc) << 1)
 			| (buf[0] & 0x7)
@@ -1592,7 +1609,7 @@ static int avr_custom_des (RAnalEsil *esil) {
 	if (des_round != desctx.round) {
 		desctx.round = des_round;
 	}
-	
+
 	if (!desctx.round) {
 		int i;
 		//generating all round keys
