@@ -133,7 +133,7 @@ RAnalPlugin r_anal_plugin_bf = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_bf,
 	.version = R2_VERSION
