@@ -1465,7 +1465,7 @@ RAnalPlugin r_anal_plugin_gb = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_gb,
 	.version = R2_VERSION
