@@ -198,7 +198,7 @@ RAnalPlugin r_anal_plugin_i8080 = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_i8080,
 	.version = R2_VERSION
