@@ -726,7 +726,11 @@ static int analop(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 			break;
 		}
 		break;
+	case MIPS_INS_LI:
 	case MIPS_INS_LUI:
+		SET_VAL (op, 1);
+		op->type = R_ANAL_OP_TYPE_MOV;
+		break;
 	case MIPS_INS_MOVE:
 		op->type = R_ANAL_OP_TYPE_MOV;
 		SET_SRC_DST_2_REGS (op);
@@ -869,12 +873,12 @@ static int analop(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 		break;
 	case MIPS_INS_JR:
 	case MIPS_INS_JRC:
-		op->type = R_ANAL_OP_TYPE_JMP;
+		op->type = R_ANAL_OP_TYPE_RJMP;
 		op->delay = 1;
-        // register is $ra, so jmp is a return
-        if (insn->detail->mips.operands[0].reg == MIPS_REG_RA) {
-            op->type = R_ANAL_OP_TYPE_RET;
-        }
+		// register is $ra, so jmp is a return
+		if (insn->detail->mips.operands[0].reg == MIPS_REG_RA) {
+			op->type = R_ANAL_OP_TYPE_RET;
+		}
 		break;
 	case MIPS_INS_SLTI:
 	case MIPS_INS_SLTIU:
@@ -983,7 +987,7 @@ RAnalPlugin r_anal_plugin_mips_cs = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_mips_cs,
 	.version = R2_VERSION
