@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2014 - jn */
+/* radare - LGPL - Copyright 2014 - jn, maijin */
 
 #include <r_anal.h>
 #include <r_types.h>
@@ -10,17 +10,22 @@ static int null_anal(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int l
 	return op->size = 1;
 }
 
-struct r_anal_plugin_t r_anal_plugin_null = {
+static int null_set_reg_profile(RAnal* anal){
+	return r_reg_set_profile_string(anal->reg, "");
+}
+
+RAnalPlugin r_anal_plugin_null = {
 	.name = "null",
 	.desc = "Fallback/Null analysis plugin",
 	.arch = "none",
 	.license = "LGPL3",
 	.bits = 8|16|32|64,	/* is this used? */
 	.op = &null_anal,
+	.set_reg_profile = &null_set_reg_profile,
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_null,
 	.version = R2_VERSION
