@@ -53,8 +53,12 @@ static const char* cmask64(const char *mb_c, const char *me_c) {
 	static char cmask[32];
 	ut64 mb = 0;
 	ut64 me = 0;
-	if (mb_c) mb = atol (mb_c);
-	if (me_c) me = atol (me_c);
+	if (mb_c) {
+		mb = strtol (mb_c, NULL, 16);
+	}
+	if (me_c) {
+		me = strtol (me_c, NULL, 16);
+	}
 	snprintf (cmask, sizeof (cmask), "0x%"PFMT64x"", mask64 (mb, me));
 	return cmask;
 }
@@ -84,8 +88,8 @@ static const char* cmask32(const char *mb_c, const char *me_c) {
 	static char cmask[32];
 	ut32 mb = 32;
 	ut32 me = 32;
-	if (mb_c) mb += atol (mb_c);
-	if (me_c) me += atol (me_c);
+	if (mb_c) mb += strtol (mb_c, NULL, 16);
+	if (me_c) me += strtol (me_c, NULL, 16);
 	snprintf (cmask, sizeof (cmask), "0x%"PFMT32x"", mask32 (mb, me));
 	return cmask;
 }
@@ -465,6 +469,8 @@ static int analop_vle(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 		//op->id = instr->type;
 
 		switch (op->type) {
+		case R_ANAL_OP_TYPE_ILL:
+			break;
 		case R_ANAL_OP_TYPE_ADD:
 			break;
 		case R_ANAL_OP_TYPE_AND:
@@ -499,6 +505,10 @@ static int analop_vle(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 			break;
 		case R_ANAL_OP_TYPE_OR:
 			break;
+		case R_ANAL_OP_TYPE_ROR:
+			break;
+		case R_ANAL_OP_TYPE_ROL:
+			break;
 		case R_ANAL_OP_TYPE_RCALL:
 			op->eob = true;
 			break;
@@ -524,7 +534,7 @@ static int analop_vle(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 		case R_ANAL_OP_TYPE_XOR:
 			break;
 		default:
-			eprintf ("Missing an R_ANAL_OP_TYPE");
+			//eprintf ("Missing an R_ANAL_OP_TYPE (%"PFMT64u")\n", op->type);
 			break;
 		}
 		vle_free (instr);
@@ -535,11 +545,11 @@ static int analop_vle(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 
 static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 	static csh handle = 0;
-	static int omode = -1;
+	static int omode = -1, obits = -1;
 	int n, ret;
 	cs_insn *insn;
 	int mode = (a->bits == 64) ? CS_MODE_64 : (a->bits == 32) ? CS_MODE_32 : 0;
-	mode |= CS_MODE_BIG_ENDIAN;
+	mode |= a->big_endian ? CS_MODE_BIG_ENDIAN : CS_MODE_LITTLE_ENDIAN;
 
 	op->delay = 0;
 	op->type = R_ANAL_OP_TYPE_NULL;
@@ -547,16 +557,21 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 	op->fail = UT64_MAX;
 	op->ptr = op->val = UT64_MAX;
 	if (a->cpu && strncmp (a->cpu, "vle", 3) == 0) {
+		// vle is big-endian only
+		if (!a->big_endian) {
+			return -1;
+		}
 		ret = analop_vle (a, op, addr, buf, len);
 		if (ret >= 0) {
 			return op->size;
 		}
 	}
 
-	if (mode != omode) {
+	if (mode != omode || a->bits != obits) {
 		cs_close (&handle);
 		handle = 0;
 		omode = mode;
+		obits = a->bits;
 	}
 	if (handle == 0) {
 		ret = cs_open (CS_ARCH_PPC, mode, &handle);
@@ -618,11 +633,11 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			break;
 		case PPC_INS_CLRLWI:
 			op->type = R_ANAL_OP_TYPE_AND;
-			esilprintf (op, "%s,0x%"PFMT64x",&,%s,=", ARG (1), cmask32 (ARG (2), "31"), ARG (0));
+			esilprintf (op, "%s,%s,&,%s,=", ARG (1), (ut64) cmask32 (ARG (2), "0x1F"), ARG (0));
 			break;
 		case PPC_INS_RLWINM:
 			op->type = R_ANAL_OP_TYPE_ROL;
-			esilprintf (op, "%s,%s,<<<,0x%"PFMT64x",&,%s,=", ARG (2), ARG (1), cmask32 (ARG (3), ARG (4)), ARG (0));
+			esilprintf (op, "%s,%s,<<<,%s,&,%s,=", ARG (2), ARG (1), cmask32 (ARG (3), ARG (4)), ARG (0));
 			break;
 		case PPC_INS_SC:
 			op->type = R_ANAL_OP_TYPE_SWI;
@@ -659,11 +674,13 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case PPC_INS_STWU:
 		case PPC_INS_STWUX:
 		case PPC_INS_STWX:
-		case PPC_INS_STWBRX:
 		case PPC_INS_STWCX:
 			op->type = R_ANAL_OP_TYPE_STORE;
 			esilprintf (op, "%s,%s", ARG (0), ARG2 (1, "=[4]"));
 			break;
+		case PPC_INS_STWBRX:
+			op->type = R_ANAL_OP_TYPE_STORE;
+			break;
 		case PPC_INS_STB:
 		case PPC_INS_STBU:
 			op->type = R_ANAL_OP_TYPE_STORE;
@@ -679,8 +696,10 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			op->type = R_ANAL_OP_TYPE_STORE;
 			esilprintf (op, "%s,%s", ARG (0), ARG2 (1, "=[8]"));
 			break;
-		case PPC_INS_LA:
 		case PPC_INS_LBZ:
+#if CS_API_MAJOR >= 4
+		case PPC_INS_LBZCIX:
+#endif
 		case PPC_INS_LBZU:
 		case PPC_INS_LBZUX:
 		case PPC_INS_LBZX:
@@ -689,13 +708,18 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			break;
 		case PPC_INS_LD:
 		case PPC_INS_LDARX:
-		case PPC_INS_LDBRX:
+#if CS_API_MAJOR >= 4
+		case PPC_INS_LDCIX:
+#endif
 		case PPC_INS_LDU:
 		case PPC_INS_LDUX:
 		case PPC_INS_LDX:
 			op->type = R_ANAL_OP_TYPE_LOAD;
 			esilprintf (op, "%s,%s,=", ARG2 (1, "[8]"), ARG (0));
 			break;
+		case PPC_INS_LDBRX:
+			op->type = R_ANAL_OP_TYPE_LOAD;
+			break;
 		case PPC_INS_LFD:
 		case PPC_INS_LFDU:
 		case PPC_INS_LFDUX:
@@ -713,24 +737,31 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case PPC_INS_LHAU:
 		case PPC_INS_LHAUX:
 		case PPC_INS_LHAX:
-		case PPC_INS_LHBRX:
 		case PPC_INS_LHZ:
 		case PPC_INS_LHZU:
 			op->type = R_ANAL_OP_TYPE_LOAD;
 			esilprintf (op, "%s,%s,=", ARG2 (1, "[2]"), ARG (0));
 			break;
+		case PPC_INS_LHBRX:
+			op->type = R_ANAL_OP_TYPE_LOAD;
+			break;
 		case PPC_INS_LWA:
 		case PPC_INS_LWARX:
 		case PPC_INS_LWAUX:
 		case PPC_INS_LWAX:
-		case PPC_INS_LWBRX:
 		case PPC_INS_LWZ:
+#if CS_API_MAJOR >= 4
+		case PPC_INS_LWZCIX:
+#endif
 		case PPC_INS_LWZU:
 		case PPC_INS_LWZUX:
 		case PPC_INS_LWZX:
 			op->type = R_ANAL_OP_TYPE_LOAD;
 			esilprintf (op, "%s,%s,=", ARG2 (1, "[4]"), ARG (0));
 			break;
+		case PPC_INS_LWBRX:
+			op->type = R_ANAL_OP_TYPE_LOAD;
+			break;
 		case PPC_INS_SLW:
 		case PPC_INS_SLWI:
 			op->type = R_ANAL_OP_TYPE_SHL;
@@ -866,7 +897,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			op->type = R_ANAL_OP_TYPE_CJMP;
 			op->jump = IMM (0);
 			op->fail = addr + op->size;
-			esilprintf (op, "ctr,?{,%s,pc,=,}", ARG (0));
+			esilprintf (op, "1,ctr,-=,ctr,?{,%s,pc,=,}", ARG (0));
 			break;
 		case PPC_INS_BDNZA:
 			op->type = R_ANAL_OP_TYPE_CJMP;
@@ -886,7 +917,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case PPC_INS_BDNZLR:
 			op->type = R_ANAL_OP_TYPE_CJMP;
 			op->fail = addr + op->size;
-			esilprintf (op, "ctr,?{,lr,pc,=,},");
+			esilprintf (op, "1,ctr,-=,ctr,?{,lr,pc,=,},");
 			break;
 		case PPC_INS_BDNZLRL:
 			op->fail = addr + op->size;
@@ -896,7 +927,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			op->type = R_ANAL_OP_TYPE_CJMP;
 			op->jump = IMM (0);
 			op->fail = addr + op->size;
-			esilprintf (op, "ctr,0,==,?{,%s,pc,=,}", ARG (0));
+			esilprintf (op, "1,ctr,-=,ctr,0,==,?{,%s,pc,=,}", ARG (0));
 			break;
 		case PPC_INS_BDZA:
 			op->type = R_ANAL_OP_TYPE_CJMP;
@@ -916,7 +947,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case PPC_INS_BDZLR:
 			op->type = R_ANAL_OP_TYPE_CJMP;
 			op->fail = addr + op->size;
-			esilprintf (op, "ctr,0,==,?{,lr,pc,=,}");
+			esilprintf (op, "1,ctr,-=,ctr,0,==,?{,lr,pc,=,}");
 			break;
 		case PPC_INS_BDZLRL:
 			op->type = R_ANAL_OP_TYPE_CJMP;
@@ -985,23 +1016,23 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			break;
 		case PPC_INS_NOR:
 			op->type = R_ANAL_OP_TYPE_NOR;
-			esilprintf (op, "%s,!,%s,|,%s,=", ARG (1), ARG (2), ARG (0));
+			esilprintf (op, "%s,%s,|,!,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
 		case PPC_INS_XOR:
 		case PPC_INS_XORI:
 			op->type = R_ANAL_OP_TYPE_XOR;
-			esilprintf (op, "%s,%s,^,%s,=", ARG (1), ARG (2), ARG (0));
+			esilprintf (op, "%s,%s,^,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
 		case PPC_INS_XORIS:
 			op->type = R_ANAL_OP_TYPE_XOR;
-			esilprintf (op, "16,%s,>>,%s,^,%s,=", ARG (1), ARG (2), ARG (0));
+			esilprintf (op, "16,%s,<<,%s,^,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
 		case PPC_INS_DIVD:
 		case PPC_INS_DIVDU:
 		case PPC_INS_DIVW:
 		case PPC_INS_DIVWU:
 			op->type = R_ANAL_OP_TYPE_DIV;
-			esilprintf (op, "%s,%s,/,%s,=", ARG (1), ARG (2), ARG (0));
+			esilprintf (op, "%s,%s,/,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
 		case PPC_INS_BL:
 		case PPC_INS_BLA:
@@ -1016,17 +1047,22 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case PPC_INS_AND:
 		case PPC_INS_NAND:
 		case PPC_INS_ANDI:
-		case PPC_INS_ANDIS:
 			op->type = R_ANAL_OP_TYPE_AND;
 			esilprintf (op, "%s,%s,&,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
+		case PPC_INS_ANDIS:
+			op->type = R_ANAL_OP_TYPE_AND;
+			esilprintf (op, "16,%s,<<,%s,&,%s,=", ARG (2), ARG (1), ARG (0));
+			break;
 		case PPC_INS_OR:
-		case PPC_INS_ORC:
 		case PPC_INS_ORI:
-		case PPC_INS_ORIS:
 			op->type = R_ANAL_OP_TYPE_OR;
 			esilprintf (op, "%s,%s,|,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
+		case PPC_INS_ORIS:
+			op->type = R_ANAL_OP_TYPE_OR;
+			esilprintf (op, "16,%s,<<,%s,|,%s,=", ARG (2), ARG (1), ARG (0));
+			break;
 		case PPC_INS_MFPVR:
 			op->type = R_ANAL_OP_TYPE_MOV;
 			esilprintf (op, "pvr,%s,=", ARG (0));
@@ -1083,7 +1119,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			break;
 		case PPC_INS_CLRLDI:
 			op->type = R_ANAL_OP_TYPE_AND;
-			esilprintf (op, "%s,0x%"PFMT64x",&,%s,=", ARG (1), cmask64 (ARG (2), "63"), ARG (0));
+			esilprintf (op, "%s,%s,&,%s,=", ARG (1), cmask64 (ARG (2), "0x3F"), ARG (0));
 			break;
 		case PPC_INS_ROTLDI:
 			op->type = R_ANAL_OP_TYPE_ROL;
@@ -1092,7 +1128,12 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case PPC_INS_RLDCL:
 		case PPC_INS_RLDICL:
 			op->type = R_ANAL_OP_TYPE_ROL;
-			esilprintf (op, "%s,%s,<<<,0x%"PFMT64x",&,%s,=", ARG (2), ARG (1), cmask64 (ARG (3), "63"), ARG (0));
+			esilprintf (op, "%s,%s,<<<,%s,&,%s,=", ARG (2), ARG (1), cmask64 (ARG (3), "0x3F"), ARG (0));
+			break;
+		case PPC_INS_RLDCR:
+		case PPC_INS_RLDICR:
+			op->type = R_ANAL_OP_TYPE_ROL;
+			esilprintf (op, "%s,%s,<<<,%s,&,%s,=", ARG (2), ARG (1), cmask64 (0, ARG (3)), ARG (0));
 			break;
 		}
 		r_strbuf_fini (&op->esil);
@@ -1119,7 +1160,7 @@ RAnalPlugin r_anal_plugin_ppc_cs = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_ppc_cs,
 	.version = R2_VERSION
