@@ -254,7 +254,7 @@ RAnalPlugin r_anal_plugin_snes = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_snes,
 	.version = R2_VERSION
