@@ -334,7 +334,7 @@ RAnalPlugin r_anal_plugin_sparc_cs = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_sparc_cs,
 	.version = R2_VERSION
