@@ -11,22 +11,20 @@
 
 
 static ut64 ws_find_label(int l, RIOBind iob) {
-	ut64 cur = 0, size = iob.size(iob.io);
+	RIO *io = iob.io;
+	ut64 cur = 0, size = iob.desc_size (io->desc);
 	ut8 buf[128];
-	RAsmOp *aop;
-	aop = R_NEW0(RAsmOp);
-	iob.read_at(iob.io, cur, buf, 128);
-	while(cur <= size && wsdis(aop, buf, 128)) {
-		if(	aop->buf_asm[0] == 'm' &&
-			aop->buf_asm[1] == 'a' &&
-			l == atoi(&aop->buf_asm[5])) {
-				r_asm_op_free(aop);
+	RAsmOp aop;;
+	iob.read_at (iob.io, cur, buf, 128);
+	while (cur <= size && wsdis(&aop, buf, 128)) {
+		if(	aop.buf_asm[0] == 'm' &&
+			aop.buf_asm[1] == 'a' &&
+			l == atoi(&aop.buf_asm[5])) {
 				return cur;
 		}
-		cur = cur + aop->size;
+		cur = cur + aop.size;
 		iob.read_at(iob.io, cur, buf, 128);
 	}
-	r_asm_op_free(aop);
 	return 0;
 }
 
@@ -141,7 +139,7 @@ RAnalPlugin r_anal_plugin_ws = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_ws,
 	.version = R2_VERSION
