@@ -213,7 +213,7 @@ RAnalPlugin r_anal_plugin_xap = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_xap,
 	.version = R2_VERSION
