@@ -2008,7 +2008,7 @@ RAnalPlugin r_anal_plugin_xtensa = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_xtensa,
 	.version = R2_VERSION
