@@ -32,6 +32,8 @@
 
 #define INSNLEN 4
 
+#define SIZE 128
+
 /* Cached mapping symbol state.  */
 enum map_type
 {
@@ -137,7 +139,7 @@ extract_fields (aarch64_insn code, aarch64_insn mask, ...)
   while (num--)
     {
       kind = va_arg (va, enum aarch64_field_kind);
-      field = &fields[kind];
+      field = &aarch64_fields[kind];
       value <<= field->width;
       value |= extract_field (kind, code, mask);
     }
@@ -151,7 +153,10 @@ sign_extend (aarch64_insn value, unsigned i)
 {
   uint32_t ret = value;
 
-  assert (i < 32);
+  if (i >= 32) {
+    // assert (i < 32);
+    return 0;
+  }
   if ((value >> i) & 0x1)
     {
       uint32_t val = (uint32_t)(-1) << i;
@@ -723,15 +728,12 @@ aarch64_ext_limm (const aarch64_operand *self ATTRIBUTE_UNUSED,
     }
   else
     {
-      switch (S)
-	{
-	case 0x00 ... 0x1f: /* 0xxxxx */ simd_size = 32;           break;
-	case 0x20 ... 0x2f: /* 10xxxx */ simd_size = 16; S &= 0xf; break;
-	case 0x30 ... 0x37: /* 110xxx */ simd_size =  8; S &= 0x7; break;
-	case 0x38 ... 0x3b: /* 1110xx */ simd_size =  4; S &= 0x3; break;
-	case 0x3c ... 0x3d: /* 11110x */ simd_size =  2; S &= 0x1; break;
-	default: return 0;
-	}
+      if (S <= 0x1f) { simd_size = 32; }
+      else if (S >= 0x20 && S <= 0x2f) { simd_size = 16; S &= 0xf; }
+      else if (S >= 0x30 && S <= 0x37) { simd_size = 8; S &= 0x7; }
+      else if (S >= 0x38 && S <= 0x3b) { simd_size = 4; S &= 0x3; }
+      else if (S >= 0x3c && S <= 0x3d) { simd_size = 2; S &= 0x1; }
+      else { return 0; }
       mask = (1ull << simd_size) - 1;
       /* Top bits are IGNORED.  */
       R &= simd_size - 1;
@@ -871,7 +873,7 @@ aarch64_ext_addr_simm (const aarch64_operand *self, aarch64_opnd_info *info,
   info->addr.base_regno = extract_field (FLD_Rn, code, 0);
   /* simm (imm9 or imm7)  */
   imm = extract_field (self->fields[0], code, 0);
-  info->addr.offset.imm = sign_extend (imm, fields[self->fields[0]].width - 1);
+  info->addr.offset.imm = sign_extend (imm, aarch64_fields[self->fields[0]].width - 1);
   if (self->fields[0] == FLD_imm7)
     /* scaled immediate in ld/st pair instructions.  */
     info->addr.offset.imm *= aarch64_get_qualifier_esize (info->qualifier);
@@ -2049,8 +2051,7 @@ print_operands (bfd_vma pc, const aarch64_opcode *opcode,
   int i, pcrel_p, num_printed;
   for (i = 0, num_printed = 0; i < AARCH64_MAX_OPND_NUM; ++i)
     {
-      const size_t size = 128;
-      char str[size];
+      char str[SIZE];
       /* We regard the opcode operand info more, however we also look into
 	 the inst->operands to support the disassembling of the optional
 	 operand.
@@ -2061,7 +2062,7 @@ print_operands (bfd_vma pc, const aarch64_opcode *opcode,
 	break;
 
       /* Generate the operand string in STR.  */
-      aarch64_print_operand (str, size, pc, opcode, opnds, i, &pcrel_p,
+      aarch64_print_operand (str, SIZE, pc, opcode, opnds, i, &pcrel_p,
 			     &info->target);
 
       /* Print the delimiter (taking account of omitted operand(s)).  */
