@@ -72,9 +72,11 @@ static ut32 arm_disasm_branch(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short link = (inst >> 24) & 0x01;
 	int offset = (inst << 2) & 0x03ffffff;
 
-	if (offset & 0x02000000) offset |= 0xfc000000;
+	if (offset & 0x02000000){
+		offset |= 0xfc000000;
+	}
 	offset += 8;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "b%s%s 0x%"PFMT64x, link ? "l" : "", get_cond(inst), arminsn->pc+offset);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "b%s%s 0x%"PFMT64x, link ? "l" : "", get_cond (inst), arminsn->pc+offset);
 
 	arminsn->jmp = arminsn->pc+offset;
 	arminsn->fail = arminsn->pc+4;
@@ -85,14 +87,16 @@ static ut32 arm_disasm_mul(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short accu = (inst >> 21) & 0x01;
 	short condcodes = (inst >> 20) & 0x01;
 
-	if (accu)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mla%s%s %s, %s, %s, %s", get_cond(inst), condcodes ? "s" : "",
-				tbl_regs[get_nibble(inst, 4)], tbl_regs[get_nibble(inst, 0)],
-				tbl_regs[get_nibble(inst, 2)], tbl_regs[get_nibble(inst, 3)]);
-	else
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mul%s%s %s, %s, %s", get_cond(inst), condcodes ? "s" : "",
-				tbl_regs[get_nibble(inst, 4)], tbl_regs[get_nibble(inst, 0)],
-				tbl_regs[get_nibble(inst, 2)]);
+	if (accu) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mla%s%s %s, %s, %s, %s", get_cond (inst), condcodes ? "s" : "",
+				tbl_regs[get_nibble (inst, 4)], tbl_regs[get_nibble (inst, 0)],
+				tbl_regs[get_nibble (inst, 2)], tbl_regs[get_nibble (inst, 3)]);
+	}
+	else {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mul%s%s %s, %s, %s", get_cond (inst), condcodes ? "s" : "",
+				tbl_regs[get_nibble (inst, 4)], tbl_regs[get_nibble (inst, 0)],
+				tbl_regs[get_nibble (inst, 2)]);
+	}
 	return 0;
 }
 
@@ -101,36 +105,36 @@ static ut32 arm_disasm_longmul(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short accu = (inst >> 21) & 0x01;
 	short condcodes = (inst >> 20) & 0x01;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s%s%s %s, %s, %s, %s", sign ? "s" : "u", accu ? "mlal" : "mull",
-			get_cond(inst), condcodes ? "s" : "",
-			tbl_regs[get_nibble(inst, 3)], tbl_regs[get_nibble(inst, 4)],
-			tbl_regs[get_nibble(inst, 0)], tbl_regs[get_nibble(inst, 2)]);
-    return 0;
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s%s%s %s, %s, %s, %s", sign ? "s" : "u", accu ? "mlal" : "mull",
+			get_cond (inst), condcodes ? "s" : "",
+			tbl_regs[get_nibble (inst, 3)], tbl_regs[get_nibble (inst, 4)],
+			tbl_regs[get_nibble (inst, 0)], tbl_regs[get_nibble (inst, 2)]);
+	return 0;
 }
 
 static ut32 arm_disasm_swp(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short byte = (inst >> 22) & 0x01;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "swp%s%s %s, %s, [%s]", get_cond(inst), byte ? "b" : "",
-			tbl_regs[get_nibble(inst, 3)], tbl_regs[get_nibble(inst, 0)],
-			tbl_regs[get_nibble(inst, 4)]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "swp%s%s %s, %s, [%s]", get_cond (inst), byte ? "b" : "",
+			tbl_regs[get_nibble (inst, 3)], tbl_regs[get_nibble (inst, 0)],
+			tbl_regs[get_nibble (inst, 4)]);
 	return 0;
 }
 
 static ut32 arm_disasm_branchreg(struct winedbg_arm_insn *arminsn, ut32 inst) {
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "b%s %s", get_cond(inst), tbl_regs[get_nibble(inst, 0)]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "b%s %s", get_cond (inst), tbl_regs[get_nibble (inst, 0)]);
 	return 0;
 }
 
 static ut32 arm_disasm_branchxchg(struct winedbg_arm_insn *arminsn, ut32 inst) {
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "bx%s %s", get_cond(inst), tbl_regs[get_nibble(inst, 0)]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "bx%s %s", get_cond (inst), tbl_regs[get_nibble (inst, 0)]);
 	return 0;
 }
 
 static ut32 arm_disasm_mrstrans(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short src = (inst >> 22) & 0x01;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mrs%s %s, %s", get_cond(inst), tbl_regs[get_nibble(inst, 3)],
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mrs%s %s, %s", get_cond (inst), tbl_regs[get_nibble (inst, 3)],
 			src ? "spsr" : "cpsr");
 	return 0;
 }
@@ -141,26 +145,26 @@ static ut32 arm_disasm_msrtrans(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short simple = (inst >> 16) & 0x01;
 
 	if (simple || !immediate) {
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "msr%s %s, %s", get_cond(inst), dst ? "spsr" : "cpsr",
-				tbl_regs[get_nibble(inst, 0)]);
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "msr%s %s, %s", get_cond (inst), dst ? "spsr" : "cpsr",
+				tbl_regs[get_nibble (inst, 0)]);
 		return 0;
 	}
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "msr%s %s, #%u", get_cond(inst), dst ? "spsr" : "cpsr",
-			ROR32(inst & 0xff, 2 * get_nibble(inst, 2)));
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "msr%s %s, #%u", get_cond (inst), dst ? "spsr" : "cpsr",
+			ROR32 (inst & 0xff, 2 * get_nibble (inst, 2)));
 	return 0;
 }
 
 static ut32 arm_disasm_wordmov(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short top = (inst >> 22) & 0x01;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mov%s%s %s, #%u", top ? "t" : "w", get_cond(inst),
-			tbl_regs[get_nibble(inst, 3)],
-			(get_nibble(inst, 4) << 12) | (inst & 0x0fff));
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mov%s%s %s, #%u", top ? "t" : "w", get_cond (inst),
+			tbl_regs[get_nibble (inst, 3)],
+			(get_nibble (inst, 4) << 12) | (inst & 0x0fff));
 	return 0;
 }
 
 static ut32 arm_disasm_nop(struct winedbg_arm_insn *arminsn, ut32 inst) {
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "nop%s", get_cond(inst));
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "nop%s", get_cond (inst));
 	return 0;
 }
 
@@ -171,31 +175,42 @@ static ut32 arm_disasm_dataprocessing(struct winedbg_arm_insn *arminsn, ut32 ins
 	short no_op1    = (opcode & 0x0d) == 0x0d;
 	short no_dst    = (opcode & 0x0c) == 0x08;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s%s", tbl_dataops[opcode], condcodes ? "s" : "", get_cond(inst));
-	if (!no_dst) arminsn->str_asm = r_str_appendf(arminsn->str_asm, " %s, ", tbl_regs[get_nibble(inst, 3)]);
-	else arminsn->str_asm = r_str_appendf(arminsn->str_asm, " ");
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s%s", tbl_dataops[opcode], condcodes ? "s" : "", get_cond (inst));
+	if (!no_dst) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, " %s, ", tbl_regs[get_nibble (inst, 3)]);
+	}
+	else {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, " ");	
+	}
 	if (no_op1) {
-		if (immediate)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "#%u", ROR32(inst & 0xff, 2 * get_nibble(inst, 2)));
-		else
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s", tbl_regs[get_nibble(inst, 0)]);
+		if (immediate) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "#%u", ROR32 (inst & 0xff, 2 * get_nibble (inst, 2)));
+		}
+		else {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s", tbl_regs[get_nibble (inst, 0)]);
+		}
 	} else {
-		if (immediate)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, #%u", tbl_regs[get_nibble(inst, 4)],
-					ROR32(inst & 0xff, 2 * get_nibble(inst, 2)));
-		else if (((inst >> 4) & 0xff) == 0x00) /* no shift */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, %s", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)]);
-		else if (((inst >> 4) & 0x09) == 0x01) /* register shift */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, %s, %s %s", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)],
+		if (immediate) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, #%u", tbl_regs[get_nibble (inst, 4)],
+					ROR32 (inst & 0xff, 2 * get_nibble (inst, 2)));
+		}
+		else if (((inst >> 4) & 0xff) == 0x00) { /* no shift */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, %s", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)]);
+		}
+		else if (((inst >> 4) & 0x09) == 0x01) { /* register shift */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, %s, %s %s", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)],
 					tbl_shifts[(inst >> 5) & 0x03], tbl_regs[(inst >> 8) & 0x0f]);
-		else if (((inst >> 4) & 0x01) == 0x00) /* immediate shift */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, %s, %s #%d", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)], tbl_shifts[(inst >> 5) & 0x03],
+		}
+		else if (((inst >> 4) & 0x01) == 0x00) { /* immediate shift */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, %s, %s #%d", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)], tbl_shifts[(inst >> 5) & 0x03],
 					(inst >> 7) & 0x1f);
-		else
+		}
+		else {
 			return inst;
+		}
 	}
 	return 0;
 }
@@ -209,35 +224,45 @@ static ut32 arm_disasm_singletrans(struct winedbg_arm_insn *arminsn, ut32 inst)
 	short immediate = !((inst >> 25) & 0x01);
 	short offset    = inst & 0x0fff;
 
-	if (!direction) offset *= -1;
+	if (!direction) {
+		offset *= -1;
+	}
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s%s%s", load ? "ldr" : "str", byte ? "b" : "", writeback ? "t" : "",
-			get_cond(inst));
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, " %s, ", tbl_regs[get_nibble(inst, 3)]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s%s%s", load ? "ldr" : "str", byte ? "b" : "", writeback ? "t" : "",
+			get_cond (inst));
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, " %s, ", tbl_regs[get_nibble (inst, 3)]);
 	if (indexing) {
-		if (immediate)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s, #%d]", tbl_regs[get_nibble(inst, 4)], offset);
-		else if (((inst >> 4) & 0xff) == 0x00) /* no shift */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s, %s]", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)]);
-		else if (((inst >> 4) & 0x01) == 0x00) /* immediate shift (there's no register shift) */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s, %s, %s #%d]", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)], tbl_shifts[(inst >> 5) & 0x03],
+		if (immediate) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s, #%d]", tbl_regs[get_nibble (inst, 4)], offset);
+		}
+		else if (((inst >> 4) & 0xff) == 0x00) { /* no shift */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s, %s]", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)]);
+		}
+		else if (((inst >> 4) & 0x01) == 0x00) {/* immediate shift (there's no register shift) */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s, %s, %s #%d]", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)], tbl_shifts[(inst >> 5) & 0x03],
 					(inst >> 7) & 0x1f);
-		else
+		}
+		else {
 			return inst;
+		}
 	} else {
-		if (immediate)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s], #%d", tbl_regs[get_nibble(inst, 4)], offset);
-		else if (((inst >> 4) & 0xff) == 0x00) /* no shift */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s], %s", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)]);
-		else if (((inst >> 4) & 0x01) == 0x00) /* immediate shift (there's no register shift) */
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s], %s, %s #%d", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)], tbl_shifts[(inst >> 5) & 0x03],
+		if (immediate) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s], #%d", tbl_regs[get_nibble (inst, 4)], offset);
+		}
+		else if (((inst >> 4) & 0xff) == 0x00) { /* no shift */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s], %s", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)]);
+		}
+		else if (((inst >> 4) & 0x01) == 0x00) { /* immediate shift (there's no register shift) */
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s], %s, %s #%d", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)], tbl_shifts[(inst >> 5) & 0x03],
 					(inst >> 7) & 0x1f);
-		else
+		}
+		else {
 			return inst;
+		}
 	}
 	return 0;
 }
@@ -252,23 +277,29 @@ static ut32 arm_disasm_halfwordtrans(struct winedbg_arm_insn *arminsn, ut32 inst
 	short indexing  = (inst >> 24) & 0x01;
 	short offset    = ((inst >> 4) & 0xf0) + (inst & 0x0f);
 
-	if (!direction) offset *= -1;
+	if (!direction){
+		offset *= -1;
+	}
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s%s%s%s", load ? "ldr" : "str", sign ? "s" : "",
-			halfword ? "h" : (sign ? "b" : ""), writeback ? "t" : "", get_cond(inst));
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, " %s, ", tbl_regs[get_nibble(inst, 3)]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s%s%s%s", load ? "ldr" : "str", sign ? "s" : "",
+			halfword ? "h" : (sign ? "b" : ""), writeback ? "t" : "", get_cond (inst));
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, " %s, ", tbl_regs[get_nibble (inst, 3)]);
 	if (indexing) {
-		if (immediate)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s, #%d]", tbl_regs[get_nibble(inst, 4)], offset);
-		else
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s, %s]", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)]);
+		if (immediate) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s, #%d]", tbl_regs[get_nibble (inst, 4)], offset);
+		}
+		else {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s, %s]", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)]);
+		}
 	} else {
-		if (immediate)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s], #%d", tbl_regs[get_nibble(inst, 4)], offset);
-		else
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "[%s], %s", tbl_regs[get_nibble(inst, 4)],
-					tbl_regs[get_nibble(inst, 0)]);
+		if (immediate) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s], #%d", tbl_regs[get_nibble (inst, 4)], offset);
+		}
+		else {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "[%s], %s", tbl_regs[get_nibble (inst, 4)],
+					tbl_regs[get_nibble (inst, 0)]);
+		}
 	}
 	return 0;
 }
@@ -281,26 +312,28 @@ static ut32 arm_disasm_blocktrans(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	short i;
 	short last=15;
 
-	for (i=15;i>=0;i--)
+	for (i=15;i>=0;i--) {
 		if ((inst>>i) & 1) {
 			last = i;
 			break;
 		}
+	}
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s%s %s%s, {", load ? "ldm" : "stm", tbl_addrmode[addrmode],
-			get_cond(inst), tbl_regs[get_nibble(inst, 4)], writeback ? "!" : "");
-	for (i=0;i<=15;i++)
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s%s %s%s, {", load ? "ldm" : "stm", tbl_addrmode[addrmode],
+			get_cond (inst), tbl_regs[get_nibble (inst, 4)], writeback ? "!" : "");
+	for (i=0;i<=15;i++) {
 		if ((inst>>i) & 1) {
-			if (i == last) arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s", tbl_regs[i]);
-			else arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, ", tbl_regs[i]);
+			if (i == last) arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s", tbl_regs[i]);
+			else arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, ", tbl_regs[i]);
 		}
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "}%s", psr ? "^" : "");
+	}
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "}%s", psr ? "^" : "");
 	return 0;
 }
 
 static ut32 arm_disasm_swi(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	ut32 comment = inst & 0x00ffffff;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "swi%s #%d", get_cond(inst), comment);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "swi%s #%d", get_cond (inst), comment);
 	return 0;
 }
 
@@ -312,8 +345,8 @@ static ut32 arm_disasm_coproctrans(struct winedbg_arm_insn *arminsn, ut32 inst)
 	ut16 load   = (inst >> 20) & 0x01;
 	ut16 CP_Opc = (inst >> 21) & 0x07;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s %u, %u, %s, cr%u, cr%u, {%u}", load ? "mrc" : "mcr",
-			get_cond(inst), CPnum, CP, tbl_regs[get_nibble(inst, 3)], CRn, CRm, CP_Opc);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s %u, %u, %s, cr%u, cr%u, {%u}", load ? "mrc" : "mcr",
+			get_cond (inst), CPnum, CP, tbl_regs[get_nibble (inst, 3)], CRn, CRm, CP_Opc);
 	return 0;
 }
 
@@ -325,7 +358,7 @@ static ut32 arm_disasm_coprocdataop(struct winedbg_arm_insn *arminsn, ut32 inst)
 	ut16 CRn    = (inst >> 16) & 0x0f;
 	ut16 CP_Opc = (inst >> 20) & 0x0f;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "cdp%s %u, %u, cr%u, cr%u, cr%u, {%u}", get_cond(inst),
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "cdp%s %u, %u, cr%u, cr%u, cr%u, {%u}", get_cond (inst),
 			CPnum, CP, CRd, CRn, CRm, CP_Opc);
 	return 0;
 }
@@ -340,14 +373,16 @@ static ut32 arm_disasm_coprocdatatrans(struct winedbg_arm_insn *arminsn, ut32 in
 	ut16 indexing  = (inst >> 24) & 0x01;
 	short offset    = (inst & 0xff) << 2;
 
-	if (!direction) offset *= -1;
+	if (!direction) {
+		offset *= -1;
+	}
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s%s", load ? "ldc" : "stc", translen ? "l" : "", get_cond(inst));
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s%s", load ? "ldc" : "stc", translen ? "l" : "", get_cond (inst));
 	if (indexing)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, " %u, cr%u, [%s, #%d]%s", CPnum, CRd, tbl_regs[get_nibble(inst, 4)],
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, " %u, cr%u, [%s, #%d]%s", CPnum, CRd, tbl_regs[get_nibble (inst, 4)],
 				offset, writeback?"!":"");
 	else
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, " %u, cr%u, [%s], #%d", CPnum, CRd, tbl_regs[get_nibble(inst, 4)],
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, " %u, cr%u, [%s], #%d", CPnum, CRd, tbl_regs[get_nibble (inst, 4)],
 				offset);
 	return 0;
 }
@@ -359,19 +394,24 @@ static ut16 thumb_disasm_hireg(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	short h1  = (inst >> 7) & 0x01;
 	short op  = (inst >> 8) & 0x03;
 
-	if (h1) dst += 8;
-	if (h2) src += 8;
+	if (h1) {
+		dst += 8;
+	}
+	if (h2) {
+		src += 8;
+	}
 
 	if (op == 2 && dst == src) { /* mov rx, rx */
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "nop");
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "nop");
 		return 0;
 	}
 
-	if (op == 3)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "b%sx %s", h1?"l":"", tbl_regs[src]);
-	else
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, %s", tbl_hiops_t[op], tbl_regs[dst], tbl_regs[src]);
-
+	if (op == 3) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "b%sx %s", h1?"l":"", tbl_regs[src]);
+	}
+	else {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, %s", tbl_hiops_t[op], tbl_regs[dst], tbl_regs[src]);
+	}
 	return 0;
 }
 
@@ -380,7 +420,7 @@ static ut16 thumb_disasm_aluop(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	short src = (inst >> 3) & 0x07;
 	short op  = (inst >> 6) & 0x0f;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, %s", tbl_aluops_t[op], tbl_regs[dst], tbl_regs[src]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, %s", tbl_aluops_t[op], tbl_regs[dst], tbl_regs[src]);
 	return 0;
 }
 
@@ -390,21 +430,30 @@ static ut16 thumb_disasm_pushpop(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	short i;
 	short last;
 
-	for (i=7;i>=0;i--)
-		if ((inst>>i) & 1) break;
+	for (i=7;i>=0;i--) {
+		if ((inst>>i) & 1) {
+			break;
+		}
+	}
 	last = i;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s {", load ? "pop" : "push");
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s {", load ? "pop" : "push");
 
-	for (i=0;i<=7;i++)
+	for (i=0;i<=7;i++) {
 		if ((inst>>i) & 1) {
-			if (i == last) arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s", tbl_regs[i]);
-			else arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, ", tbl_regs[i]);
+			if (i == last) {
+				arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s", tbl_regs[i]);
+			}
+			else {
+				arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, ", tbl_regs[i]);
+			}
 		}
-	if (lrpc)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s", last ? ", " : "", load ? "pc" : "lr");
+	}
+	if (lrpc) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s", last ? ", " : "", load ? "pc" : "lr");
+	}
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "}");
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "}");
 	return 0;
 }
 
@@ -413,25 +462,33 @@ static ut16 thumb_disasm_blocktrans(struct winedbg_arm_insn *arminsn, ut16 inst)
 	short i;
 	short last;
 
-	for (i=7;i>=0;i--)
-		if ((inst>>i) & 1) break;
+	for (i=7;i>=0;i--) {
+		if ((inst>>i) & 1) {
+			break;
+		}
+	}
 	last = i;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s!, {", load ? "ldmia" : "stmia", tbl_regs[(inst >> 8) & 0x07]);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s!, {", load ? "ldmia" : "stmia", tbl_regs[(inst >> 8) & 0x07]);
 
-	for (i=0;i<=7;i++)
+	for (i=0;i<=7;i++) {
 		if ((inst>>i) & 1) {
-			if (i == last) arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s", tbl_regs[i]);
-			else arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, ", tbl_regs[i]);
+			if (i == last) {
+				arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s", tbl_regs[i]);
+			}
+			else {
+				arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, ", tbl_regs[i]);
+			}
 		}
+	}
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "}");
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "}");
 	return 0;
 }
 
 static ut16 thumb_disasm_condbranch(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 offset = inst & 0x00ff;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "b%s 0x%"PFMT64x, tbl_cond[(inst >> 8) & 0x0f], arminsn->pc+offset);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "b%s 0x%"PFMT64x, tbl_cond[(inst >> 8) & 0x0f], arminsn->pc+offset);
 
 	arminsn->jmp = arminsn->pc+offset;
 	arminsn->fail = arminsn->pc+4;
@@ -441,9 +498,11 @@ static ut16 thumb_disasm_condbranch(struct winedbg_arm_insn *arminsn, ut16 inst)
 static ut16 thumb_disasm_uncondbranch(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	short offset = (inst & 0x07ff) << 1;
 
-	if (offset & 0x0800) offset |= 0xf000;
+	if (offset & 0x0800) {
+		offset |= 0xf000;
+	}
 	offset += 4;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "b 0x%"PFMT64x, arminsn->pc+offset);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "b 0x%"PFMT64x, arminsn->pc+offset);
 
 	arminsn->jmp = arminsn->pc+offset;
 	return 0;
@@ -453,47 +512,49 @@ static ut16 thumb_disasm_loadadr(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 src = (inst >> 11) & 0x01;
 	ut16 offset = (inst & 0xff) << 2;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "add %s, %s, #%d", tbl_regs[(inst >> 8) & 0x07], src ? "sp" : "pc",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "add %s, %s, #%d", tbl_regs[(inst >> 8) & 0x07], src ? "sp" : "pc",
 			offset);
 	return 0;
 }
 
 static ut16 thumb_disasm_swi(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 comment = inst & 0x00ff;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "swi #%d", comment);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "swi #%d", comment);
 	return 0;
 }
 
 static ut16 thumb_disasm_nop(struct winedbg_arm_insn *arminsn, ut16 inst) {
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "nop");
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "nop");
 	return 0;
 }
 
 static ut16 thumb_disasm_ldrpcrel(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 offset = (inst & 0xff) << 2;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "ldr %s, [pc, #%u]", tbl_regs[(inst >> 8) & 0x07], offset);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "ldr %s, [pc, #%u]", tbl_regs[(inst >> 8) & 0x07], offset);
 	return 0;
 }
 
 static ut16 thumb_disasm_ldrsprel(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 offset = (inst & 0xff) << 2;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, [sp, #%u]", (inst & 0x0800)?"ldr":"str",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, [sp, #%u]", (inst & 0x0800)?"ldr":"str",
 			tbl_regs[(inst >> 8) & 0x07], offset);
 	return 0;
 }
 
 static ut16 thumb_disasm_addsprel(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 offset = (inst & 0x7f) << 2;
-	if ((inst >> 7) & 0x01)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "sub sp, sp, #%u", offset);
-	else
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "add sp, sp, #%u", offset);
+	if ((inst >> 7) & 0x01) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "sub sp, sp, #%u", offset);
+	}
+	else {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "add sp, sp, #%u", offset);
+	}
 	return 0;
 }
 
 static ut16 thumb_disasm_ldrimm(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 offset = (inst & 0x07c0) >> 6;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s %s, [%s, #%u]",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s %s, [%s, #%u]",
 			(inst & 0x0800)?"ldr":"str", (inst & 0x1000)?"b":"",
 			tbl_regs[inst & 0x07], tbl_regs[(inst >> 3) & 0x07],
 			(inst & 0x1000)?offset:(offset << 2));
@@ -502,13 +563,13 @@ static ut16 thumb_disasm_ldrimm(struct winedbg_arm_insn *arminsn, ut16 inst) {
 
 static ut16 thumb_disasm_ldrhimm(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 offset = (inst & 0x07c0) >> 5;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, [%s, #%u]", (inst & 0x0800)?"ldrh":"strh",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, [%s, #%u]", (inst & 0x0800)?"ldrh":"strh",
 			tbl_regs[inst & 0x07], tbl_regs[(inst >> 3) & 0x07], offset);
 	return 0;
 }
 
 static ut16 thumb_disasm_ldrreg(struct winedbg_arm_insn *arminsn, ut16 inst) {
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s%s %s, [%s, %s]",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s%s %s, [%s, %s]",
 			(inst & 0x0800)?"ldr":"str", (inst & 0x0400)?"b":"",
 			tbl_regs[inst & 0x07], tbl_regs[(inst >> 3) & 0x07],
 			tbl_regs[(inst >> 6) & 0x07]);
@@ -516,7 +577,7 @@ static ut16 thumb_disasm_ldrreg(struct winedbg_arm_insn *arminsn, ut16 inst) {
 }
 
 static ut16 thumb_disasm_ldrsreg(struct winedbg_arm_insn *arminsn, ut16 inst) {
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, [%s, %s]",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, [%s, %s]",
 			tbl_sregops_t[(inst >> 10) & 0x03], tbl_regs[inst & 0x07],
 			tbl_regs[(inst >> 3) & 0x07], tbl_regs[(inst >> 6) & 0x07]);
 	return 0;
@@ -524,7 +585,7 @@ static ut16 thumb_disasm_ldrsreg(struct winedbg_arm_insn *arminsn, ut16 inst) {
 
 static ut16 thumb_disasm_immop(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 op = (inst >> 11) & 0x03;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, #%u", tbl_immops_t[op], tbl_regs[(inst >> 8) & 0x07],
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, #%u", tbl_immops_t[op], tbl_regs[(inst >> 8) & 0x07],
 			inst & 0xff);
 	return 0;
 }
@@ -533,25 +594,27 @@ static ut16 thumb_disasm_addsub(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 op = (inst >> 9) & 0x01;
 	ut16 immediate = (inst >> 10) & 0x01;
 
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, %s, ", op ? "sub" : "add",
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, %s, ", op ? "sub" : "add",
 			tbl_regs[inst & 0x07], tbl_regs[(inst >> 3) & 0x07]);
-	if (immediate)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "#%d", (inst >> 6) & 0x07);
-	else
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s", tbl_regs[(inst >> 6) & 0x07]);
+	if (immediate) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "#%d", (inst >> 6) & 0x07);
+	}
+	else {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s", tbl_regs[(inst >> 6) & 0x07]);
+	}
 	return 0;
 }
 
 static ut16 thumb_disasm_movshift(struct winedbg_arm_insn *arminsn, ut16 inst) {
 	ut16 op = (inst >> 11) & 0x03;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s %s, %s, #%u", tbl_shifts[op],
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s %s, %s, #%u", tbl_shifts[op],
 			tbl_regs[inst & 0x07], tbl_regs[(inst >> 3) & 0x07], (inst >> 6) & 0x1f);
 	return 0;
 }
 
 static ut32 thumb2_disasm_branchlinked(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	ut32 offset = (((inst & 0x07ff0000) >> 4) | ((inst & 0x000007ff) << 1)) + 4;
-	arminsn->str_asm = r_str_appendf(arminsn->str_asm, "bl 0x%"PFMT64x, arminsn->pc+offset);
+	arminsn->str_asm = r_str_appendf (arminsn->str_asm, "bl 0x%"PFMT64x, arminsn->pc+offset);
 
 	arminsn->jmp = arminsn->pc+offset;
 	return 0;
@@ -561,31 +624,32 @@ static ut32 thumb2_disasm_misc(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	ut16 op1 = (inst >> 20) & 0x03;
 	ut16 op2 = (inst >> 4) & 0x03;
 
-	if (get_nibble(inst, 4) != get_nibble(inst, 0))
+	if (get_nibble (inst, 4) != get_nibble (inst, 0)) {
 		return inst;
+	}
 
 	if (op1 == 3 && op2 == 0) {
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "clz %s, %s ", tbl_regs[get_nibble(inst, 2)],
-				tbl_regs[get_nibble(inst, 0)]);
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "clz %s, %s ", tbl_regs[get_nibble (inst, 2)],
+				tbl_regs[get_nibble (inst, 0)]);
 		return 0;
 	}
 
 	if (op1 == 1) {
 		switch (op2) {
 		case 0:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "rev ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "rev ");
 			break;
 		case 1:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "rev16 ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "rev16 ");
 			break;
 		case 2:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "rbit ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "rbit ");
 			break;
 		case 3:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "revsh ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "revsh ");
 			break;
 		}
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, %s ", tbl_regs[get_nibble(inst, 2)], tbl_regs[get_nibble(inst, 0)]);
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, %s ", tbl_regs[get_nibble (inst, 2)], tbl_regs[get_nibble (inst, 0)]);
 		return 0;
 	}
 
@@ -596,29 +660,30 @@ static ut32 thumb2_disasm_mul(struct winedbg_arm_insn *arminsn, ut32 inst) {
 	ut16 op1 = (inst >> 20) & 0x07;
 	ut16 op2 = (inst >> 4) & 0x03;
 
-	if (op1)
+	if (op1) {
 		return inst;
+	}
 
-	if (op2 == 0 && get_nibble(inst, 3) != 0xf) {
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mla %s, %s, %s, %s ", tbl_regs[get_nibble(inst, 2)],
-				tbl_regs[get_nibble(inst, 4)],
-				tbl_regs[get_nibble(inst, 0)],
-				tbl_regs[get_nibble(inst, 3)]);
+	if (op2 == 0 && get_nibble (inst, 3) != 0xf) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mla %s, %s, %s, %s ", tbl_regs[get_nibble (inst, 2)],
+				tbl_regs[get_nibble (inst, 4)],
+				tbl_regs[get_nibble (inst, 0)],
+				tbl_regs[get_nibble (inst, 3)]);
 		return 0;
 	}
 
-	if (op2 == 0 && get_nibble(inst, 3) == 0xf) {
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mul %s, %s, %s ", tbl_regs[get_nibble(inst, 2)],
-				tbl_regs[get_nibble(inst, 4)],
-				tbl_regs[get_nibble(inst, 0)]);
+	if (op2 == 0 && get_nibble (inst, 3) == 0xf) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mul %s, %s, %s ", tbl_regs[get_nibble (inst, 2)],
+				tbl_regs[get_nibble (inst, 4)],
+				tbl_regs[get_nibble (inst, 0)]);
 		return 0;
 	}
 
 	if (op2 == 1) {
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "mls %s, %s, %s, %s ", tbl_regs[get_nibble(inst, 2)],
-				tbl_regs[get_nibble(inst, 4)],
-				tbl_regs[get_nibble(inst, 0)],
-				tbl_regs[get_nibble(inst, 3)]);
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "mls %s, %s, %s, %s ", tbl_regs[get_nibble (inst, 2)],
+				tbl_regs[get_nibble (inst, 4)],
+				tbl_regs[get_nibble (inst, 0)],
+				tbl_regs[get_nibble (inst, 3)]);
 		return 0;
 	}
 
@@ -632,39 +697,39 @@ static ut32 thumb2_disasm_longmuldiv(struct winedbg_arm_insn *arminsn, ut32 inst
 	if (op2 == 0) {
 		switch (op1) {
 		case 0:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "smull ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "smull ");
 			break;
 		case 2:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "umull ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "umull ");
 			break;
 		case 4:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "smlal ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "smlal ");
 			break;
 		case 6:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "umlal ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "umlal ");
 			break;
 		default:
 			return inst;
 		}
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, %s, %s, %s ",
-				tbl_regs[get_nibble(inst, 3)], tbl_regs[get_nibble(inst, 2)],
-				tbl_regs[get_nibble(inst, 4)], tbl_regs[get_nibble(inst, 0)]);
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, %s, %s, %s ",
+				tbl_regs[get_nibble (inst, 3)], tbl_regs[get_nibble (inst, 2)],
+				tbl_regs[get_nibble (inst, 4)], tbl_regs[get_nibble (inst, 0)]);
 		return 0;
 	}
 
 	if (op2 == 0xffff) {
 		switch (op1) {
 		case 1:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "sdiv ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "sdiv ");
 			break;
 		case 3:
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "udiv ");
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "udiv ");
 			break;
 		default:
 			return inst;
 		}
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm, "%s, %s, %s ", tbl_regs[get_nibble(inst, 2)], tbl_regs[get_nibble(inst, 4)],
-				tbl_regs[get_nibble(inst, 0)]);
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm, "%s, %s, %s ", tbl_regs[get_nibble (inst, 2)], tbl_regs[get_nibble (inst, 4)],
+				tbl_regs[get_nibble (inst, 0)]);
 		return 0;
 	}
 
@@ -675,16 +740,18 @@ static ut32 thumb2_disasm_coprocmov1(struct winedbg_arm_insn *arminsn, ut32 inst
 	ut16 opc1 = (inst >> 21) & 0x07;
 	ut16 opc2 = (inst >> 5) & 0x07;
 
-	if (opc2)
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm,
+	if (opc2) {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm,
 				"%s%s\tp%u, #%u, %s, cr%u, cr%u, #%u", (inst & 0x00100000)?"mrc":"mcr",
-				(inst & 0x10000000)?"2":"", get_nibble(inst, 2), opc1,
-				tbl_regs[get_nibble(inst, 3)], get_nibble(inst, 4), get_nibble(inst, 0), opc2);
-	else
-		arminsn->str_asm = r_str_appendf(arminsn->str_asm,
+				(inst & 0x10000000)?"2":"", get_nibble (inst, 2), opc1,
+				tbl_regs[get_nibble (inst, 3)], get_nibble (inst, 4), get_nibble (inst, 0), opc2);
+	}
+	else {
+		arminsn->str_asm = r_str_appendf (arminsn->str_asm,
 				"%s%s\tp%u, #%u, %s, cr%u, cr%u", (inst & 0x00100000)?"mrc":"mcr",
-				(inst & 0x10000000)?"2":"", get_nibble(inst, 2), opc1,
-				tbl_regs[get_nibble(inst, 3)], get_nibble(inst, 4), get_nibble(inst, 0));
+				(inst & 0x10000000)?"2":"", get_nibble (inst, 2), opc1,
+				tbl_regs[get_nibble (inst, 3)], get_nibble (inst, 4), get_nibble (inst, 0));
+	}
 	return 0;
 }
 
@@ -814,10 +881,13 @@ int arm_disasm_one_insn(struct winedbg_arm_insn *arminsn) {
 			a_ptr++;
 		}
 
-		if (!matched)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "Unknown ARM Instruction: %08x", inst);
-		else a_ptr->func(arminsn, inst);
-		arminsn->str_hex = r_str_appendf(arminsn->str_hex, "%02x%02x%02x%02x",
+		if (!matched) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "Unknown ARM Instruction: %08x", inst);
+		}
+		else {
+			a_ptr->func(arminsn, inst);
+		}
+		arminsn->str_hex = r_str_appendf (arminsn->str_hex, "%02x%02x%02x%02x",
 				*((ut8*)(&inst)), *((ut8*)(&inst)+1),  *((ut8*)(&inst)+2),  *((ut8*)(&inst)+3));
 		return size;
 	} else {
@@ -838,10 +908,13 @@ int arm_disasm_one_insn(struct winedbg_arm_insn *arminsn) {
 				t2_ptr++;
 			}
 
-			if (!matched)
-				arminsn->str_asm = r_str_appendf(arminsn->str_asm, "Unknown Thumb2 Instruction: %08x", inst);
-			else t2_ptr->func(arminsn, inst);
-			arminsn->str_hex = r_str_appendf(arminsn->str_hex, "%02x%02x%02x%02x",
+			if (!matched) {
+				arminsn->str_asm = r_str_appendf (arminsn->str_asm, "Unknown Thumb2 Instruction: %08x", inst);
+			}
+			else {
+				t2_ptr->func(arminsn, inst);
+			} 
+			arminsn->str_hex = r_str_appendf (arminsn->str_hex, "%02x%02x%02x%02x",
 					*((ut8*)(&inst)), *((ut8*)(&inst)+1),  *((ut8*)(&inst)+2),  *((ut8*)(&inst)+3));
 			return size;
 		default:
@@ -857,10 +930,13 @@ int arm_disasm_one_insn(struct winedbg_arm_insn *arminsn) {
 			t_ptr++;
 		}
 
-		if (!matched)
-			arminsn->str_asm = r_str_appendf(arminsn->str_asm, "Unknown Thumb Instruction: %04x", tinst);
-		else t_ptr->func(arminsn, tinst);
-		arminsn->str_hex = r_str_appendf(arminsn->str_hex, "%02x%02x", *((ut8*)(&tinst)), *((ut8*)(&tinst)+1));
+		if (!matched) {
+			arminsn->str_asm = r_str_appendf (arminsn->str_asm, "Unknown Thumb Instruction: %04x", tinst);
+		}
+		else {
+			t_ptr->func(arminsn, tinst);
+		}
+		arminsn->str_hex = r_str_appendf (arminsn->str_hex, "%02x%02x", *((ut8*)(&tinst)), *((ut8*)(&tinst)+1));
 		return size;
 	}
 	return size;
