@@ -496,7 +496,7 @@ static void set_e_fields(vle_t * v, const e_vle_t* p, ut32 data) {
 			v->fields[0].type = p->types[0];
 			v->fields[1].value = (data & 0x1F0000) >> 5;
 			v->fields[1].value |= (data & 0x3FF);
-			if (v->fields[1].value & 0x4000) {
+			if (v->fields[1].value & 0x8000) {
 				v->fields[1].value = 0xFFF8000 | v->fields[1].value;
 			}
 			v->fields[1].type = p->types[1];
@@ -806,7 +806,7 @@ int vle_option(vle_handle* handle, ut32 option) {
 
 vle_t* vle_next(vle_handle* handle) {
 	vle_t *op = NULL;
-	if (!handle || handle->pos + 2 >= handle->end) {
+	if (!handle || handle->pos + handle->inc >= handle->end) {
 		return NULL;
 	}
 	handle->pos += handle->inc;
