@@ -9,6 +9,7 @@ R_API RAsmCode *r_asm_code_new(void) {
 
 R_API void* r_asm_code_free(RAsmCode *acode) {
 	if (acode) {
+		r_list_free (acode->equs);
 		free (acode->buf);
 		free (acode->buf_hex);
 		free (acode->buf_asm);
@@ -17,6 +18,12 @@ R_API void* r_asm_code_free(RAsmCode *acode) {
 	return NULL;
 }
 
+R_API void r_asm_equ_item_free(RAsmEqu *equ) {
+	free (equ->key);
+	free (equ->value);
+	free (equ);
+}
+
 R_API bool r_asm_code_set_equ (RAsmCode *code, const char *key, const char *value) {
 	RAsmEqu *equ;
 	RListIter *iter;
@@ -25,8 +32,7 @@ R_API bool r_asm_code_set_equ (RAsmCode *code, const char *key, const char *valu
 		return false;
 	}
 	if (!code->equs) {
-		code->equs = r_list_new ();
-		code->equs->free = free;
+		code->equs = r_list_newf ((RListFree)r_asm_equ_item_free);
 	} else {
 		r_list_foreach (code->equs, iter, equ) {
 			if (!strcmp (equ->key, key)) {
