@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2012-2013 pancake */
+/* radare2 - LGPL - Copyright 2012-2015 pancake */
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -8,14 +8,15 @@
 #include <r_lib.h>
 #include <r_asm.h>
 #include "../arch/dcpu16/dcpu16.h"
+#include "../arch/dcpu16/dis.c"
+#include "../arch/dcpu16/asm.c"
 
-// ut64 for length here is overkill!
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	if (len<2) return -1; // at least 2 bytes!
-	op->inst_len = dcpu16_disasm (op->buf_asm, (const ut16*)buf, len, NULL);
-	if (op->inst_len == -1)
+	op->size = dcpu16_disasm (op->buf_asm, (const ut16*)buf, len, NULL);
+	if (op->size == -1)
 		strcpy (op->buf_asm, " (data)");
-	return op->inst_len;
+	return op->size;
 }
 
 static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
@@ -25,17 +26,18 @@ static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 RAsmPlugin r_asm_plugin_dcpu16 = {
 	.name = "dcpu16",
 	.arch = "dpcu",
-	.bits = (int[]){ 16, 0 },
-	.desc = "DCPU16 assembler/disassembler",
-	.init = NULL,
-	.fini = NULL,
+	.bits = 16,
+	.endian = R_SYS_ENDIAN_LITTLE,
+	.desc = "Mojang's DCPU-16",
+	.license = "PD",
 	.disassemble = &disassemble,
 	.assemble = &assemble 
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
-	.data = &r_asm_plugin_dcpu16
+	.data = &r_asm_plugin_dcpu16,
+	.version = R2_VERSION
 };
 #endif
