@@ -32,7 +32,7 @@ RAsmPlugin r_asm_plugin_gb = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_gb,
 	.version = R2_VERSION
