@@ -30,7 +30,7 @@ RAsmPlugin r_asm_plugin_h8300 = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_h8300,
 	.version = R2_VERSION
