@@ -21,7 +21,7 @@ RAsmPlugin r_asm_plugin_i4004 = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_i4004,
 	.version = R2_VERSION
