@@ -25,7 +25,7 @@ RAsmPlugin r_asm_plugin_i8080 = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_i8080,
 	.version = R2_VERSION
