@@ -95,7 +95,7 @@ RAsmPlugin r_asm_plugin_sparc_gnu = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_sparc_gnu,
 	.version = R2_VERSION
