@@ -27,7 +27,7 @@ RAsmPlugin r_asm_plugin_spc700 = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_spc700,
 	.version = R2_VERSION
