@@ -63,7 +63,7 @@ RAsmPlugin r_asm_plugin_sysz = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_sysz,
 	.version = R2_VERSION
