@@ -33,7 +33,7 @@ RAsmPlugin r_asm_plugin_v810 = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_v810,
 	.version = R2_VERSION
