@@ -70,11 +70,10 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	} else {
 		cs_option (cd, CS_OPT_SYNTAX, CS_OPT_SYNTAX_INTEL);
 	}
-	if (op) {
-		op->size = 1;
-	} else {
+	if (!op) {
 		return true;
 	}
+	op->size = 1;
 	cs_insn *insn = NULL;
 #if USE_ITER_API
 	{
@@ -102,7 +101,7 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			strcpy (op->buf_asm, "illegal");
 		}
 	}
-	if (op->size==0 && n>0 && insn->size>0) {
+	if (op->size == 0 && n > 0 && insn->size > 0) {
 		char *ptrstr;
 		op->size = insn->size;
 		snprintf (op->buf_asm, R_ASM_BUFSIZE, "%s%s%s",
@@ -120,6 +119,26 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			memcpy (op->buf_asm, "jnz", 3);
 		}
 	}
+#if 0
+	// [eax + ebx*4]  =>  [eax + ebx * 4]
+	char *ast = strchr (op->buf_asm, '*');
+	if (ast && ast > op->buf_asm) {
+		ast--;
+		if (ast[0] != ' ') {
+			char *tmp = strdup (ast + 1);
+			if (tmp) {
+				ast[0] = ' ';
+				if (tmp[0] && tmp[1] && tmp[1] != ' ') {
+					strcpy (ast, " * ");
+					strcpy (ast + 3, tmp + 1);
+				} else {
+					strcpy (ast + 1, tmp);
+				}
+				free (tmp);
+			}
+		}
+	}
+#endif
 #if USE_ITER_API
 	/* do nothing because it should be allocated once and freed in the_end */
 #else
