@@ -23,8 +23,46 @@ R_API char *r_bin_addr2text(RBin *bin, ut64 addr, int origin) {
 	char file[4096];
 	int line;
 	char *out = NULL, *out2 = NULL;
-	char *file_nopath;
-
+	char *file_nopath = NULL;
+	if (!bin || !bin->cur) {
+		return NULL;
+	}
+	{
+		char *key = r_str_newf ("0x%"PFMT64x, addr);
+		char *file_line = sdb_get (bin->cur->sdb_addrinfo, key, 0);
+		if (file_line) {
+			char *token = strchr (file_line, '|');
+			if (token) {
+				*token ++ = 0;
+				line = atoi (token);
+				out = r_file_slurp_line (file_line, line, 0);
+				*token ++ = ':';
+			}
+		}
+		free (key);
+		if (out) {
+			if (origin > 1) {
+				file_nopath = file_line;
+			} else {
+				file_nopath = strrchr (file_line, '/');
+				if (file_nopath) {
+					file_nopath ++;
+				} else {
+					file_nopath = file_line;
+				}
+			}
+			if (origin) {
+				char *res = r_str_newf ("%s:%d%s%s",
+						file_nopath? file_nopath: "",
+						line, file_nopath? " ": "",
+						out? out: "");
+				free (out);
+				return res;
+			}
+			return out;
+		}
+		free (file_line);
+	}
 	file[0] = 0;
 	if (r_bin_addr2line (bin, addr, file, sizeof (file), &line)) {
 		if (bin->srcdir && *bin->srcdir) {
@@ -36,7 +74,7 @@ R_API char *r_bin_addr2text(RBin *bin, ut64 addr, int origin) {
 		// TODO: this is slow. must use a cached pool of mmaped files and line:off entries
 		out = r_file_slurp_line (file, line, 0);
 		if (!out) {
-			return 0;
+			return r_str_newf ("%s:%d", file, line);
 		}
 		out2 = malloc ((strlen (file) + 64 + strlen (out)) * sizeof (char));
 		if (origin > 1) {
