@@ -22,10 +22,11 @@ struct r_bin_coff_obj {
 	size_t size;
 	ut8 endian;
 	Sdb *kv;
+	bool verbose;
 };
 
 bool r_coff_supported_arch(const ut8 *buf); /* Reads two bytes from buf. */
-struct r_bin_coff_obj* r_bin_coff_new_buf(struct r_buf_t *buf);
+struct r_bin_coff_obj* r_bin_coff_new_buf(RBuffer *buf, bool verbose);
 void r_bin_coff_free(struct r_bin_coff_obj *obj);
 RBinAddr *r_coff_get_entry(struct r_bin_coff_obj *obj);
 char *r_coff_symbol_name (struct r_bin_coff_obj *obj, void *ptr);
