@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008 nibble<.ds@gmail.com> */
+/* radare - LGPL - Copyright 2008 nibble */
 
 #undef PE_
 #undef ILT_MASK1
@@ -160,100 +160,100 @@ typedef struct {
 
 //language
 
-#define LANG_NEUTRAL       0x00
-#define LANG_INVARIANT     0x7f
-#define LANG_AFRIKAANS     0x36
-#define LANG_ALBANIAN      0x1c
-#define LANG_ARABIC        0x01
-#define LANG_ARMENIAN      0x2b
-#define LANG_ASSAMESE      0x4d
-#define LANG_AZERI         0x2c
-#define LANG_BASQUE        0x2d
-#define LANG_BELARUSIAN    0x23
-#define LANG_BENGALI       0x45
-#define LANG_BULGARIAN     0x02
-#define LANG_CATALAN       0x03
-#define LANG_CHINESE       0x04
-#define LANG_CROATIAN      0x1a
-#define LANG_CZECH         0x05
-#define LANG_DANISH        0x06
-#define LANG_DIVEHI        0x65
-#define LANG_DUTCH         0x13
-#define LANG_ENGLISH       0x09
-#define LANG_ESTONIAN      0x25
-#define LANG_FAEROESE      0x38
-#define LANG_FARSI         0x29
-#define LANG_FINNISH       0x0b
-#define LANG_FRENCH        0x0c
-#define LANG_GALICIAN      0x56
-#define LANG_GEORGIAN      0x37
-#define LANG_GERMAN        0x07
-#define LANG_GREEK         0x08
-#define LANG_GUJARATI      0x47
-#define LANG_HEBREW        0x0d
-#define LANG_HINDI         0x39
-#define LANG_HUNGARIAN     0x0e
-#define LANG_ICELANDIC     0x0f
-#define LANG_INDONESIAN    0x21
-#define LANG_ITALIAN       0x10
-#define LANG_JAPANESE      0x11
-#define LANG_KANNADA       0x4b
-#define LANG_KASHMIRI      0x60
-#define LANG_KAZAK         0x3f
-#define LANG_KONKANI       0x57
-#define LANG_KOREAN        0x12
-#define LANG_KYRGYZ        0x40
-#define LANG_LATVIAN       0x26
-#define LANG_LITHUANIAN    0x27
-#define LANG_MACEDONIAN    0x2f
-#define LANG_MALAY         0x3e
-#define LANG_MALAYALAM     0x4c
-#define LANG_MANIPURI      0x58
-#define LANG_MARATHI       0x4e
-#define LANG_MONGOLIAN     0x50
-#define LANG_NEPALI        0x61
-#define LANG_NORWEGIAN     0x14
-#define LANG_ORIYA         0x48
-#define LANG_POLISH        0x15
-#define LANG_PORTUGUESE    0x16
-#define LANG_PUNJABI       0x46
-#define LANG_ROMANIAN      0x18
-#define LANG_RUSSIAN       0x19
-#define LANG_SANSKRIT      0x4f
-#define LANG_SERBIAN       0x1a
-#define LANG_SINDHI        0x59
-#define LANG_SLOVAK        0x1b
-#define LANG_SLOVENIAN     0x24
-#define LANG_SPANISH       0x0a
-#define LANG_SWAHILI       0x41
-#define LANG_SWEDISH       0x1d
-#define LANG_SYRIAC        0x5a
-#define LANG_TAMIL         0x49
-#define LANG_TATAR         0x44
-#define LANG_TELUGU        0x4a
-#define LANG_THAI          0x1e
-#define LANG_TURKISH       0x1f
-#define LANG_UKRAINIAN     0x22
-#define LANG_URDU          0x20
-#define LANG_UZBEK         0x43
-#define LANG_VIETNAMESE    0x2a
-#define LANG_GAELIC        0x3c
-#define LANG_MALTESE       0x3a
-#define LANG_MAORI         0x28
-#define LANG_RHAETO_ROMANCE 0x17
-#define LANG_SAAMI         0x3b
-#define LANG_SORBIAN       0x2e
-#define LANG_SUTU          0x30
-#define LANG_TSONGA        0x31
-#define LANG_TSWANA        0x32
-#define LANG_VENDA         0x33
-#define LANG_XHOSA         0x34
-#define LANG_ZULU          0x35
-#define LANG_ESPERANTO     0x8f
-#define LANG_WALON         0x90
-#define LANG_CORNISH       0x91
-#define LANG_WELSH         0x92
-#define LANG_BRETON        0x93
+#define PE_LANG_NEUTRAL       0x00
+#define PE_LANG_INVARIANT     0x7f
+#define PE_LANG_AFRIKAANS     0x36
+#define PE_LANG_ALBANIAN      0x1c
+#define PE_LANG_ARABIC        0x01
+#define PE_LANG_ARMENIAN      0x2b
+#define PE_LANG_ASSAMESE      0x4d
+#define PE_LANG_AZERI         0x2c
+#define PE_LANG_BASQUE        0x2d
+#define PE_LANG_BELARUSIAN    0x23
+#define PE_LANG_BENGALI       0x45
+#define PE_LANG_BULGARIAN     0x02
+#define PE_LANG_CATALAN       0x03
+#define PE_LANG_CHINESE       0x04
+#define PE_LANG_CROATIAN      0x1a
+#define PE_LANG_CZECH         0x05
+#define PE_LANG_DANISH        0x06
+#define PE_LANG_DIVEHI        0x65
+#define PE_LANG_DUTCH         0x13
+#define PE_LANG_ENGLISH       0x09
+#define PE_LANG_ESTONIAN      0x25
+#define PE_LANG_FAEROESE      0x38
+#define PE_LANG_FARSI         0x29
+#define PE_LANG_FINNISH       0x0b
+#define PE_LANG_FRENCH        0x0c
+#define PE_LANG_GALICIAN      0x56
+#define PE_LANG_GEORGIAN      0x37
+#define PE_LANG_GERMAN        0x07
+#define PE_LANG_GREEK         0x08
+#define PE_LANG_GUJARATI      0x47
+#define PE_LANG_HEBREW        0x0d
+#define PE_LANG_HINDI         0x39
+#define PE_LANG_HUNGARIAN     0x0e
+#define PE_LANG_ICELANDIC     0x0f
+#define PE_LANG_INDONESIAN    0x21
+#define PE_LANG_ITALIAN       0x10
+#define PE_LANG_JAPANESE      0x11
+#define PE_LANG_KANNADA       0x4b
+#define PE_LANG_KASHMIRI      0x60
+#define PE_LANG_KAZAK         0x3f
+#define PE_LANG_KONKANI       0x57
+#define PE_LANG_KOREAN        0x12
+#define PE_LANG_KYRGYZ        0x40
+#define PE_LANG_LATVIAN       0x26
+#define PE_LANG_LITHUANIAN    0x27
+#define PE_LANG_MACEDONIAN    0x2f
+#define PE_LANG_MALAY         0x3e
+#define PE_LANG_MALAYALAM     0x4c
+#define PE_LANG_MANIPURI      0x58
+#define PE_LANG_MARATHI       0x4e
+#define PE_LANG_MONGOLIAN     0x50
+#define PE_LANG_NEPALI        0x61
+#define PE_LANG_NORWEGIAN     0x14
+#define PE_LANG_ORIYA         0x48
+#define PE_LANG_POLISH        0x15
+#define PE_LANG_PORTUGUESE    0x16
+#define PE_LANG_PUNJABI       0x46
+#define PE_LANG_ROMANIAN      0x18
+#define PE_LANG_RUSSIAN       0x19
+#define PE_LANG_SANSKRIT      0x4f
+#define PE_LANG_SERBIAN       0x1a
+#define PE_LANG_SINDHI        0x59
+#define PE_LANG_SLOVAK        0x1b
+#define PE_LANG_SLOVENIAN     0x24
+#define PE_LANG_SPANISH       0x0a
+#define PE_LANG_SWAHILI       0x41
+#define PE_LANG_SWEDISH       0x1d
+#define PE_LANG_SYRIAC        0x5a
+#define PE_LANG_TAMIL         0x49
+#define PE_LANG_TATAR         0x44
+#define PE_LANG_TELUGU        0x4a
+#define PE_LANG_THAI          0x1e
+#define PE_LANG_TURKISH       0x1f
+#define PE_LANG_UKRAINIAN     0x22
+#define PE_LANG_URDU          0x20
+#define PE_LANG_UZBEK         0x43
+#define PE_LANG_VIETNAMESE    0x2a
+#define PE_LANG_GAELIC        0x3c
+#define PE_LANG_MALTESE       0x3a
+#define PE_LANG_MAORI         0x28
+#define PE_LANG_RHAETO_ROMANCE 0x17
+#define PE_LANG_SAAMI         0x3b
+#define PE_LANG_SORBIAN       0x2e
+#define PE_LANG_SUTU          0x30
+#define PE_LANG_TSONGA        0x31
+#define PE_LANG_TSWANA        0x32
+#define PE_LANG_VENDA         0x33
+#define PE_LANG_XHOSA         0x34
+#define PE_LANG_ZULU          0x35
+#define PE_LANG_ESPERANTO     0x8f
+#define PE_LANG_WALON         0x90
+#define PE_LANG_CORNISH       0x91
+#define PE_LANG_WELSH         0x92
+#define PE_LANG_BRETON        0x93
 
 typedef struct {
 	ut32 VirtualAddress;
