@@ -21,40 +21,40 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return 0;
 }
 
-static void *load_bytes(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
+static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
 	if (!check_bytes (buf, sz)) {
 		return NULL;
 	}
 	return R_NOTNULL;
 }
 
-static bool load(RBinFile *arch) {
-	const ut8 *bytes = arch? r_buf_buffer (arch->buf): NULL;
-	ut64 sz = arch? r_buf_size (arch->buf): 0;
+static bool load(RBinFile *bf) {
+	const ut8 *bytes = bf? r_buf_buffer (bf->buf): NULL;
+	ut64 sz = bf? r_buf_size (bf->buf): 0;
 	return check_bytes (bytes, sz);
 }
 
-static int destroy(RBinFile *arch) {
-	// r_bin_bios_free ((struct r_bin_bios_obj_t*)arch->o->bin_obj);
+static int destroy(RBinFile *bf) {
+	// r_bin_bios_free ((struct r_bin_bios_obj_t*)bf->o->bin_obj);
 	return true;
 }
 
-static ut64 baddr(RBinFile *arch) {
+static ut64 baddr(RBinFile *bf) {
 	return 0;
 }
 
 /* accelerate binary load */
-static RList *strings(RBinFile *arch) {
+static RList *strings(RBinFile *bf) {
 	return NULL;
 }
 
-static RBinInfo *info(RBinFile *arch) {
+static RBinInfo *info(RBinFile *bf) {
 	RBinInfo *ret = NULL;
 	if (!(ret = R_NEW0 (RBinInfo))) {
 		return NULL;
 	}
 	ret->lang = NULL;
-	ret->file = arch->file? strdup (arch->file): NULL;
+	ret->file = bf->file? strdup (bf->file): NULL;
 	ret->type = strdup ("bios");
 	ret->bclass = strdup ("1.0");
 	ret->rclass = strdup ("bios");
@@ -69,7 +69,7 @@ static RBinInfo *info(RBinFile *arch) {
 	return ret;
 }
 
-static RList *sections(RBinFile *arch) {
+static RList *sections(RBinFile *bf) {
 	RList *ret = NULL;
 	RBinSection *ptr = NULL;
 
@@ -82,7 +82,7 @@ static RList *sections(RBinFile *arch) {
 	}
 	strcpy (ptr->name, "bootblk");
 	ptr->vsize = ptr->size = 0x10000;
-	ptr->paddr = arch->buf->length - ptr->size;
+	ptr->paddr = bf->buf->length - ptr->size;
 	ptr->vaddr = 0xf0000;
 	ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE |
 	            R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
@@ -91,7 +91,7 @@ static RList *sections(RBinFile *arch) {
 	return ret;
 }
 
-static RList *entries(RBinFile *arch) {
+static RList *entries(RBinFile *bf) {
 	RList *ret;
 	RBinAddr *ptr = NULL;
 	if (!(ret = r_list_new ())) {
@@ -123,7 +123,7 @@ RBinPlugin r_bin_plugin_bios = {
 };
 
 #ifndef CORELIB
-RlibStruct radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_bios,
 	.version = R2_VERSION
