@@ -25,6 +25,7 @@
 #define R_DEX_METH_MIRANDA 0x8000
 #define R_DEX_METH_CONSTRUCTOR 0x10000
 #define R_DEX_METH_DECLARED_SYNCHRONIZED 0x20000
+#define r_str_const(x) x
 
 extern struct r_bin_dbginfo_t r_bin_dbginfo_dex;
 
@@ -39,29 +40,33 @@ static char *getstr(RBinDexObj *bin, int idx) {
 	int uleblen;
 	// null terminate the buf wtf
 	if (!bin || idx < 0 || idx >= bin->header.strings_size || !bin->strings) {
-		return "";
+		return NULL;
 	}
 	if (bin->strings[idx] >= bin->size) {
-		return "";
+		return NULL;
 	}
 	if (r_buf_read_at (bin->b, bin->strings[idx], buf, sizeof (buf)) < 1) {
-		return "";
+		return NULL;
 	}
 	bin->b->buf[bin->b->length - 1] = 0;
 	uleblen = r_uleb128 (buf, sizeof (buf), &len) - buf;
 	if (!uleblen || uleblen >= bin->size) {
-		return "";
+		return NULL;
 	}
 	if (!len || len >= bin->size) {
-		return "";
+		return NULL;
+	}
+	if (bin->strings[idx] + uleblen >= bin->strings[idx] + bin->header.strings_size) {
+		return NULL;
 	}
 	char* ptr = (char*) r_buf_get_at (bin->b, bin->strings[idx] + uleblen, NULL);
 	if (!ptr) {
-		return "";
+		return NULL;
 	}
-	if (len != strlen (ptr)) {
-		eprintf ("WARNING: Invalid string for index %d\n", idx);
-		return "";
+
+	if (len != r_utf8_strlen ((const ut8*)ptr)) {
+		// eprintf ("WARNING: Invalid string for index %d\n", idx);
+		return NULL;
 	}
 	return ptr;
 }
@@ -192,22 +197,18 @@ static char *dex_type_descriptor(RBinDexObj *bin, int type_idx) {
 	return getstr (bin, bin->types[type_idx].descriptor_id);
 }
 
-static char *dex_method_signature(RBinDexObj *bin, int method_idx) {
-	ut32 proto_id, params_off, type_id, list_size;
+static char *dex_get_proto(RBinDexObj *bin, int proto_id) {
+	ut32 params_off, type_id, list_size;
 	char *r = NULL, *return_type = NULL, *signature = NULL, *buff = NULL;
 	ut8 *bufptr;
 	ut16 type_idx;
 	int pos = 0, i, size = 1;
 
-	if (method_idx < 0 || method_idx >= bin->header.method_size) {
-		return NULL;
-	}
-	proto_id = bin->methods[method_idx].proto_id;
 	if (proto_id >= bin->header.prototypes_size) {
 		return NULL;
 	}
 	params_off = bin->protos[proto_id].parameters_off;
-	if (params_off  >= bin->size) {
+	if (params_off >= bin->size) {
 		return NULL;
 	}
 	type_id = bin->protos[proto_id].return_type_id;
@@ -224,15 +225,17 @@ static char *dex_method_signature(RBinDexObj *bin, int method_idx) {
 	bufptr = bin->b->buf;
 	// size of the list, in entries
 	list_size = r_read_le32 (bufptr + params_off);
-	//XXX again list_size is user controlled huge loop
+	if (list_size * sizeof (ut16) >= bin->size) {
+		return NULL;
+	}
+
 	for (i = 0; i < list_size; i++) {
 		int buff_len = 0;
 		if (params_off + 4 + (i * 2) >= bin->size) {
 			break;
 		}
 		type_idx = r_read_le16 (bufptr + params_off + 4 + (i * 2));
-		if (type_idx < 0 ||
-		    type_idx >=
+		if (type_idx >=
 			    bin->header.types_size || type_idx >= bin->size) {
 			break;
 		}
@@ -259,6 +262,13 @@ static char *dex_method_signature(RBinDexObj *bin, int method_idx) {
 	return r;
 }
 
+static char *dex_method_signature(RBinDexObj *bin, int method_idx) {
+	if (method_idx < 0 || method_idx >= bin->header.method_size) {
+		return NULL;
+	}
+	return dex_get_proto (bin, bin->methods[method_idx].proto_id);
+}
+
 static RList *dex_method_signature2(RBinDexObj *bin, int method_idx) {
 	ut32 proto_id, params_off, list_size;
 	char *buff = NULL;
@@ -348,7 +358,7 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 	struct dex_debug_local_t *debug_locals = calloc (sizeof (struct dex_debug_local_t), regsz + 1);
 	if (!(MA & 0x0008)) {
 		debug_locals[argReg].name = "this";
-		debug_locals[argReg].descriptor = r_str_newf("%s;", class_name);
+		debug_locals[argReg].descriptor = r_str_newf ("%s;", class_name);
 		debug_locals[argReg].startAddress = 0;
 		debug_locals[argReg].signature = NULL;
 		debug_locals[argReg].live = true;
@@ -387,7 +397,7 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 			argReg += 1;
 			break;
 		}
-		if (strcmp(name, "")) {
+		if (!name || !*name) {
 			debug_locals[reg].name = name;
 			debug_locals[reg].descriptor = type;
 			debug_locals[reg].signature = NULL;
@@ -398,7 +408,10 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 	}
 
 	if (!p4) {
+		free (debug_positions);
+		free (params);
 		free (debug_locals);
+		free (emitted_debug_locals);
 		return;
 	}
 	ut8 opcode = *(p4++) & 0xff;
@@ -434,6 +447,7 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 				r_list_free (debug_positions);
 				free (params);
 				free (debug_locals);
+				free (emitted_debug_locals);
 				return;
 			}
 			// Emit what was previously there, if anything
@@ -586,26 +600,36 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 		binfile->sdb_addrinfo = sdb_new0 ();
 	}
 
-	char *fileline;
-	char offset[64];
-	char *offset_ptr;
 
 	RListIter *iter1;
 	struct dex_debug_position_t *pos;
+// Loading the debug info takes too much time and nobody uses this afaik
+#if 1
 	r_list_foreach (debug_positions, iter1, pos) {
-		fileline = r_str_newf ("%s|%"PFMT64d, getstr (bin, pos->source_file_idx), pos->line);
-		offset_ptr = sdb_itoa (pos->address + paddr, offset, 16);
+		const char *line = getstr (bin, pos->source_file_idx);
+#if 1
+		char offset[64] = {0};
+		if (!line || !*line) {
+			continue;
+		}
+		char *fileline = r_str_newf ("%s|%"PFMT64d, line, pos->line);
+		char *offset_ptr = sdb_itoa (pos->address + paddr, offset, 16);
 		sdb_set (binfile->sdb_addrinfo, offset_ptr, fileline, 0);
 		sdb_set (binfile->sdb_addrinfo, fileline, offset_ptr, 0);
+		free (fileline);
+#endif
 		RBinDwarfRow *rbindwardrow = R_NEW0 (RBinDwarfRow);
 		if (!rbindwardrow) {
 			return;
 		}
-		rbindwardrow->file = strdup (getstr (bin, pos->source_file_idx));
-		rbindwardrow->address = pos->address;
-		rbindwardrow->line = pos->line;
-		r_list_append (dex->lines_list, rbindwardrow);
+		if (line) {
+			rbindwardrow->file = strdup (line);
+			rbindwardrow->address = pos->address;
+			rbindwardrow->line = pos->line;
+			r_list_append (dex->lines_list, rbindwardrow);
+		}
 	}
+#endif
 
 	if (!dexdump) {
 		free (debug_positions);
@@ -620,7 +644,7 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 
 	rbin->cb_printf ("      positions     :\n");
 	r_list_foreach (debug_positions, iter2, position) {
-		rbin->cb_printf ("        0x%04llx line=%llu\n",
+		rbin->cb_printf ("        0x%04"PFMT64x" line=%llu\n",
 				 position->address, position->line);
 	}
 
@@ -644,6 +668,9 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 	}
 
 	for (reg = 0; reg < regsz; reg++) {
+		if (!debug_locals[reg].name) {
+			continue;
+		}
 		if (debug_locals[reg].live) {
 			if (debug_locals[reg].signature) {
 				rbin->cb_printf (
@@ -678,7 +705,7 @@ static Sdb *get_sdb (RBinFile *bf) {
 	return bin? bin->kv: NULL;
 }
 
-static void *load_bytes(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
+static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
 	void *res = NULL;
 	RBuffer *tbuf = NULL;
 	if (!buf || !sz || sz == UT64_MAX) {
@@ -694,18 +721,18 @@ static void *load_bytes(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr,
 	return res;
 }
 
-static bool load(RBinFile *arch) {
-	const ut8 *bytes = arch ? r_buf_buffer (arch->buf) : NULL;
-	ut64 sz = arch ? r_buf_size (arch->buf): 0;
+static bool load(RBinFile *bf) {
+	const ut8 *bytes = bf ? r_buf_buffer (bf->buf) : NULL;
+	ut64 sz = bf ? r_buf_size (bf->buf): 0;
 
-	if (!arch || !arch->o) {
+	if (!bf || !bf->o) {
 		return false;
 	}
-	arch->o->bin_obj = load_bytes (arch, bytes, sz, arch->o->loadaddr, arch->sdb);
-	return arch->o->bin_obj ? true: false;
+	bf->o->bin_obj = load_bytes (bf, bytes, sz, bf->o->loadaddr, bf->sdb);
+	return bf->o->bin_obj ? true: false;
 }
 
-static ut64 baddr(RBinFile *arch) {
+static ut64 baddr(RBinFile *bf) {
 	return 0;
 }
 
@@ -721,6 +748,10 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	if (!memcmp (buf, "dex\n036\0", 8)) {
 		return true;
 	}
+	// Two new opcodes: invoke-polymorphic and invoke-custom (sdk level 26+)
+	if (!memcmp (buf, "dex\n038\0", 8)) {
+		return true;
+	}
 	// M3 (Nov-Dec 07)
 	if (!memcmp (buf, "dex\n009\0", 8)) {
 		return true;
@@ -736,20 +767,21 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return false;
 }
 
-static RBinInfo *info(RBinFile *arch) {
+static RBinInfo *info(RBinFile *bf) {
 	RBinHash *h;
 	RBinInfo *ret = R_NEW0 (RBinInfo);
 	if (!ret) {
 		return NULL;
 	}
-	ret->file = arch->file? strdup (arch->file): NULL;
+	ret->file = bf->file? strdup (bf->file): NULL;
 	ret->type = strdup ("DEX CLASS");
 	ret->has_va = false;
-	ret->bclass = r_bin_dex_get_version (arch->o->bin_obj);
+	ret->has_lit = true;
+	ret->bclass = r_bin_dex_get_version (bf->o->bin_obj);
 	ret->rclass = strdup ("class");
 	ret->os = strdup ("linux");
 	const char *kw = "Landroid/support/wearable/view";
-	if (r_mem_mem (arch->buf->buf, arch->buf->length, (const ut8*)kw, strlen (kw))) {
+	if (r_mem_mem (bf->buf->buf, bf->buf->length, (const ut8*)kw, strlen (kw))) {
 		ret->subsystem = strdup ("android-wear");
 	} else {
 		ret->subsystem = strdup ("android");
@@ -760,20 +792,20 @@ static RBinInfo *info(RBinFile *arch) {
 	h->len = 20;
 	h->addr = 12;
 	h->from = 12;
-	h->to = arch->buf->length-32;
-	memcpy (h->buf, arch->buf->buf + 12, 20);
+	h->to = bf->buf->length-32;
+	memcpy (h->buf, bf->buf->buf + 12, 20);
 	h = &ret->sum[1];
 	h->type = "adler32";
 	h->len = 4;
 	h->addr = 0x8;
 	h->from = 12;
-	h->to = arch->buf->length-h->from;
+	h->to = bf->buf->length-h->from;
 	h = &ret->sum[2];
 	h->type = 0;
-	memcpy (h->buf, arch->buf->buf + 8, 4);
+	memcpy (h->buf, bf->buf->buf + 8, 4);
 	{
-		ut32 *fc = (ut32 *)(arch->buf->buf + 8);
-		ut32  cc = __adler32 (arch->buf->buf + 12, arch->buf->length - 12);
+		ut32 *fc = (ut32 *)(bf->buf->buf + 8);
+		ut32  cc = __adler32 (bf->buf->buf + 12, bf->buf->length - 12);
 		if (*fc != cc) {
 			eprintf ("# adler32 checksum doesn't match. Type this to fix it:\n");
 			eprintf ("wx `ph sha1 $s-32 @32` @12 ; wx `ph adler32 $s-12 @12` @8\n");
@@ -787,17 +819,17 @@ static RBinInfo *info(RBinFile *arch) {
 	return ret;
 }
 
-static RList *strings(RBinFile *arch) {
+static RList *strings(RBinFile *bf) {
 	struct r_bin_dex_obj_t *bin = NULL;
 	RBinString *ptr = NULL;
 	RList *ret = NULL;
 	int i, len;
 	ut8 buf[6];
 	ut64 off;
-	if (!arch || !arch->o) {
+	if (!bf || !bf->o) {
 		return NULL;
 	}
-	bin = (struct r_bin_dex_obj_t *) arch->o->bin_obj;
+	bin = (struct r_bin_dex_obj_t *) bf->o->bin_obj;
 	if (!bin || !bin->strings) {
 		return NULL;
 	}
@@ -818,7 +850,7 @@ static RList *strings(RBinFile *arch) {
 		r_buf_read_at (bin->b, bin->strings[i], (ut8*)&buf, 6);
 		len = dex_read_uleb128 (buf, sizeof (buf));
 
-		if (len > 1 && len < R_BIN_SIZEOF_STRINGS) {
+		if (len > 5 && len < R_BIN_SIZEOF_STRINGS) {
 			ptr->string = malloc (len + 1);
 			if (!ptr->string) {
 				goto out_error;
@@ -830,6 +862,10 @@ static RList *strings(RBinFile *arch) {
 			}
 			r_buf_read_at (bin->b, off, (ut8*)ptr->string, len);
 			ptr->string[len] = 0;
+			if ((ptr->string[0] == 'L' && strchr (ptr->string, '/')) || !strncmp (ptr->string, "[L", 2)) {
+				free (ptr->string);
+				continue;
+			}
 			ptr->vaddr = ptr->paddr = bin->strings[i];
 			ptr->size = len;
 			ptr->length = len;
@@ -848,15 +884,15 @@ out_error:
 
 static char *dex_method_name(RBinDexObj *bin, int idx) {
 	if (idx < 0 || idx >= bin->header.method_size) {
-		return "";
+		return NULL;
 	}
 	int cid = bin->methods[idx].class_id;
 	if (cid < 0 || cid >= bin->header.strings_size) {
-		return "";
+		return NULL;
 	}
 	int tid = bin->methods[idx].name_id;
 	if (tid < 0 || tid >= bin->header.strings_size) {
-		return "";
+		return NULL;
 	}
 	return getstr (bin, tid);
 }
@@ -864,10 +900,10 @@ static char *dex_method_name(RBinDexObj *bin, int idx) {
 static char *dex_class_name_byid(RBinDexObj *bin, int cid) {
 	int tid;
 	if (!bin || !bin->types) {
-		return "";
+		return NULL;
 	}
 	if (cid < 0 || cid >= bin->header.types_size) {
-		return "";
+		return NULL;
 	}
 	tid = bin->types[cid].descriptor_id;
 	return getstr (bin, tid);
@@ -880,18 +916,18 @@ static char *dex_class_name(RBinDexObj *bin, RBinDexClass *c) {
 static char *dex_field_name(RBinDexObj *bin, int fid) {
 	int cid, tid, type_id;
 	if (!bin || !bin->fields) {
-		return strdup ("");
+		return NULL;
 	}
 	if (fid < 0 || fid >= bin->header.fields_size) {
-		return strdup ("");
+		return NULL;
 	}
 	cid = bin->fields[fid].class_id;
 	if (cid < 0 || cid >= bin->header.types_size) {
-		return strdup ("");
+		return NULL;
 	}
 	type_id = bin->fields[fid].type_id;
 	if (type_id < 0 || type_id >= bin->header.types_size) {
-		return strdup ("");
+		return NULL;
 	}
 	tid = bin->fields[fid].name_id;
 	return r_str_newf ("%s->%s %s", getstr (bin, bin->types[cid].descriptor_id),
@@ -900,27 +936,47 @@ static char *dex_field_name(RBinDexObj *bin, int fid) {
 
 static char *dex_method_fullname(RBinDexObj *bin, int method_idx) {
 	if (!bin || !bin->types) {
-		return strdup ("");
+		return NULL;
 	}
 	if (method_idx < 0 || method_idx >= bin->header.method_size) {
-		return strdup ("");
+		return NULL;
 	}
 	int cid = bin->methods[method_idx].class_id;
 	if (cid < 0 || cid >= bin->header.types_size) {
-		return strdup ("");
+		return NULL;
+	}
+	const char *name = dex_method_name (bin, method_idx);
+	if (!name) {
+		return NULL;
+	}
+	const char *className = dex_class_name_byid (bin, cid);
+	char *flagname = NULL;
+	if (className) {
+		char *class_name = strdup (className);
+		r_str_replace_char (class_name, ';', 0);
+		char *signature = dex_method_signature (bin, method_idx);
+		if (signature) {
+			flagname = r_str_newf ("%s.%s%s", class_name, name, signature);
+			free (signature);
+		} else {
+			flagname = r_str_newf ("%s.%s%s", class_name, name, "???");
+		}
+		free (class_name);
+	} else {
+		char *signature = dex_method_signature (bin, method_idx);
+		if (signature) {
+			flagname = r_str_newf ("%s.%s%s", "???", name, signature);
+			free (signature);
+		} else {
+			flagname = r_str_newf ("%s.%s%s", "???", name, "???");
+			free (signature);
+		}
 	}
-	char *name = dex_method_name (bin, method_idx);
-	char *class_name = strdup (dex_class_name_byid (bin, cid));
-	class_name = r_str_replace (class_name, ";", "", 0); //TODO: move to func
-	char *signature = dex_method_signature (bin, method_idx);
-	char *flagname = r_str_newf ("%s.%s%s", class_name, name, signature);
-	free (class_name);
-	free (signature);
 	return flagname;
 }
 
-static ut64 dex_get_type_offset(RBinFile *arch, int type_idx) {
-	RBinDexObj *bin = (RBinDexObj*) arch->o->bin_obj;
+static ut64 dex_get_type_offset(RBinFile *bf, int type_idx) {
+	RBinDexObj *bin = (RBinDexObj*) bf->o->bin_obj;
 	if (!bin || !bin->types) {
 		return 0;
 	}
@@ -930,14 +986,14 @@ static ut64 dex_get_type_offset(RBinFile *arch, int type_idx) {
 	return bin->header.types_offset + type_idx * 0x04; //&bin->types[type_idx];
 }
 
-static char *dex_class_super_name(RBinDexObj *bin, RBinDexClass *c) {
+static const char *dex_class_super_name(RBinDexObj *bin, RBinDexClass *c) {
 	int cid, tid;
 	if (!bin || !c || !bin->types) {
-		return "";
+		return NULL;
 	}
 	cid = c->super_class;
 	if (cid < 0 || cid >= bin->header.types_size) {
-		return "";
+		return NULL;
 	}
 	tid = bin->types[cid].descriptor_id;
 	return getstr (bin, tid);
@@ -991,7 +1047,7 @@ static const ut8 *parse_dex_class_fields(RBinFile *binfile, RBinDexObj *bin,
 			sym->type = r_str_const ("FIELD");
 		}
 		sym->name = r_str_replace (sym->name, "method.", "", 0);
-		//sym->name = r_str_replace (sym->name, ";", "", 0);
+		r_str_replace_char (sym->name, ';', 0);
 		sym->paddr = sym->vaddr = total;
 		sym->ordinal = (*sym_count)++;
 
@@ -1010,7 +1066,7 @@ static const ut8 *parse_dex_class_fields(RBinFile *binfile, RBinDexObj *bin,
 		RBinField *field = R_NEW0 (RBinField);
 		if (!field) {
 			return NULL;
-		}		
+		}
 		field->vaddr = field->paddr = sym->paddr;
 		field->name = strdup (sym->name);
 		field->flags = get_method_flags (accessFlags);
@@ -1024,20 +1080,24 @@ static const ut8 *parse_dex_class_fields(RBinFile *binfile, RBinDexObj *bin,
 // TODO: refactor this method
 // XXX it needs a lot of love!!!
 static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
-					  RBinDexClass *c, RBinClass *cls,
-					  const ut8 *p, const ut8 *p_end,
-					  int *sym_count, ut64 DM, int *methods,
-					  bool is_direct) {
+		RBinDexClass *c, RBinClass *cls,
+		const ut8 *p, const ut8 *p_end,
+		int *sym_count, ut64 DM, int *methods,
+		bool is_direct) {
 	struct r_bin_t *rbin = binfile->rbin;
-	ut8 ff2[16] = {0};
-	ut8 ff3[8] = {0};
-	int i;
+	bool bin_dbginfo = rbin->want_dbginfo;
+	int i, left;
 	ut64 omi = 0;
 	bool catchAll;
 	ut16 regsz, ins_size, outs_size, tries_size;
 	ut16 handler_off, start_addr, insn_count;
 	ut32 debug_info_off, insns_size;
 	const ut8 *encoded_method_addr;
+
+	if (DM > 4096) {
+		eprintf ("This DEX is probably corrupted. Chopping DM from %d to 4KB\n", (int)DM);
+		DM = 4096;
+	}
 	for (i = 0; i < DM; i++) {
 		encoded_method_addr = p;
 		char *method_name, *flag_name;
@@ -1058,10 +1118,10 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 		if (!method_name) {
 			method_name = "unknown";
 		}
-		flag_name = r_str_newf ("%s.method.%s%s", cls->name,
-					method_name, signature);
-		if (!flag_name) {
+		flag_name = r_str_newf ("%s.method.%s%s", cls->name, method_name, signature);
+		if (!flag_name || !*flag_name) {
 			//R_FREE (method_name);
+			R_FREE (flag_name);
 			R_FREE (signature);
 			continue;
 		}
@@ -1078,9 +1138,8 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 				R_FREE (signature);
 				continue;
 			}
-			if (r_buf_read_at (binfile->buf,
-					   binfile->buf->base + MC, ff2,
-					   16) < 1) {
+			const ut8 *ff2 = r_buf_get_at (binfile->buf, binfile->buf->base + MC, &left);
+			if (!ff2 || left < 16) {
 				//R_FREE (method_name);
 				R_FREE (flag_name);
 				R_FREE (signature);
@@ -1130,41 +1189,36 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 						R_FREE (signature);
 						break;
 					}
-					if (r_buf_read_at (
-						    binfile->buf,
-						    binfile->buf->base + offset,
-						    ff3, 8) < 1) {
-						// free (method_name);
+					const ut8 *ptr = r_buf_get_at (binfile->buf, binfile->buf->base + offset, &left);
+					if (!ptr || left < 8) {
 						R_FREE (signature);
 						break;
 					}
-					start_addr = r_read_le32 (ff3);
-					insn_count = r_read_le16 (ff3 + 4);
-					handler_off = r_read_le16 (ff3 + 6);
+					start_addr = r_read_le32 (ptr);
+					insn_count = r_read_le16 (ptr + 4);
+					handler_off = r_read_le16 (ptr + 6);
+
 					char* s = NULL;
 					if (dexdump) {
-						rbin->cb_printf (
-							"        0x%04x - "
-							"0x%04x\n",
-							start_addr,
-							(start_addr +
-							 insn_count));
+						rbin->cb_printf ("        0x%04x - 0x%04x\n",
+							start_addr, (start_addr + insn_count));
 					}
 
-					const ut8 *p3, *p3_end;
 					//XXX tries_size is tainted and oob here
 					int off = MC + t + tries_size * 8 + handler_off;
 					if (off >= bin->size || off < tries_size) {
 						R_FREE (signature);
 						break;
 					}
-					p3 = r_buf_get_at (binfile->buf, off, NULL);
-					p3_end = p3 + binfile->buf->length - off;
+					// TODO: catch left instead of null
+					const ut8 *p3 = r_buf_get_at (binfile->buf, off, NULL);
+					const ut8 *p3_end = p3 + binfile->buf->length - off;
 					st64 size = r_sleb128 (&p3, p3_end);
 
 					if (size <= 0) {
 						catchAll = true;
 						size = -size;
+						// XXX this is probably wrong
 					} else {
 						catchAll = false;
 					}
@@ -1173,35 +1227,25 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 						p3 = r_uleb128 (p3, p3_end - p3, &handler_type);
 						p3 = r_uleb128 (p3, p3_end - p3, &handler_addr);
 
-						if (handler_type > 0 &&
-						    handler_type <
-							    bin->header.types_size) {
+						if (handler_type > 0 && handler_type < bin->header.types_size) {
 							s = getstr (bin, bin->types[handler_type].descriptor_id);
 							if (dexdump) {
 								rbin->cb_printf (
 									"          %s "
-									"-> 0x%04llx\n",
+									"-> 0x%04"PFMT64x"\n",
 									s,
 									handler_addr);
 							}
 						} else {
 							if (dexdump) {
-								rbin->cb_printf (
-									"          "
-									"(error) -> "
-									"0x%04llx\n",
-									handler_addr);
+								rbin->cb_printf ("          (error) -> 0x%04"PFMT64x"\n", handler_addr);
 							}
 						}
 					}
 					if (catchAll) {
 						p3 = r_uleb128 (p3, p3_end - p3, &v2);
 						if (dexdump) {
-							rbin->cb_printf (
-								"          "
-								"<any> -> "
-								"0x%04llx\n",
-								v2);
+							rbin->cb_printf ("          <any> -> 0x%04"PFMT64x"\n", v2);
 						}
 					}
 				}
@@ -1221,6 +1265,7 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 		if (*flag_name) {
 			RBinSymbol *sym = R_NEW0 (RBinSymbol);
 			if (!sym) {
+				R_FREE (flag_name);
 				return NULL;
 			}
 			sym->name = flag_name;
@@ -1246,7 +1291,9 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 
 			sym->ordinal = (*sym_count)++;
 			if (MC > 0) {
-				if (r_buf_read_at (binfile->buf, binfile->buf->base + MC, ff2, 16) < 1) {
+				const ut8 *ff2 = r_buf_get_at (binfile->buf, binfile->buf->base + MC, &left);
+				if (!ff2 || left < 16) {
+				// if (r_buf_read_at (binfile->buf, binfile->buf->base + MC, ff2, 16) < 1) {
 					R_FREE (sym);
 					R_FREE (signature);
 					continue;
@@ -1302,8 +1349,10 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 			}
 			if (MC > 0 && debug_info_off > 0 && bin->header.data_offset < debug_info_off &&
 				debug_info_off < bin->header.data_offset + bin->header.data_size) {
-				dex_parse_debug_item (binfile, bin, c, MI, MA, sym->paddr, ins_size,
-					insns_size, cls->name, regsz, debug_info_off);
+				if (bin_dbginfo) {
+					dex_parse_debug_item (binfile, bin, c, MI, MA, sym->paddr, ins_size,
+							insns_size, cls->name, regsz, debug_info_off);
+				}
 			} else if (MC > 0) {
 				if (dexdump) {
 					rbin->cb_printf ("      positions     :\n");
@@ -1381,32 +1430,38 @@ static void parse_class(RBinFile *binfile, RBinDexObj *bin, RBinDexClass *c,
 		return;
 	}
 	class_name = dex_class_name (bin, c);
-	if (!strcmp(class_name, "")) {
+	if (!class_name || !*class_name) {
+		return;
+	}
+	const char *superClass = dex_class_super_name (bin, c);
+	if (!superClass) {
 		return;
 	}
 	class_name = strdup (class_name);
-	class_name = r_str_replace (class_name, ";", "", 0); //TODO: move to func
+	r_str_replace_char (class_name, ';', 0);
 
 	if (!class_name || !*class_name) {
 		return;
 	}
-
 	RBinClass *cls = R_NEW0 (RBinClass);
 	if (!cls) {
+		free (class_name);
 		return;
 	}
 	cls->name = class_name;
 	cls->index = class_index;
 	cls->addr = bin->header.class_offset + class_index * DEX_CLASS_SIZE;
 	cls->methods = r_list_new ();
-	cls->super = strdup (dex_class_super_name (bin, c));
+	cls->super = strdup (superClass);
 	if (!cls->methods) {
 		free (cls);
+		free (class_name);
 		return;
 	}
 	cls->fields = r_list_new ();
 	if (!cls->fields) {
 		r_list_free (cls->methods);
+		free (class_name);
 		free (cls);
 		return;
 	}
@@ -1442,7 +1497,6 @@ static void parse_class(RBinFile *binfile, RBinDexObj *bin, RBinDexClass *c,
 			}
 		}
 	}
-
 	// TODO: this is quite ugly
 	if (!c || !c->class_data_offset) {
 		if (dexdump) {
@@ -1526,8 +1580,8 @@ static bool is_class_idx_in_code_classes(RBinDexObj *bin, int class_idx) {
 	return false;
 }
 
-static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
-	struct r_bin_t *rbin = arch->rbin;
+static int dex_loadcode(RBinFile *bf, RBinDexObj *bin) {
+	struct r_bin_t *rbin = bf->rbin;
 	int i;
 	int *methods = NULL;
 	int sym_count = 0;
@@ -1536,6 +1590,7 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 	if (!bin || bin->methods_list) {
 		return false;
 	}
+	bin->version = r_bin_dex_get_version (bin);
 	bin->code_from = UT64_MAX;
 	bin->code_to = 0;
 	bin->methods_list = r_list_newf ((RListFree)free);
@@ -1549,7 +1604,6 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 	}
 	bin->lines_list = r_list_newf ((RListFree)free);
 	if (!bin->lines_list) {
-		r_list_free (bin->lines_list);
 		return false;
 	}
 	bin->classes_list = r_list_newf ((RListFree)r_bin_class_free);
@@ -1587,10 +1641,9 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 			if (dexdump) {
 				rbin->cb_printf ("Class #%d            -\n", i);
 			}
-			parse_class (arch, bin, c, i, methods, &sym_count);
+			parse_class (bf, bin, c, i, methods, &sym_count);
 		}
 	}
-
 	if (methods) {
 		int import_count = 0;
 		int sym_count = bin->methods_list->length;
@@ -1609,9 +1662,11 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 				continue;
 			}
 
-			char *class_name = strdup (getstr (
-				bin, bin->types[bin->methods[i].class_id]
-						.descriptor_id));
+			const char *className = getstr (bin, bin->types[bin->methods[i].class_id].descriptor_id);
+			if (!className) {
+				continue;
+			}
+			char *class_name = strdup (className);
 			if (!class_name) {
 				free (class_name);
 				continue;
@@ -1621,12 +1676,15 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 				free (class_name);
 				continue;
 			}
-			class_name = r_str_replace (class_name, ";", "", 0);
+			r_str_replace_char (class_name, ';', 0);
 			char *method_name = dex_method_name (bin, i);
 			char *signature = dex_method_signature (bin, i);
 			if (method_name && *method_name) {
 				RBinImport *imp = R_NEW0 (RBinImport);
 				if (!imp) {
+					free (methods);
+					free (signature);
+					free (class_name);
 					return false;
 				}
 				imp->name  = r_str_newf ("%s.method.%s%s", class_name, method_name, signature);
@@ -1637,6 +1695,9 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 
 				RBinSymbol *sym = R_NEW0 (RBinSymbol);
 				if (!sym) {
+					free (methods);
+					free (signature);
+					free (class_name);
 					return false;
 				}
 				sym->name = r_str_newf ("imp.%s", imp->name);
@@ -1651,43 +1712,44 @@ static int dex_loadcode(RBinFile *arch, RBinDexObj *bin) {
 
 			}
 			free (signature);
+			free (class_name);
 		}
 		free (methods);
 	}
 	return true;
 }
 
-static RList* imports(RBinFile *arch) {
-	RBinDexObj *bin = (RBinDexObj*) arch->o->bin_obj;
+static RList* imports(RBinFile *bf) {
+	RBinDexObj *bin = (RBinDexObj*) bf->o->bin_obj;
 	if (!bin) {
 		return NULL;
 	}
 	if (bin && bin->imports_list) {
 		return bin->imports_list;
 	}
-	dex_loadcode (arch, bin);
+	dex_loadcode (bf, bin);
 	return bin->imports_list;
 }
 
-static RList *methods(RBinFile *arch) {
-	if (!arch || !arch->o || !arch->o->bin_obj) {
+static RList *methods(RBinFile *bf) {
+	if (!bf || !bf->o || !bf->o->bin_obj) {
 		return NULL;
 	}
-	RBinDexObj *bin = (RBinDexObj*) arch->o->bin_obj;
+	RBinDexObj *bin = (RBinDexObj*) bf->o->bin_obj;
 	if (!bin->methods_list) {
-		dex_loadcode (arch, bin);
+		dex_loadcode (bf, bin);
 	}
 	return bin->methods_list;
 }
 
-static RList *classes(RBinFile *arch) {
+static RList *classes(RBinFile *bf) {
 	RBinDexObj *bin;
-	if (!arch || !arch->o || !arch->o->bin_obj) {
+	if (!bf || !bf->o || !bf->o->bin_obj) {
 		return NULL;
 	}
-	bin = (RBinDexObj*) arch->o->bin_obj;
+	bin = (RBinDexObj*) bf->o->bin_obj;
 	if (!bin->classes_list) {
-		dex_loadcode (arch, bin);
+		dex_loadcode (bf, bin);
 	}
 	return bin->classes_list;
 }
@@ -1703,27 +1765,27 @@ static int already_entry(RList *entries, ut64 vaddr) {
 	return 0;
 }
 
-static RList *entries(RBinFile *arch) {
+static RList *entries(RBinFile *bf) {
 	RListIter *iter;
 	RBinDexObj *bin;
 	RBinSymbol *m;
 	RBinAddr *ptr;
 	RList *ret;
 
-	if (!arch || !arch->o || !arch->o->bin_obj) {
+	if (!bf || !bf->o || !bf->o->bin_obj) {
 		return NULL;
 	}
-	bin = (RBinDexObj*) arch->o->bin_obj;
+	bin = (RBinDexObj*) bf->o->bin_obj;
 	ret = r_list_newf ((RListFree)free);
 
 	if (!bin->methods_list) {
-		dex_loadcode (arch, bin);
+		dex_loadcode (bf, bin);
 	}
 
 	// STEP 1. ".onCreate(Landroid/os/Bundle;)V"
 	r_list_foreach (bin->methods_list, iter, m) {
 		if (strlen (m->name) > 30 && m->bind &&
-			!strcmp(m->bind, "GLOBAL") &&
+			(!strcmp (m->bind, "LOCAL") || !strcmp (m->bind, "GLOBAL")) &&
 		    !strcmp (m->name + strlen (m->name) - 31,
 			     ".onCreate(Landroid/os/Bundle;)V")) {
 			if (!already_entry (ret, m->paddr)) {
@@ -1764,19 +1826,18 @@ static RList *entries(RBinFile *arch) {
 	return ret;
 }
 
-static ut64 offset_of_method_idx(RBinFile *arch, struct r_bin_dex_obj_t *dex, int idx) {
-	ut64 off = dex->header.method_offset + idx;
-	off = sdb_num_get (mdb, sdb_fmt (0, "method.%d", idx), 0);
-	return (ut64) off;
+static ut64 offset_of_method_idx(RBinFile *bf, struct r_bin_dex_obj_t *dex, int idx) {
+	// ut64 off = dex->header.method_offset + idx;
+	return sdb_num_get (mdb, sdb_fmt (0, "method.%d", idx), 0);
 }
 
 // TODO: change all return type for all getoffset
-static int getoffset(RBinFile *arch, int type, int idx) {
-	struct r_bin_dex_obj_t *dex = arch->o->bin_obj;
+static int getoffset(RBinFile *bf, int type, int idx) {
+	struct r_bin_dex_obj_t *dex = bf->o->bin_obj;
 	switch (type) {
 	case 'm': // methods
 		// TODO: ADD CHECK
-		return offset_of_method_idx (arch, dex, idx);
+		return offset_of_method_idx (bf, dex, idx);
 	case 'o': // objects
 		break;
 	case 's': // strings
@@ -1785,15 +1846,15 @@ static int getoffset(RBinFile *arch, int type, int idx) {
 		}
 		break;
 	case 't': // type
-		return dex_get_type_offset (arch, idx);
+		return dex_get_type_offset (bf, idx);
 	case 'c': // class
-		return dex_get_type_offset (arch, idx);
+		return dex_get_type_offset (bf, idx);
 	}
 	return -1;
 }
 
-static char *getname(RBinFile *arch, int type, int idx) {
-	struct r_bin_dex_obj_t *dex = arch->o->bin_obj;
+static char *getname(RBinFile *bf, int type, int idx) {
+	struct r_bin_dex_obj_t *dex = bf->o->bin_obj;
 	switch (type) {
 	case 'm': // methods
 		return dex_method_fullname (dex, idx);
@@ -1801,13 +1862,15 @@ static char *getname(RBinFile *arch, int type, int idx) {
 		return dex_class_name_byid (dex, idx);
 	case 'f': // fields
 		return dex_field_name (dex, idx);
+	case 'p': // proto
+		return dex_get_proto (dex, idx);
 	}
 	return NULL;
 }
 
-static RList *sections(RBinFile *arch) {
-	struct r_bin_dex_obj_t *bin = arch->o->bin_obj;
-	RList *ml = methods (arch);
+static RList *sections(RBinFile *bf) {
+	struct r_bin_dex_obj_t *bin = bf->o->bin_obj;
+	RList *ml = methods (bf);
 	RBinSection *ptr = NULL;
 	int ns, fsymsz = 0;
 	RList *ret = NULL;
@@ -1820,7 +1883,7 @@ static RList *sections(RBinFile *arch) {
 			fsym = m->paddr;
 		}
 		ns = m->paddr + m->size;
-		if (ns > arch->buf->length) {
+		if (ns > bf->buf->length) {
 			continue;
 		}
 		if (ns > fsymsz) {
@@ -1848,6 +1911,7 @@ static RList *sections(RBinFile *arch) {
 		//ptr->size = ptr->vsize = fsym;
 		ptr->paddr= ptr->vaddr = sizeof (struct dex_header_t);
 		ptr->size = bin->code_from - ptr->vaddr; // fix size
+		ptr->vsize = ptr->size;
 		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_MAP;
 		ptr->add = true;
 		r_list_append (ret, ptr);
@@ -1856,19 +1920,20 @@ static RList *sections(RBinFile *arch) {
 		strcpy (ptr->name, "code");
 		ptr->vaddr = ptr->paddr = bin->code_from; //ptr->vaddr = fsym;
 		ptr->size = bin->code_to - ptr->paddr;
+		ptr->vsize = ptr->size;
 		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
 		ptr->add = true;
 		r_list_append (ret, ptr);
 	}
 	if ((ptr = R_NEW0 (RBinSection))) {
-		//ut64 sz = arch ? r_buf_size (arch->buf): 0;
+		//ut64 sz = bf ? r_buf_size (bf->buf): 0;
 		strcpy (ptr->name, "data");
 		ptr->paddr = ptr->vaddr = fsymsz+fsym;
-		if (ptr->vaddr > arch->buf->length) {
+		if (ptr->vaddr > bf->buf->length) {
 			ptr->paddr = ptr->vaddr = bin->code_to;
-			ptr->size = ptr->vsize = arch->buf->length - ptr->vaddr;
+			ptr->size = ptr->vsize = bf->buf->length - ptr->vaddr;
 		} else {
-			ptr->size = ptr->vsize = arch->buf->length - ptr->vaddr;
+			ptr->size = ptr->vsize = bf->buf->length - ptr->vaddr;
 			// hacky workaround
 			//ptr->size = ptr->vsize = 1024;
 		}
@@ -1879,9 +1944,9 @@ static RList *sections(RBinFile *arch) {
 	return ret;
 }
 
-static void header(RBinFile *arch) {
-	struct r_bin_dex_obj_t *bin = arch->o->bin_obj;
-	struct r_bin_t *rbin = arch->rbin;
+static void header(RBinFile *bf) {
+	struct r_bin_dex_obj_t *bin = bf->o->bin_obj;
+	struct r_bin_t *rbin = bf->rbin;
 
 	rbin->cb_printf ("DEX file header:\n");
 	rbin->cb_printf ("magic               : 'dex\\n035\\0'\n");
@@ -1909,21 +1974,21 @@ static void header(RBinFile *arch) {
 	// TODO: print information stored in the RBIN not this ugly fix
 	dexdump = true;
 	bin->methods_list = NULL;
-	dex_loadcode (arch, bin);
+	dex_loadcode (bf, bin);
 	dexdump = false;
 }
 
-static ut64 size(RBinFile *arch) {
+static ut64 size(RBinFile *bf) {
 	int ret;
 	ut32 off = 0, len = 0;
 	ut8 u32s[sizeof (ut32)] = {0};
 
-	ret = r_buf_read_at (arch->buf, 108, u32s, 4);
+	ret = r_buf_read_at (bf->buf, 108, u32s, 4);
 	if (ret != 4) {
 		return 0;
 	}
 	off = r_read_le32 (u32s);
-	ret = r_buf_read_at (arch->buf, 104, u32s, 4);
+	ret = r_buf_read_at (bf->buf, 104, u32s, 4);
 	if (ret != 4) {
 		return 0;
 	}
@@ -1931,9 +1996,12 @@ static ut64 size(RBinFile *arch) {
 	return off + len;
 }
 
-static RList *lines(RBinFile *arch) {
-	struct r_bin_dex_obj_t *dex = arch->o->bin_obj;
-	return r_list_clone (dex->lines_list);
+static RList *lines(RBinFile *bf) {
+	struct r_bin_dex_obj_t *dex = bf->o->bin_obj;
+	/// XXX this is called more than once
+	// r_sys_backtrace();
+	return dex->lines_list;
+	// return r_list_clone (dex->lines_list);
 }
 
 RBinPlugin r_bin_plugin_dex = {
