@@ -11,16 +11,16 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return (!memcmp (buf, INES_MAGIC, 4));
 }
 
-static void * load_bytes(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
+static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
 	check_bytes (buf, sz);
 	return R_NOTNULL;
 }
 
-static RBinInfo* info(RBinFile *arch) {
+static RBinInfo* info(RBinFile *bf) {
 	RBinInfo *ret = NULL;
 	ines_hdr ihdr;
 	memset (&ihdr, 0, INES_HDR_SIZE);
-	int reat = r_buf_read_at (arch->buf, 0, (ut8*)&ihdr, INES_HDR_SIZE);
+	int reat = r_buf_read_at (bf->buf, 0, (ut8*)&ihdr, INES_HDR_SIZE);
 	if (reat != INES_HDR_SIZE) {
 		eprintf ("Truncated Header\n");
 		return NULL;
@@ -28,7 +28,7 @@ static RBinInfo* info(RBinFile *arch) {
 	if (!(ret = R_NEW0 (RBinInfo))) {
 		return NULL;
 	}
-	ret->file = strdup (arch->file);
+	ret->file = strdup (bf->file);
 	ret->type = strdup ("ROM");
 	ret->machine = strdup ("Nintendo NES");
 	ret->os = strdup ("nes");
@@ -48,7 +48,7 @@ static void addsym(RList *ret, const char *name, ut64 addr, ut32 size) {
 	r_list_append (ret, ptr);
 }
 
-static RList* symbols(RBinFile *arch) {
+static RList* symbols(RBinFile *bf) {
 	RList *ret = NULL;
 	if (!(ret = r_list_newf (free))) {
 		return NULL;
@@ -78,12 +78,12 @@ static RList* symbols(RBinFile *arch) {
 	return ret;
 }
 
-static RList* sections(RBinFile *arch) {
+static RList* sections(RBinFile *bf) {
 	RList *ret = NULL;
 	RBinSection *ptr = NULL;
 	ines_hdr ihdr;
 	memset (&ihdr, 0, INES_HDR_SIZE);
-	int reat = r_buf_read_at (arch->buf, 0, (ut8*)&ihdr, INES_HDR_SIZE);
+	int reat = r_buf_read_at (bf->buf, 0, (ut8*)&ihdr, INES_HDR_SIZE);
 	if (reat != INES_HDR_SIZE) {
 		eprintf ("Truncated Header\n");
 		return NULL;
@@ -105,7 +105,7 @@ static RList* sections(RBinFile *arch) {
 	return ret;
 }
 
-static RList *mem (RBinFile *arch) {
+static RList *mem (RBinFile *bf) {
 	RList *ret;
 	RBinMem *m, *n;
 	if (!(ret = r_list_new())) {
@@ -184,7 +184,7 @@ static RList *mem (RBinFile *arch) {
 	return ret;
 }
 
-static RList* entries(RBinFile *arch) { //Should be 3 offsets pointed by NMI, RESET, IRQ after mapping && default = 1st CHR
+static RList* entries(RBinFile *bf) { //Should be 3 offsets pointed by NMI, RESET, IRQ after mapping && default = 1st CHR
 	RList *ret;
 	RBinAddr *ptr = NULL;
 	if (!(ret = r_list_new ())) {
@@ -199,7 +199,7 @@ static RList* entries(RBinFile *arch) { //Should be 3 offsets pointed by NMI, RE
 	return ret;
 }
 
-static ut64 baddr(RBinFile *arch) {
+static ut64 baddr(RBinFile *bf) {
 	// having this we make r2 -B work, otherwise it doesnt works :??
 	return 0;
 }
