@@ -15,14 +15,14 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	if (length < 0x8000) {
 		return false;
 	}
-	//determine if ROM is headered, and add a 0x200 gap if so. 
+	//determine if ROM is headered, and add a 0x200 gap if so.
 	cksum1 = r_read_le16 (buf_hdr + 0x7FDC);
 	cksum2 = r_read_le16 (buf_hdr + 0x7FDE);
 
 	if (cksum1 == (ut16)~cksum2) {
 		return true;
 	}
-	
+
 	if (length < 0xffee) {
 		return false;
 	}
@@ -31,20 +31,20 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return (cksum1 == (ut16)~cksum2);
 }
 
-static void * load_bytes(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
+static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
 	return check_bytes (buf, sz) ? R_NOTNULL : NULL;
 }
 
-static RBinInfo* info(RBinFile *arch) {
+static RBinInfo* info(RBinFile *bf) {
 	sfc_int_hdr sfchdr = {{0}};
 	RBinInfo *ret = NULL;
 	int hdroffset = 0;
-
-	if ((arch->size & 0x8000) == 0x200) {
+#if THIS_IS_ALWAYS_FALSE_WTF
+	if ((bf->size & 0x8000) == 0x200) {
 		hdroffset = 0x200;
 	}
-
-	int reat = r_buf_read_at (arch->buf, 0x7FC0 + hdroffset,
+#endif
+	int reat = r_buf_read_at (bf->buf, 0x7FC0 + hdroffset,
 		(ut8*)&sfchdr, SFC_HDR_SIZE);
 	if (reat != SFC_HDR_SIZE) {
 		eprintf ("Unable to read SFC/SNES header\n");
@@ -54,24 +54,24 @@ static RBinInfo* info(RBinFile *arch) {
 	if ( (sfchdr.comp_check != (ut16)~(sfchdr.checksum)) || ((sfchdr.rom_setup & 0x1) != 0) ){
 
 		// if the fixed 0x33 byte or the LoROM indication are not found, then let's try interpreting the ROM as HiROM
-		
-		reat = r_buf_read_at (arch->buf, 0xFFC0 + hdroffset, (ut8*)&sfchdr, SFC_HDR_SIZE);
+
+		reat = r_buf_read_at (bf->buf, 0xFFC0 + hdroffset, (ut8*)&sfchdr, SFC_HDR_SIZE);
 		if (reat != SFC_HDR_SIZE) {
 			eprintf ("Unable to read SFC/SNES header\n");
 			return NULL;
 		}
-	
+
 		if ( (sfchdr.comp_check != (ut16)~(sfchdr.checksum)) || ((sfchdr.rom_setup & 0x1) != 1) ) {
 
 			eprintf ("Cannot determine if this is a LoROM or HiROM file\n");
 			return NULL;
 		}
 	}
-	
+
 	if (!(ret = R_NEW0 (RBinInfo))) {
 		return NULL;
 	}
-	ret->file = strdup (arch->file);
+	ret->file = strdup (bf->file);
 	ret->type = strdup ("ROM");
 	ret->machine = strdup ("Super NES / Super Famicom");
 	ret->os = strdup ("snes");
@@ -109,24 +109,24 @@ static void addsym(RList *ret, const char *name, ut64 addr, ut32 size) {
 }
 #endif
 
-static RList* symbols(RBinFile *arch) {
+static RList* symbols(RBinFile *bf) {
 	return NULL;
 }
 
-static RList* sections(RBinFile *arch) {
+static RList* sections(RBinFile *bf) {
 	RList *ret = NULL;
 	// RBinSection *ptr = NULL;
 	int hdroffset = 0;
 	bool is_hirom = false;
 	int i = 0; //0x8000-long bank number for loops
-	
-	if ((arch->size & 0x8000) == 0x200) {
+#if THIS_IS_ALWAYS_FALSE_WTF
+	if ((bf->size & 0x8000) == 0x200) {
 		hdroffset = 0x200;
 	}
-	sfc_int_hdr sfchdr;
-	memset (&sfchdr, 0, SFC_HDR_SIZE);
+#endif
+	sfc_int_hdr sfchdr = {{0}};
 
-	int reat = r_buf_read_at (arch->buf, 0x7FC0 + hdroffset, (ut8*)&sfchdr, SFC_HDR_SIZE);
+	int reat = r_buf_read_at (bf->buf, 0x7FC0 + hdroffset, (ut8*)&sfchdr, SFC_HDR_SIZE);
 	if (reat != SFC_HDR_SIZE) {
 		eprintf ("Unable to read SFC/SNES header\n");
 		return NULL;
@@ -135,13 +135,13 @@ static RList* sections(RBinFile *arch) {
 	if ( (sfchdr.comp_check != (ut16)~(sfchdr.checksum)) || ((sfchdr.rom_setup & 0x1) != 0) ){
 
 		// if the fixed 0x33 byte or the LoROM indication are not found, then let's try interpreting the ROM as HiROM
-		
-		reat = r_buf_read_at (arch->buf, 0xFFC0 + hdroffset, (ut8*)&sfchdr, SFC_HDR_SIZE);
+
+		reat = r_buf_read_at (bf->buf, 0xFFC0 + hdroffset, (ut8*)&sfchdr, SFC_HDR_SIZE);
 		if (reat != SFC_HDR_SIZE) {
 			eprintf ("Unable to read SFC/SNES header\n");
 			return NULL;
 		}
-	
+
 		if ( (sfchdr.comp_check != (ut16)~(sfchdr.checksum)) || ((sfchdr.rom_setup & 0x1) != 1) ) {
 
 			eprintf ("Cannot determine if this is a LoROM or HiROM file\n");
@@ -149,13 +149,13 @@ static RList* sections(RBinFile *arch) {
 		}
 		is_hirom = true;
 	}
-	
+
 	if (!(ret = r_list_new ())) {
 		return NULL;
 	}
 
 	if (is_hirom) {
-		for (i = 0; i < ((arch->size - hdroffset) / 0x8000) ; i++) {
+		for (i = 0; i < ((bf->size - hdroffset) / 0x8000) ; i++) {
 			// XXX check integer overflow here
 			addrom (ret, "ROM",i,hdroffset + i * 0x8000, 0x400000 + (i * 0x8000), 0x8000);
 			if (i % 2) {
@@ -164,7 +164,7 @@ static RList* sections(RBinFile *arch) {
 		}
 
 	} else {
-		for (i=0; i < ((arch->size - hdroffset)/ 0x8000) ; i++) {
+		for (i=0; i < ((bf->size - hdroffset)/ 0x8000) ; i++) {
 
 			addrom(ret,"ROM",i,hdroffset + i*0x8000,0x8000 + (i*0x10000), 0x8000);
 		}
@@ -172,7 +172,7 @@ static RList* sections(RBinFile *arch) {
 	return ret;
 }
 
-static RList *mem (RBinFile *arch) {
+static RList *mem (RBinFile *bf) {
 	RList *ret;
 	RBinMem *m;
 	RBinMem *m_bak;
@@ -255,7 +255,7 @@ static RList *mem (RBinFile *arch) {
 	return ret;
 }
 
-static RList* entries(RBinFile *arch) { //Should be 3 offsets pointed by NMI, RESET, IRQ after mapping && default = 1st CHR
+static RList* entries(RBinFile *bf) { //Should be 3 offsets pointed by NMI, RESET, IRQ after mapping && default = 1st CHR
 	RList *ret;
 	if (!(ret = r_list_new ())) {
 		return NULL;
