@@ -13,43 +13,43 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return (buf && length >= 4 && !memcmp (buf, R_BIN_WASM_MAGIC_BYTES, 4));
 }
 
-static void *load_bytes(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
+static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
 	if (!buf || !sz || sz == UT64_MAX) {
 		return NULL;
 	}
-	if (!check_bytes(buf, sz)) {
+	if (!check_bytes (buf, sz)) {
 		return NULL;
 	}
-	return r_bin_wasm_init (arch);
+	return r_bin_wasm_init (bf);
 }
 
-static bool load(RBinFile *arch) {
-	const ut8 *bytes = arch ? r_buf_buffer (arch->buf) : NULL;
-	ut64 sz = arch ? r_buf_size (arch->buf): 0;
-	if (!arch || !arch->o) {
+static bool load(RBinFile *bf) {
+	const ut8 *bytes = bf ? r_buf_buffer (bf->buf) : NULL;
+	ut64 sz = bf ? r_buf_size (bf->buf): 0;
+	if (!bf || !bf->o) {
 		return false;
 	}
-	arch->o->bin_obj = load_bytes (arch, bytes, sz, arch->o->loadaddr, arch->sdb);
-	return arch->o->bin_obj != NULL;
+	bf->o->bin_obj = load_bytes (bf, bytes, sz, bf->o->loadaddr, bf->sdb);
+	return bf->o->bin_obj != NULL;
 }
 
-static int destroy(RBinFile *arch) {
-	r_bin_wasm_destroy (arch);
+static int destroy(RBinFile *bf) {
+	r_bin_wasm_destroy (bf);
 	return true;
 }
 
-static ut64 baddr(RBinFile *arch) {
+static ut64 baddr(RBinFile *bf) {
 	return 0;
 }
 
-static RBinAddr *binsym(RBinFile *arch, int type) {
+static RBinAddr *binsym(RBinFile *bf, int type) {
 	return NULL; // TODO
 }
 
-static RList *sections(RBinFile *arch);
+static RList *sections(RBinFile *bf);
 
-static RList *entries(RBinFile *arch) {
-	RBinWasmObj *bin = arch && arch->o ? arch->o->bin_obj : NULL;
+static RList *entries(RBinFile *bf) {
+	RBinWasmObj *bin = bf && bf->o ? bf->o->bin_obj : NULL;
 	// TODO
 	RList *ret;
 	RBinAddr *ptr = NULL;
@@ -70,8 +70,8 @@ static RList *entries(RBinFile *arch) {
 	return ret;
 }
 
-static RList *sections(RBinFile *arch) {
-	RBinWasmObj *bin = arch && arch->o ? arch->o->bin_obj : NULL;
+static RList *sections(RBinFile *bf) {
+	RBinWasmObj *bin = bf && bf->o ? bf->o->bin_obj : NULL;
 	RList *ret = NULL;
 	RList *secs = NULL;
 	RBinSection *ptr = NULL;
@@ -87,7 +87,9 @@ static RList *sections(RBinFile *arch) {
 	RListIter *iter;
 	r_list_foreach (secs, iter, sec) {
 		if (!(ptr = R_NEW0 (RBinSection))) {
-			break;
+			r_list_free (secs);
+			r_list_free (ret);
+			return NULL;
 		}
 		strncpy (ptr->name, (char*)sec->name, R_BIN_SIZEOF_STRINGS);
 		if (sec->id == R_BIN_WASM_SECTION_DATA || sec->id == R_BIN_WASM_SECTION_MEMORY) {
@@ -105,37 +107,33 @@ static RList *sections(RBinFile *arch) {
 	return ret;
 }
 
-static RList *symbols(RBinFile *arch) {
-	RBinWasmObj *bin;
-	RList *ret, *codes, *imports;
+static RList *symbols(RBinFile *bf) {
+	RBinWasmObj *bin = NULL;
+	RList *ret = NULL, *codes = NULL, *imports = NULL;
 	RBinSymbol *ptr = NULL;
 
-	if (!arch || !arch->o || !arch->o->bin_obj) {
+	if (!bf || !bf->o || !bf->o->bin_obj) {
 		return NULL;
 	}
-	bin = arch->o->bin_obj;
+	bin = bf->o->bin_obj;
 	if (!(ret = r_list_newf ((RListFree)free))) {
 		return NULL;
 	}
 	if (!(codes = r_bin_wasm_get_codes (bin))) {
-		free (ret);
-		return NULL;
+		goto bad_alloc;
 	}
 	if (!(imports = r_bin_wasm_get_imports (bin))) {
-		free (ret);
-		return NULL;
+		goto bad_alloc;
 	}
-	
+
 	ut32 i = 0;
 	RBinWasmImportEntry *imp;
 	RListIter *iter;
 	r_list_foreach (imports, iter, imp) {
 		if (!(ptr = R_NEW0 (RBinSymbol))) {
-			break;
+			goto bad_alloc;
 		}
-		char tmp[R_BIN_SIZEOF_STRINGS];
-		snprintf (tmp, R_BIN_SIZEOF_STRINGS, "imp.%s.%s", imp->module_str, imp->field_str);
-		ptr->name = strdup(tmp);
+		ptr->name = r_str_newf ("imp.%s.%s", imp->module_str, imp->field_str);
 		ptr->forwarder = r_str_const ("NONE");
 		ptr->bind = r_str_const ("NONE");
 		switch (imp->kind) {
@@ -145,17 +143,17 @@ static RList *symbols(RBinFile *arch) {
 		case 3: ptr->type = r_str_const ("GLOBAL"); break;
 		}
 		ptr->size = 0;
-		ptr->vaddr = 0;
-		ptr->paddr = 0;
+		ptr->vaddr = -1;
+		ptr->paddr = -1;
 		ptr->ordinal = i;
 		i += 1;
 		r_list_append (ret, ptr);
 	}
-	
+
 	RBinWasmCodeEntry *func;
 	r_list_foreach (codes, iter, func) {
 		if (!(ptr = R_NEW0 (RBinSymbol))) {
-			break;
+			goto bad_alloc;
 		}
 		char tmp[R_BIN_SIZEOF_STRINGS];
 		snprintf (tmp, R_BIN_SIZEOF_STRINGS, "fnc.%d", i);
@@ -167,31 +165,35 @@ static RList *symbols(RBinFile *arch) {
 		ptr->vaddr = (ut64)func->code;
 		ptr->paddr = (ut64)func->code;
 		ptr->ordinal = i;
-		i += 1;
+		i++;
 		r_list_append (ret, ptr);
 	}
 
 	// TODO: use custom section "name" if present
 	// TODO: exports, globals, tables and memories
 	return ret;
+bad_alloc:
+	// not so sure if imports should be freed.
+	r_list_free (codes);
+	r_list_free (ret);
+	return NULL;
 }
 
-static RList *imports(RBinFile *arch) {
+static RList *imports(RBinFile *bf) {
 	RBinWasmObj *bin = NULL;
 	RList *imports = NULL;
 	RBinImport *ptr = NULL;
 	RList *ret = NULL;
 
-	if (!arch || !arch->o || !arch->o->bin_obj) {
+	if (!bf || !bf->o || !bf->o->bin_obj) {
 		return NULL;
 	}
-	bin = arch->o->bin_obj;
+	bin = bf->o->bin_obj;
 	if (!(ret = r_list_newf (r_bin_import_free))) {
 		return NULL;
 	}
 	if (!(imports = r_bin_wasm_get_imports (bin))) {
-		r_list_free (ret);
-		return NULL;
+		goto bad_alloc;
 	}
 
 	RBinWasmImportEntry *import = NULL;
@@ -199,7 +201,7 @@ static RList *imports(RBinFile *arch) {
 	RListIter *iter;
 	r_list_foreach (imports, iter, import) {
 		if (!(ptr = R_NEW0 (RBinImport))) {
-			break;
+			goto bad_alloc;
 		}
 		ptr->name = strdup (import->field_str);
 		ptr->classname = strdup (import->module_str);
@@ -222,19 +224,23 @@ static RList *imports(RBinFile *arch) {
 		r_list_append (ret, ptr);
 	}
 	return ret;
-}
-
-static RList *libs(RBinFile *arch) {
+bad_alloc:
+	r_list_free (imports);
+	r_list_free (ret);
 	return NULL;
 }
 
-static RBinInfo *info(RBinFile *arch) {
+static RList *libs(RBinFile *bf) {
+	return NULL;
+}
+
+static RBinInfo *info(RBinFile *bf) {
 	RBinInfo *ret = NULL;
 
 	if (!(ret = R_NEW0 (RBinInfo))) {
 		return NULL;
 	}
-	ret->file = strdup (arch->file);
+	ret->file = strdup (bf->file);
 	ret->bclass = strdup ("module");
 	ret->rclass = strdup ("wasm");
 	ret->os = strdup ("Wasm");
@@ -243,21 +249,20 @@ static RBinInfo *info(RBinFile *arch) {
 	ret->subsystem = strdup ("wasm");
 	ret->type = strdup ("EXEC");
 	ret->bits = 32;
-	ret->has_va = true;
+	ret->has_va = 0;
 	ret->big_endian = false;
 	ret->dbg_info = 0;
-	ret->dbg_info = 0;
 	return ret;
 }
 
-static ut64 size(RBinFile *arch) {
-	if (!arch->o->info) {
-		arch->o->info = info (arch);
+static ut64 size(RBinFile *bf) {
+	if (!bf->o->info) {
+		bf->o->info = info (bf);
 	}
-	if (!arch->o->info) {
+	if (!bf->o->info) {
 		return 0;
 	}
-	return arch->buf->length;
+	return bf->buf->length;
 }
 
 /* inspired in http://www.phreedom.org/solar/code/tinype/tiny.97/tiny.asm */
