@@ -9,11 +9,11 @@ static bool MACH0_(write_addlib)(struct MACH0_(obj_t) *obj, const char *lib) {
 	return false;
 }
 
-static bool addlib(RBinFile *arch, const char *lib) {
-	struct MACH0_(obj_t) *obj = arch->o->bin_obj;
-	bool ret = MACH0_(write_addlib) (arch->o->bin_obj, lib);
-	r_buf_free (arch->buf);
-	arch->buf = obj->b;
+static bool addlib(RBinFile *bf, const char *lib) {
+	struct MACH0_(obj_t) *obj = bf->o->bin_obj;
+	bool ret = MACH0_(write_addlib) (bf->o->bin_obj, lib);
+	r_buf_free (bf->buf);
+	bf->buf = obj->b;
 	obj->b = NULL;
 	return ret;
 }
