@@ -1,7 +1,9 @@
 #ifndef PDB_DOWNLOADER_H
 #define PDB_DOWNLOADER_H
+
 #include <r_types.h>
 #include <r_core.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -9,7 +11,7 @@ extern "C" {
 typedef struct SPDBOptions {
 	char *user_agent;
 	char *symbol_server;
-	int extract;
+	ut64 extract;
 } SPDBOptions;
 
 typedef struct SPDBDownloaderOpt {
@@ -18,7 +20,7 @@ typedef struct SPDBDownloaderOpt {
 	char *dbg_file;
 	char *guid;
 	char *path;
-	int extract;
+	ut64 extract;
 } SPDBDownloaderOpt;
 
 typedef struct SPDBDownloader {
@@ -42,7 +44,7 @@ void deinit_pdb_downloader(SPDBDownloader *pdb_downloader);
 
 ///
 /// \brief download PDB file
-int r_bin_pdb_download (RCore* core, int isradjson, int* actions_done, SPDBOptions* options);
+R_API int r_bin_pdb_download(RCore *core, int isradjson, int *actions_done, SPDBOptions *options);
 
 #ifdef __cplusplus
 }
