@@ -42,7 +42,7 @@ R_API bool r_bp_restore_except(RBreakpoint *bp, bool set, ut64 addr) {
 		if (addr && b->addr == addr) {
 			continue;
 		}
-		if (bp->breakpoint && bp->breakpoint (b, set, bp->user)) {
+		if (bp->breakpoint && bp->breakpoint (bp, b, set)) {
 			continue;
 		}
 
