@@ -5,9 +5,9 @@
 
 static struct r_bp_arch_t r_bp_plugin_ppc_bps[] = {
 	/* XXX: FIX those are not really breakpoint opcodes at all */
-	{ 4, 0, (const ut8*)"\x00\x00\x00\x0d" }, // little endian
-	{ 4, 1, (const ut8*)"\x0d\x00\x00\x00" }, // big endian
-	{ 0, 0, NULL }
+	{ 32, 4, 0, (const ut8*)"\x00\x00\x00\x0d" }, // little endian
+	{ 32, 4, 1, (const ut8*)"\x0d\x00\x00\x00" }, // big endian
+	{ 0, 0, 0, NULL }
 };
 
 struct r_bp_plugin_t r_bp_plugin_ppc = {
@@ -18,8 +18,9 @@ struct r_bp_plugin_t r_bp_plugin_ppc = {
 };
 
 #ifndef CORELIB
-struct r_lib_struct_t radare_plugin = {
+RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BP,
 	.data = &r_bp_plugin_ppc,
+	.version = R2_VERSION
 };
 #endif
