@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2008-2017 - pancake */
+/* radare2 - LGPL - Copyright 2008-2017 - pancake, Jody Frankowski */
 
 #include <r_cons.h>
 #include <r_print.h>
@@ -308,6 +308,7 @@ R_API RCons *r_cons_new() {
 	if (I.refcnt != 1) {
 		return &I;
 	}
+	I.rgbstr = r_cons_rgb_str_off;
 	I.line = r_line_new ();
 	I.highlight = NULL;
 	I.event_interrupt = NULL;
@@ -354,7 +355,7 @@ R_API RCons *r_cons_new() {
 	signal (SIGWINCH, resize);
 #elif __WINDOWS__
 	h = GetStdHandle (STD_INPUT_HANDLE);
-	GetConsoleMode (h, (PDWORD) &I.term_buf);
+	GetConsoleMode (h, &I.term_buf);
 	I.term_raw = 0;
 	if (!SetConsoleCtrlHandler ((PHANDLER_ROUTINE)__w32_control, TRUE)) {
 		eprintf ("r_cons: Cannot set control console handler\n");
@@ -506,8 +507,8 @@ R_API void r_cons_reset() {
 	I.grep.line = -1;
 	I.grep.sort = -1;
 	I.grep.sort_invert = false;
-	I.grep.str = NULL;
-	memset (I.grep.tokens, 0, R_CONS_GREP_TOKENS);
+	R_FREE (I.grep.str);
+	ZERO_FILL (I.grep.tokens);
 	I.grep.tokens_used = 0;
 }
 
@@ -523,6 +524,16 @@ R_API void r_cons_filter() {
 		I.filter = false;
 	}
 	/* html */
+	if (I.is_html) {
+		int newlen = 0;
+		char *input = r_str_ndup (I.buffer, I.buffer_len);
+		char *res = r_cons_html_filter (input, &newlen);
+		free (I.buffer);
+		free (input);
+		I.buffer = res;
+		I.buffer_len = newlen;
+		I.buffer_sz = newlen;
+	}
 	/* TODO */
 }
 
@@ -573,8 +584,9 @@ R_API void r_cons_pop() {
 		if (data->grep) {
 			memcpy (&I.grep, data->grep, sizeof (RConsGrep));
 			if (data->grep->str) {
-				free (I.grep.str);
-				I.grep.str = data->grep->str;
+				char *old = I.grep.str;
+				I.grep.str = strdup (data->grep->str);
+				R_FREE (old);
 			}
 		}
 		cons_stack_free ((void *)data);
@@ -620,7 +632,7 @@ R_API void r_cons_flush() {
 			}
 #endif
 			// fix | more | less problem
-			r_cons_set_raw (1);
+			r_cons_set_raw (true);
 		}
 	}
 	if (tee && *tee) {
@@ -636,35 +648,31 @@ R_API void r_cons_flush() {
 	}
 	r_cons_highlight (I.highlight);
 	// is_html must be a filter, not a write endpoint
-	if (I.is_html) {
-		r_cons_html_print (I.buffer);
-	} else {
-		if (I.is_interactive && !r_sandbox_enable (false)) {
-			if (I.linesleep > 0 && I.linesleep < 1000) {
-				int i = 0;
-				int pagesize = R_MAX (1, I.pagesize);
-				char *ptr = I.buffer;
-				char *nl = strchr (ptr, '\n');
-				int len = I.buffer_len;
-				I.buffer[I.buffer_len] = 0;
-				r_cons_break_push (NULL, NULL);
-				while (nl && !r_cons_is_breaked ()) {
-					r_cons_write (ptr, nl - ptr + 1);
-					if (!(i % pagesize)) {
-						r_sys_usleep (I.linesleep * 1000);
-					}
-					ptr = nl + 1;
-					nl = strchr (ptr, '\n');
-					i++;
+	if (I.is_interactive && !r_sandbox_enable (false)) {
+		if (I.linesleep > 0 && I.linesleep < 1000) {
+			int i = 0;
+			int pagesize = R_MAX (1, I.pagesize);
+			char *ptr = I.buffer;
+			char *nl = strchr (ptr, '\n');
+			int len = I.buffer_len;
+			I.buffer[I.buffer_len] = 0;
+			r_cons_break_push (NULL, NULL);
+			while (nl && !r_cons_is_breaked ()) {
+				r_cons_write (ptr, nl - ptr + 1);
+				if (!(i % pagesize)) {
+					r_sys_usleep (I.linesleep * 1000);
 				}
-				r_cons_write (ptr, I.buffer + len - ptr);
-				r_cons_break_pop ();
-			} else {
-				r_cons_write (I.buffer, I.buffer_len);
+				ptr = nl + 1;
+				nl = strchr (ptr, '\n');
+				i++;
 			}
+			r_cons_write (ptr, I.buffer + len - ptr);
+			r_cons_break_pop ();
 		} else {
 			r_cons_write (I.buffer, I.buffer_len);
 		}
+	} else {
+		r_cons_write (I.buffer, I.buffer_len);
 	}
 
 	r_cons_reset ();
@@ -698,8 +706,12 @@ R_API void r_cons_visual_flush() {
 		fps = 0;
 		if (prev) {
 			ut64 now = r_sys_now ();
-			ut64 diff = now-prev;
-			fps = (diff<1000000)? (1000000/diff): 0;
+			st64 diff = (st64)(now - prev);
+			if (diff < 0) {
+				fps = 0;
+			} else {
+				fps = (diff < 1000000)? (1000000.0/diff): 0;
+			}
 			prev = now;
 		} else {
 			prev = r_sys_now ();
@@ -712,7 +724,7 @@ static int real_strlen(const char *ptr, int len) {
 	int utf8len = r_str_len_utf8 (ptr);
 	int ansilen = r_str_ansi_len (ptr);
 	int diff = len - utf8len;
-	if (diff) {
+	if (diff > 0) {
 		diff--;
 	}
 	return ansilen - diff;
@@ -1066,7 +1078,7 @@ R_API void r_cons_show_cursor (int cursor) {
  *
  */
 static int oldraw = -1;
-R_API void r_cons_set_raw(int is_raw) {
+R_API void r_cons_set_raw(bool is_raw) {
 	if (oldraw != -1) {
 		if (is_raw == oldraw) {
 			return;
@@ -1084,9 +1096,9 @@ R_API void r_cons_set_raw(int is_raw) {
 	}
 #elif __WINDOWS__
 	if (is_raw) {
-		SetConsoleMode (h, (DWORD)I.term_raw);
+		SetConsoleMode (h, I.term_raw);
 	} else {
-		SetConsoleMode (h, (DWORD)I.term_buf);
+		SetConsoleMode (h, I.term_buf);
 	}
 #else
 #warning No raw console supported for this platform
@@ -1154,7 +1166,7 @@ R_API void r_cons_column(int c) {
 
 static int lasti = 0; /* last interactive mode */
 
-R_API void r_cons_set_interactive(int x) {
+R_API void r_cons_set_interactive(bool x) {
 	lasti = r_cons_singleton ()->is_interactive;
 	r_cons_singleton ()->is_interactive = x;
 }
@@ -1299,6 +1311,10 @@ R_API const char* r_cons_get_rune(const ut8 ch) {
 		case RUNECODE_CORNER_TR:  return RUNE_CORNER_TR;
 		case RUNECODE_CORNER_BR:  return RUNE_CORNER_BR;
 		case RUNECODE_CORNER_BL:  return RUNE_CORNER_BL;
+		case RUNECODE_CURVE_CORNER_TL:  return RUNE_CURVE_CORNER_TL;
+		case RUNECODE_CURVE_CORNER_TR:  return RUNE_CURVE_CORNER_TR;
+		case RUNECODE_CURVE_CORNER_BR:  return RUNE_CURVE_CORNER_BR;
+		case RUNECODE_CURVE_CORNER_BL:  return RUNE_CURVE_CORNER_BL;
 		}
 	}
 	return NULL;
@@ -1314,3 +1330,40 @@ R_API void r_cons_breakword(const char *s) {
 		I.break_word_len = 0;
 	}
 }
+
+/* Prints a coloured help message.
+ * help should be an array of the following form:
+ * {"command", "args", "description",
+ * "command2", "args2", "description"}; */
+R_API void r_cons_cmd_help(const char *help[], bool use_color) {
+	RCons *cons = r_cons_singleton ();
+	const char *pal_args_color = use_color ? cons->pal.args : "",
+			*pal_help_color = use_color ? cons->pal.help : "",
+			*pal_reset = use_color ? cons->pal.reset : "";
+	int i, max_length = 0;
+
+	for (i = 0; help[i]; i += 3) {
+		int len0 = strlen (help[i]);
+		int len1 = strlen (help[i + 1]);
+		if (i) {
+			max_length = R_MAX (max_length, len0 + len1);
+		}
+	}
+
+	for (i = 0; help[i]; i += 3) {
+		if (i) {
+			int padding = max_length - (strlen (help[i]) + strlen (help[i + 1]));
+			r_cons_printf("| %s%s%s%*s  %s%s%s\n",
+					help[i],
+					pal_args_color, help[i + 1],
+					padding, "",
+					pal_help_color, help[i + 2], pal_reset);
+		} else {
+			// no need to indent the first line
+			r_cons_printf ("|%s%s %s%s%s\n",
+					pal_help_color,
+					help[i], help[i + 1], help[i + 2],
+					pal_reset);
+		}
+	}
+}
